
// ****************************************************************************
//
// Routine:      acReadGlobalByTime
// Author:       T.Oest
//
// Description:  read global alignment constants from database
//                using the evnt time
//
// Modification:
// 16 AUg 2000 (AG) corrected include path bug
// 14 Dec 2000 (DG) take EventUnixTime from tahd.time for TAXX mode
//
// ****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>
#include <limits.h>

// ONLINE includes
#include "table.h"
#include "esp.h"

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

// globals
static int first       = 0;
static int nextMajor   = 0;
static int actualMajor = 0;
static int maxMajor    = 0;
static int minMajor    = 0;

// externals
extern int Db_dbr_last_major, Db_dbr_last_minor, Db_dbr_empty;

// prototypes
void fillACGA();
void ackuexec( char * KuipCommand );

//*****************************************************************
// function
//*****************************************************************

int 
acReadGlobalByTime( int KTstatus, char * dbserver )
{
  ARTE_mark( "acReadGlobalByTime", 0 );

  int ACGAupdate = 0;

  //==================================================================
  // event UNIX time
  //==================================================================
  
  int EventUnixTime = 0;
  
  if (ArteTable<DDAQ>::size() > 0){ 
    ArtePointer<DDAQ> ddaq( 2 );
    
    EVT_DATA *p = (EVT_DATA *) &ddaq->bits;
    ESP_get_int( &p->event_time, &EventUnixTime, 1 );
  } else {
    // Temporary workaround, maybe we should fill EVT_DATA (?)
    if ( ArteTable<TAHD>::size() )
	EventUnixTime = ArtePointer<TAHD>(1)->time;
  }

  if (EventUnixTime == 0) return -1;

//==================================================================
// Initialisation: get first tabel
//==================================================================

  char LunName[5] = {'D','B','G','A',0};
  char * KuipCommand =  (char*) malloc(max(128,
                                       int(24+strlen(dbserver))));

  if (first == 0){
   first = 1;  

   ArteTable<ACGA>::clear();
   sprintf (KuipCommand,"dbopen %s ro",LunName);
   ackuexec(KuipCommand);
   sprintf (KuipCommand,"dbread %s ACGA 0 0 gt",LunName);
   ackuexec(KuipCommand);

   if (ArteTable<ACGA>::size() != 0){
    nextMajor = Db_dbr_last_major;  } else {
    nextMajor = 0;
   };
    minMajor = nextMajor;

   ArteTable<ACGA>::clear();
   sprintf (KuipCommand,"dbread %s ACGA %i 0 le",LunName,INT_MAX);
   ackuexec(KuipCommand);

   if (ArteTable<ACGA>::size() != 0){
    maxMajor = Db_dbr_last_major;   } else {
    maxMajor = 0;
   };

   ArteTable<ACGA>::clear();
   sprintf (KuipCommand,"dbclose %s",LunName);
   ackuexec(KuipCommand);


  };
   

//==================================================================
// read in new ACGA table if table not existing or update necessay
//==================================================================

  if (EventUnixTime < minMajor) {
   ArteTable<ACGA>::clear();
   return -2;
  };

  if ( ArteTable<ACGA>::size() == 0 || 
      (EventUnixTime >= nextMajor  && actualMajor != maxMajor  ) || 
       EventUnixTime < actualMajor){

   int major;

   major = EventUnixTime + 1;

   sprintf (KuipCommand,"dbopen %s ro",LunName);
   ackuexec(KuipCommand);

   if (EventUnixTime < maxMajor){
    ArteTable<ACGA>::clear();
    sprintf (KuipCommand,"dbread %s ACGA %i 0 ge",LunName,major);
    ackuexec(KuipCommand);

    if (ArteTable<ACGA>::size() != 0){
     nextMajor = Db_dbr_last_major;  } else {
     nextMajor = 0;
    };
   } else {
    nextMajor = maxMajor;
   };

   ArteTable<ACGA>::clear();
   sprintf (KuipCommand,"dbread %s ACGA %i 0 lt",LunName,major);
   ackuexec(KuipCommand);

   actualMajor = Db_dbr_last_major;

   sprintf (KuipCommand,"dbclose %s",LunName);
   ackuexec(KuipCommand);
   
   ACGAupdate = 1;

  };

//==================================================================
// check if ACGA table exist
//==================================================================

 if (ArteTable<ACGA>::size() == 0)  ACGAupdate = -3;


 return ACGAupdate;
}
