// ==================================================================
// 
// acCnAkey.C
//
// started from KUIP 
//
// Purpose: read run number from kumac and retrieve from DB the last
//          CnA key used for that run. To be used in MC to simulate
//          the trigger and reconstruction conditions of a given run. 
//
// Usage: cnakey [return]
//                            RUN2CNAKEY  (run number) 
//
// Returns: nothing, sets global variable CnAkey.
//
// Files: CDF file alca/ackuip.cdf for parameter definitions
//
// Author: Jose Hernandez
// Modifications:
// Added printout in case no CnA key found for given run Nr 13.11.01 (JH)  
// ==================================================================

#include <strings.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <iostream.h>
#include "arte/arte.hh"
#include "database/table.h"
#include "farm/farm.hh"


int get_CnAKey(int runnumber);
extern "C" int ku_geti();

int CnAkey=0;  // global. Used in acread_
int RunNoToSimu=0; // global, needed in various place of ARTE

extern "C" void accnakey_() {
  int runnr=0;

  runnr = ku_geti();
  RunNoToSimu = runnr;
  cout << "accnakey_/RunNoToSimu = " << RunNoToSimu << endl;
  if (runnr>0) CnAkey = get_CnAKey(runnr);
  else         CnAkey = -runnr;
  if (!CnAkey) {
    char msg[256];
    sprintf(msg, "No CnA key found for RUN2CNAKEY %d", runnr);
    ARTE_exit(GRR_SFATAL, msg, 0);
  }
}



int get_CnAKey(int runnumber) {
    Tables database;
    int rc, tx, n_max = 255, n_used, i;
    char **names, **values;
    char mess[256];

      database = OpenTables("/RUN_LOG", 0);
      if (!database){
        ARTE_error(GRR_SERROR, "Failed to open /RUN_LOG database",0);
      }
      tx = BookTable(database,"run_config", 0, 0);
      if (!tx){
        ARTE_error(GRR_SERROR, "Failed to book run_config table",0);
      }
      TableStamp(database, runnumber, TABLEmaxrev);
      rc = FetchTable(database, tx, TABLEle);
      if (!rc){
        sprintf(mess,
         "Failed to find run_config table with major:%d and minor:TABLEmaxrev",
         runnumber);
        ARTE_error(GRR_SERROR, mess, 0);
      }
      SkipTableField(database,1);
      TableFieldSize(database);
      names  = (char **) malloc(n_max*sizeof(char *)); names[0]  = NULL;
      values = (char **) malloc(n_max*sizeof(char *)); values[0] = NULL;
      n_used = GetTableField(database, TABLEstring, n_max, names);
      GetTableField(database, TABLEstring, n_max, values);
      for (i=0 ; i<(n_used<n_max?n_used:n_max) ; i++){
        if (names[i]) {
          //printf("%d: %-25s %s\n", i, names[i], values[i]);
          if (!strcmp(names[i],"CNAkey")) {
            return atoi(values[i]);
          }
        }
      }
      sprintf(mess,
         "Failed to find CNAkey for runnumber %d among first %d entries",
         runnumber, n_max );
      ARTE_error(GRR_SERROR, mess, 0);
      return 0;
}
 
