//                              -*- Mode: C++ -*-
//:CmdCareMerger.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 26/5/97
//
// 20.02.01 test added
//

#include "care/CmdCareMerger.hh"
#include "care/caredata.hh"
#include "arte/ArteMessage.hh"

#ifdef USEKUIP

extern "C" {
void caremerger_(){
  CmdCareMerger careMerg;
  careMerg.readAndExe();

}
}; // end of extern
#endif

CmdCareMerger::CmdCareMerger() : minimum(1.0), valley(0.2) {}
CmdCareMerger::~CmdCareMerger(){}

void CmdCareMerger::defValues(){
  minimum = 1.0;
  valley = 0.2;
}

//--
//   Execute stored command
void CmdCareMerger::execute(){
  ECalInfo.clusThr = minimum;
  ECalInfo.valley = valley;
  if( minimum>0.5 ){
    ECalInfo.thresholdmode = 1;
  } else {
    ECalInfo.thresholdmode = 2;
  }
}

#ifdef USEKUIP
//--
//   Read a command from external
void CmdCareMerger::read(){

  minimum = readFloat();
  valley  = readFloat();
  if(minimum<0.001 )
    defValues();
}
#endif
                          
//--
//   Printout Information for the current lambda info stored
void CmdCareMerger::dump(){
  arteMsg<<" Merging Cut Off constants:";
  arteMsg.allp();
  arteMsg<<" Minimum energy: "<<ECalInfo.clusThr<<" GeV "
	 <<" Valley energy: "<<ECalInfo.valley<<" GeV";
  arteMsg.allp();
}


//--
//   Set Internal data
void CmdCareMerger::setMerger( Energy_t  min, Energy_t val){
  minimum = min;
  valley = val;
  execute();
}

