//                              -*- Mode: C++ -*-
//: ECStrategyFactory.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#ifndef ECSTRATEGYFACTORY_HH_
#define ECSTRATEGYFACTORY_HH
#include "care/ESPosition.hh"
#include "care/ESPosStrategy.hh"
#include "care/ESCTCDisStrat.hh"
#include "care/ESPIDStrategy.hh"
#include "care/ESEnCorStrategy.hh"

/**
class ECStrategyFactory.

*/
//--
//  Types of positions that have different time and resolution performances
//enum PosTyp {Peak, COG, E_CorCOG, P_CorCOG }; 

class ECStrategyFactory {
public:
 
  //--
  ///  Type of possible definitions of the Cluster-to-Cluster distance
  enum ClToClDis {Naive,Valley,ValAndDis};
  //--
  ///  Types of possible definitions of the Cluster-to-Track distance
  enum ClToTkDis {PeakDis,SecPeak,Recover};
  //--
  ///  Types of possible definitions of the shower shape analysis
  enum PIDtypes {Pid04, Pid05, Pid1, Pid3Cell};
  //--
  ///  Types of possible definitions of the energy correction analysis
  enum EnCortypes {LightYield, DeadCells};

  //--
  /// public structure that is used to store type datas
  struct  StratTypes {
    PosTyp     ptype;
    ClToClDis   ctctype;
    ClToTkDis   ctttype;
    PIDtypes    pidtype;
  };

  /// Constructors
  ECStrategyFactory();
  //-- Destructor
  /// Destructor: currently void
  ~ECStrategyFactory();
  /**@name Beheavioral methods */
  //@{
  //-- Behavioural operations
  ///   Reset stored data to default
  inline void reset();
  //--
  ///   Get a brand new strategy for position
  ESPosStrategy* PositionStrategy() const;
  //--
  ///   Get a new PositionStrategy of type pos
  ESPosStrategy* PositionStrategy(PosTyp pos) const;
  //--
  ///   Get a brand new strategy for Clus-To-Clus dist
  ESCTCDisStrat* CTCStrategy() const;
  //--
  ///   Get a new strategy for Clus-To-Clus dist of type ctcs
  ESCTCDisStrat* CTCStrategy(ClToClDis ctcs) const;
  //--
  ///   Get a new strategy for PID analysis
  ESPIDStrategy* PidStrategy() const;
  //--
  ///   Get a new strategy for PID analysis
  ESPIDStrategy* PidStrategy(PIDtypes pidt) const;
  //--
  ///   Get a new strategy for Energy correction analysis
  ESEnCorStrategy* EnCorStrategy(EnCortypes enct) const;

  //@}

  /**@name Set/Access methods */
  //@{
  //--
  ///   Inquire for the data stored
  inline const StratTypes & getData() const;
  //--
  ///  PosType get
  inline PosTyp getPosType() const;
  //--
  ///  PosType set
  inline void setPosType(PosTyp pos);
  //--
  ///  CTCType get
  inline  ClToClDis getCTCType() const;
  //--
  ///  CTCType set
  inline void setCTCType(ClToClDis ctcs);

  //--
  ///  Set the defaults
  void setDefaults(struct StratTypes & newdef);
  //@}
  //
private:
  // // ///////////////// Data Part /////////////////////////////////
  //--
  /// here the definition of what strategy we are using now
  static StratTypes currdata; 
  //--
  /// static data to be used to store the defaul values (off-line)
  static StratTypes defValues; 
};


#include "care/ECStrategyFactory.icc"

#endif  //   ECSTRATEGYFACTORY_HH_
