//                              -*- Mode: C++ -*-
//: ESEMSPOS.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 14/6/98
// Last Update ........
//
// 20.02.01 cog on reduced cluster stored
//

#ifndef ESEMSPOS_HH_
#define ESEMSPOS_HH_

#include "care/ESCorCOGPos.hh"
#include "care/ELNearCells.hh"

/**
class ESEMSPOS.
  It finds better extimations of the starting point of an
  electromagnetic shower.
*/

class ESEMSPos :  public ESCorCOGPos {
public:

  /// Constructor
  ESEMSPos();
  /// Destructor: currently void but needs to be virtual
  virtual ~ESEMSPos();
 
  /**@name Beheavioral methods */
  //@{ 
  ///   Evaluate the position for a cluster.
  virtual void evaluatePos(const EMCluster* clus);

  /// EMClusterMember Analyzer
  inline 
  virtual void doAnalysis(const EMClusterMember * clsMem);
  /// EMHitCell Analyzer : redefined!
  virtual void doAnalysis(const EMHitCell * hitT);
  //--
  /// EMCellFraction Analyzer  
  inline 
  virtual void doAnalysis(const EMCellFraction * hitT);
  /// Do a Cluster analysis
  inline
  virtual void doAnalysis(const EMCluster* clus);
  //--
  /// Do a ClusterTree analysis
  inline
  virtual void doAnalysis(const EMClusterTree* clus);
  //--
  /// EMTrackOnEcal analysis
  inline
  virtual void doAnalysis(const EMTrackOnEcal * Track);

  //@}
  inline float getXcog() const {return xcog;};
  inline float getYcog() const {return ycog;};

private:
  // The near cell object
  ELNearCells nearCells;
  float xcog;
  float ycog;
};


/// EMCellFraction Analyzer  
void  ESEMSPos::doAnalysis(const EMCellFraction * hitT){
  doAnalysis((const EMHitCell*) hitT);
};

/// EMClusterMember Analyzer
void  ESEMSPos::doAnalysis(const EMClusterMember * clsMem){
  ESCorCOGPos::doAnalysis(clsMem);
};
/// Do a Cluster analysis
void  ESEMSPos::doAnalysis(const EMCluster * cls){
  ESCorCOGPos::doAnalysis(cls);
};
/// Do a ClusterTree analysis
void  ESEMSPos::doAnalysis(const EMClusterTree * clsT){
  ESCorCOGPos::doAnalysis(clsT);
};
/// EMTrackOnEcal analysis
void  ESEMSPos::doAnalysis(const EMTrackOnEcal * track){
  ESCorCOGPos::doAnalysis(track);
};

#endif // ESEMSPOS_HH_

