//                              -*- Mode: C++ -*-
//: ESLIGHTYIELDCOR.hh
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 14/4/97
// Last Update ........
//

#ifndef ESLIGHTYIELDCOR_HH_
#define ESLIGHTYIELDCOR_HH_

#include "care/ESEnCorStrategy.hh"

/**
class ESLightYieldCor.
  It finds better extimations of the energy of an electromagnetic
  shower applying a calorimeter dependent light yield correction.

*/

class ESLightYieldCor :  public ESEnCorStrategy {
public:

  /// Constructor
  ESLightYieldCor();
  /// Destructor: currently void but needs to be virtual
  virtual ~ESLightYieldCor();
 
  /**@name Beheavioral methods */
  //@{ 
  // to have the possibility to instantiate the class
  virtual void doAnalysis(const class EMClusterMember *);
  virtual void doAnalysis(const class EMHitCell *);
  //--
  /// EMCellFraction Analyzer  
  inline
  virtual void doAnalysis(const EMCellFraction * hitT);
  //--
  /// Do a Cluster analysis
  virtual void doAnalysis(const EMCluster* clus);
  //--
  /// Do a ClusterTree analysis
  inline  
  virtual void doAnalysis(const EMClusterTree* clus);

  //--
  /// EMTrackOnEcal analysis
  inline
  virtual void doAnalysis(const EMTrackOnEcal * Track);


  ///   evaluate the energy correction for a cluster.
  virtual Energy_t evaluateEnergy(const EMCluster* clus, 
				  ESPosition & positionData);
  
  //--
  ///  Filling the static data for corrections (light)
  static void fill(SubCal subCal, unsigned int cortype, 
		   unsigned int num, float* data);
  //--
  ///  Print informations on static data
  static void dump();
  //@}
protected:
  // correction function
  float corr(float r, unsigned int n, float * array) const; 
  float corr(float x, float y, unsigned int n, float * array) const; 
private:
  // 
  enum { ncals=3, maxdegree=10};
  enum lightcorrtype {radial=0, coord=1, radial2 = 2, coord2 =3};
  // static data
  //--
  /// data on spatial uniformity
  static float corrPolin[ncals][maxdegree];
  static unsigned int degree[ncals];
  static lightcorrtype cortype[ncals];
};


//--
// ClusterTree Analyzer
void ESLightYieldCor::doAnalysis(const EMClusterTree *){}
//--
// Track Analyzer
void ESLightYieldCor::doAnalysis(const EMTrackOnEcal *){}

/// EMCellFraction Analyzer  
void ESLightYieldCor::doAnalysis(const EMCellFraction * hitT){
  doAnalysis((const EMHitCell*) hitT);
};



#endif // ESLIGHTYIELDCOR_HH_


