// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   23. Jul 1999
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: ARTE constructor for CustomTrack
//
// changes:
// 23 Jul 1999 creation by C. Borgmeier
// 22 Nov 2000 (TG) added namespace CLUE
// 13 Sep 2001 (TG) changed *_pinv() to *_p()
//
// ********************************************************************

#include "clue/cluearte.hh"         // needed for CustomTrack.hh
#include "clue/CustomTrack.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif

  //==============================================================================
  // VDS track Constructor
  //==============================================================================
  VDS_Track::VDS_Track( const ArtePointer<RSEG>& rseg )
    : RecoSegment( rseg ) {
    set_nf( static_cast<short>(rseg->time + rseg->dtim - 4 + .5 )) ;
  }


  //==============================================================================
  // RICH track Constructor
  //==============================================================================
  RICH_Track::RICH_Track( const ArtePointer<RSEG>& rseg )
    : RecoSegment( rseg ),
      radius( rseg->time ),
      signalPhotons( rseg->ze ),
      backgndPhotons( rseg->ye ),
      fraction( rseg->zf ) {

    set_x(0.);
    set_y(0.);
    set_z(1000.);
    set_cov_tx_p(0.);
    set_cov_ty_p(0.);
    set_cov_p(0.);
    
  }


  //==============================================================================
  // OTR Space Point Constructor
  //==============================================================================
  OTR_SpacePoint::OTR_SpacePoint( const ArtePointer<RSEG>& rseg )
    : RecoSegment( rseg ),
      nHits( rseg->nhit ) {
    // nHits( distance( rseg->rhit.begin(), rseg->rhit.end() ) ) {
  }


  //==============================================================================
  // MUON track Constructor
  //==============================================================================
  MuonTrack::MuonTrack( const ArtePointer<RSEG>& rseg )
    : RecoSegment( rseg ) {}

#ifdef NAMESPACE
} // namespace CLUE
#endif
