#include "arte/arte.hh"

static const float DMAINcut  =  50.;    // maximum allowed impact parameter of a good track
static const float ZMAINav   = -4.85;   // zentral z-position of a candidate main vertex
static const float ZMAINcut  =  0.2;    // allowed window around ZMAINav for a main vertex

//=======================================================================
// return bool if the impact parameter of a track at the closest primary vertex
// is greater than DMAINcut
//=======================================================================
bool Dmain(ArtePointer<RTRA> rtra) {

  float tx   = rtra->txf;
  float ty   = rtra->tyf;
  float x    = rtra->xf;
  float y    = rtra->yf;
  float z    = rtra->zf;
  float dmin = 1. + DMAINcut*DMAINcut;

  for(ArteTable<RVER>::iterator itrver = ArteTable<RVER>::begin();
      itrver != ArteTable<RVER>::end(); ++itrver) {
    if(itrver->type!=1 || fabs(itrver->zv - ZMAINav)>ZMAINcut) continue;
    float dx     = itrver->xv - x;
    float dy     = itrver->yv - y;
    float dz     = itrver->zv - z;
    float lambda = (tx*dx + ty*dy + dz)/(tx*tx + ty*ty + 1.);
    float xmin   = lambda*tx - dx;
    float ymin   = lambda*ty - dy;
    float zmin   = lambda    - dz;
    float dsqr   = xmin*xmin + ymin*ymin + zmin*zmin;
    if(dsqr<dmin) dmin = dsqr;
  }

  return sqrt(dmin) > DMAINcut;
}
