// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   08. Jun 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: ARTE constructor for EcalHit
//
// changes:
// 08 Jun 2000 creation
// 22 Nov 2000 (TG) added namespace CLUE
// 12 Jan 2001 (TG) changed constructor
// 02 Apr 2001 (TG) filling of Tx, Ty, Wid added
// 05 Apr 2001 (TG) removed Tx, Ty
// 11 Jul 2001 (TG) added rtrack initialization, added filling of valid_, Cov
// 16 Jul 2001 (TG) replaced Cov by Eerr
// 22 Jan 2002 (TG) added Xerr, Yerr
//
// ********************************************************************
#include "clue/cluearte.hh"
#include "clue/EcalHit.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif

  //==============================================================================
  // Constructor
  //==============================================================================
  EcalHit::EcalHit( const ArtePointer<RCCL>& rccl ) :
    rtrack (),
    X      ( rccl->x ),
    Y      ( rccl->y ),
    Z      ( rccl->z ),
    Esum   ( rccl->esum ),
    Xerr   ( rccl->cve[0] ),
    Yerr   ( rccl->cve[2] ),
    Eerr   ( rccl->cve[14] ),
    Wid    ( static_cast<unsigned char>(rccl->wid) ),
    valid_ ( true ),
    Zorig  ( 0. ),
    X_     ( 0. ),
    Y_     ( 0. ),
    Z_     ( 0. ),
    Tx_    ( 0. ),
    Ty_    ( 0. )
  {
#ifdef XXX
    // Structure of cov. matrix index:
    //
    //   | x  y  tx  ty  E
    // --+------------------
    // x | 0
    // y | 1  2
    // tx| 3  4  5
    // ty| 6  7  8   9
    // E | 10 11 12  13  14
    //
    // copy only those values which are measured
    Cov[0] = rccl->cve[0];  // x-err
    Cov[1] = rccl->cve[2];  // y-err
    Cov[2] = rccl->cve[14]; // E-err
#endif
  }

#ifdef NAMESPACE
} // namespace CLUE
#endif
