// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   23. Jul 1999
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: ARTE constructor for MC_Track
//
// changes:
// 23 Jul 1999 creation by C. Borgmeier
// 22 Nov 2000 (TG) added namespace CLUE
// 06 Dec 2000 (TG) added initialization of ClueRelation container
// 15 Dec 2000 (TG) bug fix in constructor
// 14 Aug 2001 (TG) added endVtx, removed Mass, Mother
// 06 Sep 2001 (TG) added filling of Flag, added initialization of endTrack
// 13 Sep 2001 (TG) added Mother filling, removed mtra->flag check
//
// ********************************************************************

#include "clue/cluearte.hh"
#include "clue/MC_Track.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif

  MC_Track::MC_Track( const ArtePointer<MTRA>& mtra ) :
    rtrack  (),
    endVtx  (),
    endTrack(),
    Px      ( mtra->px ),
    Py      ( mtra->py ),
    Pz      ( mtra->pz ),
    iTopp   ( getIDX<TOPP>( mtra->topp ) ), 
    B_Vertex( getIDX<MVER>( mtra->bver ) ),
    Mother  ( getIDX<MTRA>( mtra->btra ) ),
    Flag    ( mtra->flag )
  { }

#ifdef NAMESPACE
} // namespace CLUE
#endif
