#ifndef __FILLECALHIT_HH
#define __FILLECALHIT_HH
// **********************************************************************
//
// source:
//
// type:      source code
//
// created:   01.10.2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Code to select proper photon candidates
//
// changes:
// 01 Okt 2001 (TG) creation from Michael Schmellings code
//
// **********************************************************************

/** @name Functions needed to select proper photon candidates
 */
//@{
/** Track propagation.

    Return $(x,y)$ of a track transported to z (behind magnet) by calculating
    momentum kick in magnet.

    @author M. Schmelling
 */
//==============================================================================
// fTransport
//==============================================================================
void  fTransport(const ArtePointer<RTRA>& rtra, float z, float &x, float &y);

/** Histo initialization.

    Initialize HBOOK histograms to monitor photon selection code.
    A call to this function should be placed in the ARTE usinit() function.

    @author M. Schmelling
*/
//==============================================================================
// EHIT_init
//==============================================================================
void EHIT_init();

/** Rough RCCL selection.

    A rough selection of RCCL clusters according to their width, esum and
    asymmetry. The RCCL::flag field is set to 0, because it will be used by the
    photon selection code.
    A selected RCCL cluster gets a flag = 1, a rejected one has flag = 0.
    
    @author M. Schmelling
*/
//==============================================================================
// selectRCCL
//==============================================================================
void selectRCCL();

/** Assign clusters to tracks.

    Transport track behind magnet and look for matching ECAL cluster within a
    radius of 3 cm around track impact at $z=1344$.
    Add energy of clusters up and calculate $E/p$. Store $E/p$ value temporarily
    in $mass$ field of track (will be resetted later).
    
    @author M. Schmelling, adopted to BEE by T. Glebe
 */
//==============================================================================
// assignClusters
//==============================================================================
void assignClusters(const ArtePointer<RecoTrack>& rtrack );

/** Identify bremsstrahlung.

    Straight line exprapolation of track to ECAL and search for bremsstrahlungs
    cluster. Store bremsstrahlungs energy in Ebrem field of RecoTrack.
    Function must be called after RecoTrack table is filled.
    
    @author M. Schmelling, adopted to BEE by T. Glebe
 */
//==============================================================================
// findBremsstrahlung
//==============================================================================
void findBremsstrahlung();

/** Fill EcalHit table.

    Fill EcalHit table, set relations between RecoTracks and remaining EcalHit
    objects, and perform a final PID assignment based on $E/p$ information.
    For Electron candidates, perform bremsstrahlung recovery.
    This functions resets the RecoTrack::mass field (which was used for $E/p$
    storage). This function must be called after \texttt{findBremsstrahlung()}.
    
    @author M. Schmelling, adopted to BEE by T. Glebe
 */
//==============================================================================
// fillEcalHitTable
//==============================================================================
void fillEcalHitTable();
//@}
#endif
