#include <map>
#include <vector>
#include "arte/arte.hh"

int trSelect(int SELMASK)
/*==================================================================================
// flag selected clone-free tracks in RTRA
//================================================================================*/
{
  static int MINHITS_VDS  =  6;  // required minimum number of VDS hits
  static int MINHITS_MAIN = 16;  // required minimum number of Main tracker hits
  static int MINHITS_RICH = 10;  // required minimum number of RICH hits
  static int MINHITS_ECAL =  2;  // required minimum number of ECAL hits

  map< float, ArtePointer<RTRA> > mrtra;

  // put all VDS-OTR tracks into a sorted map

  for(ArteTable<RTRA>::iterator itrtra = ArteTable<RTRA>::begin();
      itrtra != ArteTable<RTRA>::end(); ++itrtra) {
    if( (*itrtra)->fit != 0 ) continue;
    int nvds  = (*itrtra)->hsi;
    int nmain = (*itrtra)->hou + (*itrtra)->hin;
    int nca   = (*itrtra)->hca;
    int nri   = (*itrtra)->hri;
    if( nvds  < MINHITS_VDS ) continue;
    if( nmain < MINHITS_MAIN) continue;
    if( nri   < MINHITS_RICH) nri = 0;
    if( nca   < MINHITS_ECAL) nca = 0;
    if( nri ) nri = 2;
    if( nca ) nca = 2;
    float chi2 = (*itrtra)->chi;
    int   ndf  = nvds+nmain+nri+nca-5;
    float fom = float(nmain+nvds+nca+2*nri) - chi2/float(ndf);

    mrtra.insert(pair< float, ArtePointer<RTRA> >(-fom,*itrtra));
  }

  // copy the tracks into a vector, omitting all clones

  vector< ArtePointer<RTRA> > rtra;

  for(map< float, ArtePointer<RTRA> >::iterator it=mrtra.begin();
      it != mrtra.end(); ++it) {
    int OK = 1;
    for(unsigned int i=0; i!=rtra.size(); ++i) {
      if(rtra[i]->csvd >0 && rtra[i]->csvd  == it->second->csvd ) OK=0;
      if(rtra[i]->cmain>0 && rtra[i]->cmain == it->second->cmain) OK=0;
      if(rtra[i]->crich>0 && rtra[i]->cmain == it->second->crich) OK=0;
      if(rtra[i]->cecal>0 && rtra[i]->cmain == it->second->cecal) OK=0;
      if(!OK) break;
    }
    if(OK) {
      rtra.push_back(it->second);
      it->second->cmp |= SELMASK;
    }
  }

  return rtra.size();
}
