// *****************************************************************************
//
// source:    daq_daq_if.C
//
// type:      source code
//
// created:   14 Mar 2001
//
// author:    Lars Soezueer
//            email: soezueer@mail.desy.de
//
// purpose:   functions to deliver the content of component DAQ (i.e. the event header)
//            to the class DAQ_DAQ_IF
//            called by DAQ_MC_IF::buildEvent
//
// changes:   
//
// *****************************************************************************
#include "daqif/daq_daq_if.hh"
#include "arte/arte.hh"

extern "C" {
#include "event.h"
#include "esp.h"
	   }

/***
   * The component DAQ needs to override buildComponent
   * because its record EVT_DATA_RECORD has not the structure
   * of ordinary components
   *
   ***/

unsigned int DAQ_DAQ_IF::buildComponent(char *buf, unsigned int maxSize)
{
  unsigned int nBytes = sizeof(EVT_DATA_RECORD);

  if (nBytes<=maxSize) 
    {
     EVT_DATA_RECORD *evt_data = (EVT_DATA_RECORD*) buf;

     evt_data->hdr.length        = nBytes;
     evt_data->hdr.recid         = RID_NORMAL_SLB0;
     short fltevt = get_fltevt();
     evt_data->hdr.fltevt        = fltevt;
     evt_data->hdr.compid        = COMP_ID_DAQ;
     evt_data->hdr.slbid         = 0;
     evt_data->hdr.nodeid        = 0;

     ArtePointer<EVHD> evhd(1);

     evt_data->data.run_number   = evhd->run;
     evt_data->data.event_time   = evhd->time;
     evt_data->data.event_status = 0;
     evt_data->data.flt_status   = 0;
     evt_data->data.slp_node     = 0;
     evt_data->data.slt_status   = 0;
     evt_data->data.exp_number   = evhd->exp;
     evt_data->data.farm_node    = 0;
     evt_data->data.BX_long      = 0;           /* bits  0 to 31 of the BX tag (at time trigger issued)*/
     evt_data->data.BX_upper     = 0;           /* bits 32 to 47 of the BX tag (at time trigger issued)*/
     evt_data->data.FCS_tcode    = 1;
     evt_data->data.physBX       = evhd->evt & 0x7F;
     evt_data->data.FLT_BX       = evhd->evt & 0x7F;
     evt_data->data.fltevt       = fltevt;
     evt_data->data.fill         = 0;
     evt_data->data.dbKey        = evhd->dbmajor;/* key to tables used to process event      */
     evt_data->data.dbVersion    = evhd->dbminor;/* version of key table used for this event */

     evt_data->tail.recid        = evt_data->hdr.recid;
     evt_data->tail.fltevt       = evt_data->hdr.fltevt;

     ESP_put_slb_hdr(&evt_data->hdr, &evt_data->hdr);

     ESP_put_int ( (int*) &evt_data->data, (int*) &evt_data->data, 9);
     ESP_put_short(&evt_data->data.BX_upper, &evt_data->data.BX_upper, 6);
     ESP_put_int (&evt_data->data.dbKey, &evt_data->data.dbKey, 2);

     ESP_put_slb_tail(&evt_data->tail, &evt_data->tail);
    }
  return  nBytes;
}
