#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"
#include "auto/MEMS.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"


extern DbFileDesc *DbFileDescList;


void mmbook_(int *, int *), mmrupd_(int *), mmdrop_(int *), mmfree_(int *);
void chhtot_(char *, char *, int *);
void iounit_(char *, int *, int);
int  mmrstat_(int *, int *);
int  mmwstat_(int *, int *);
DbTableDesc *DbArteTableDesc(char *);
int DbMatchType(char *opt);

extern int Db_dbr_last_major, Db_dbr_last_minor, Db_dbr_empty;


void DbReadHeader(int lun, int *Mlink, int major, int minor, 
		char *opt, int *ierr)
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   * 12.08.98: Modified by A. Semenov (JINR, Dubna)
   *
   * Description: Read from a MIZZI file a header bank and its subs.
   *
   * Called from: anywhere.
   *
   * External references: dbrtab_
   *
   * Argument in/out description
   *  lun     in     LUN
   *  Mlink   in     bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  opt     in     "eq","le",... - applied to major, minor
   *  ierr    out    operation status
   *              0  OK
   *              1  Mlink not inside link area
   *              2  bank name too long for MIZZI
   *              3  'opt' has incorrect value
   *              4  bank is not the header one
   *              5  bank is not empty
   *              6  bank is not marked for input
   *              7  bank and all subs are empty after reading
   *           >100  100*nlink+(err status from dbrtab_ routine)
   *                 dbrtab_ reported error while reading bank # nlink
   *
   ************************************************************************/

  int n, m, nlink, sub, res, maj, min, tbls_read=0;
  char subopt[4]="EQ";
  char mess[80]="DBREAD: ";
  char Name[DB_MAX_NAME_SIZE+1];
  char Csym[DB_MAX_NAME_SIZE+1];
  int var;

  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;

  nlink = (int)Mlink / 4 - artelink_.larln1;

  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    static int errPtr;
    strcat( mess, "link not inside link area" );
    mecErrorCount( &errPtr, mess );
	*ierr = 1;
    return;
  }

  var = DB_MAX_NAME_SIZE/4;
  chhtot_(MEMO(nlink).name, Name, &var );

  /*
  ** Check bank name length. Note that MIZZI allows only 16 characters names.
  */
  
  if( strlen(Name) > DB_MAX_TABLE_NAME_SIZE ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank name \"%s\" is too long for MIZZI (NameLen <= %d)",
       Name,DB_MAX_TABLE_NAME_SIZE);
    mecErrorCount( &errPtr, mess );
	*ierr = 2;
    return;
  }

  /*
  ** Check 'opt' parameter has allowed value.
  */
  
  if( DbMatchType(opt)==-1 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
    "Unknown option '%s'; should use one of \"eq\", \"le\", \"ge\", \"lt\", \"gt\"",
       opt);
    mecErrorCount( &errPtr, mess );
	*ierr = 3;
    return;
  }


  /* Check if the bank is a header one. */

  if( MEMO(nlink).sup[0] != 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s is not a header one", Name );
    mecErrorCount( &errPtr, mess );
	*ierr = 4;
    return;
  }

  /* 
   * Drop existing tables.
   */

/*  if( *Mlink != 0 ) mmdrop_( Mlink ); */

  var=mmrstat_(&lun, &nlink);
  if(var==1)
  {
	static int errPtr;
    sprintf(mess+strlen(mess),
       "header bank %s is not empty", Name);
    mecWarningCount( &errPtr, mess );
	*ierr = 5;
    return;
  }  
  if(var)
  {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "header bank %s is not marked for input on %s", Name, Csym );
    mecWarningCount( &errPtr, mess );
    *ierr = 6;
	return;
  }


  /* Start reading. */  
	
  	{
      static int errPtr;
      sprintf(mess+8,
         "reading header bank %s on %s", Name, Csym);
      mecWarningCount( &errPtr, mess );
    }
	

  DbReadTable( lun, Mlink, major, minor, opt, &var);
  if(var)
  {
	  *ierr=var+100*nlink;
	  return;
  }
  
  if(!Db_dbr_empty)
	  tbls_read++;
  
  maj=Db_dbr_last_major;
  min=Db_dbr_last_minor;

  for( n = MEMO(nlink).sub1 ; n <= MEMO(nlink).sub2 ; n++ ) {

    sub = MEMS(n).sub;

/*    printf("Reading sub-bank %d named %60.60s\n",n,MEMO(sub).name);*/

    if( sub == nlink ) continue;

    res=mmrstat_( &lun, &sub );
    
    if( res !=  0 ) {
      static int errPtr;
      var = DB_MAX_NAME_SIZE/4;
      chhtot_(MEMO( sub ).name, Name, &var );
      sprintf(mess+8,
         "subbank %s is %s,skipped", Name, 
         	res==1?"not empy":"not marked for input");
      mecWarningCount( &errPtr, mess );
      continue;
    }
	else {
      static int errPtr;
      var = DB_MAX_NAME_SIZE/4;
      chhtot_(MEMO( sub ).name, Name, &var );
      sprintf(mess+8,
         "reading bank %s", Name);
      mecWarningCount( &errPtr, mess );
    }
		

    DbReadTable( lun, (int *)((artelink_.larln1 + sub) * 4),
		       maj, min, subopt, &var);
	if(var)
	{
		*ierr=var+sub*100;
		return;
	}
	
	if(!Db_dbr_empty)
		tbls_read++;

  }
  
    
  /* Load tables holding many-many relations */
  if(!DbLedaCompatibility)
  {
  int mema, nCol, i;
  mema=MEMO(nlink).mema;
  nCol=MEMO(nlink).ocol;
  for(i=0;i<nCol;i++)
  	if(MEMA(mema+i).type==4)
  		{
  		int mmtab;
  		mmtab=MEMA(mema+i).rela;
  		DbReadTable( lun, (int *)((artelink_.larln1 + mmtab ) * 4),
  		                 maj, min, subopt, &var);
  		}
  for( n = MEMO(nlink).sub1 ; n <= MEMO(nlink).sub2 ; n++ ) 
    {
    sub = MEMS(n).sub;
    if( sub == nlink ) 
	    continue;
    mema=MEMO(nlink).mema;
    nCol=MEMO(nlink).ocol;
    for(i=0;i<nCol;i++)
  	if(MEMA(mema+i).type==4)
  		{
  		int mmtab;
  		mmtab=MEMA(mema+i).rela;
  		DbReadTable( lun, (int *)((artelink_.larln1 + mmtab ) * 4),
  		                 maj, min, subopt, &var);
  		}
     }
     
  }
  

  /* Book parallel tables and relations. */

  if(!tbls_read)
  {
	  *ierr=7;
	  return;
  }
  
  m = 0;
  mmbook_( Mlink, &m );
  mmrupd_( &nlink );

  *ierr=0;
  return;

}
