//                              -*- Mode: C++ -*-
//:dgcalche.C
// 
//:description - ADC CHANNEL -> ENERGY CONVERSION
//
// Author: Benedetto Giacobbe   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: giacobbe@bo.infn.it
//
// Created 26/7/97
// Last Update ........
//

//#include <stdlib.h>
#include <math.h>
#include "arte/arte.hh"
#include "care/dgcal.hh"

const double My_PI = 3.14159265358979323846;

float dgcalche(int idcell, int ich, float & fadc, float & enerr){
    
  if( ich<0 || ich>4095) {   // this is an hardware problem!
    fadc = 0.0;
    enerr = 0.001;
    return 0.0;
  }
  ArtePointer<ECHF> echf(idcell);      
  ArtePointer<ECHS> echs(idcell);      
  float ch = (float)(ich)-echf->pedf;  /* ADC count -pedestal */
  fadc = ch;
  float q_rec =  echs->pcal[0];
  
  if( echs->pcal[2]<20.0){
    q_rec += echs->pcal[1]*ch; // linear transfer function
  } else {
	// Here I really mean what I wrote: 
        // ADCch - knee position without any pedestal subtraction.
    double y = ((float)(ich)-echs->pcal[2])/echs->pcal[3];

    double theta = 0.5+atan(y)/My_PI;
    q_rec = echs->pcal[0]+echs->pcal[1]*ch+
      theta*(echs->pcal[4]+echs->pcal[5]*ch);
  }
  
  float en_rec = q_rec*echf->sens;   
 
  // evaluate the energy error
  enerr = q_rec*echf->esen;  // contribution due to sensitivity 
  enerr *= enerr;
  enerr += echs->ecal[0];
  float en_redpow = en_rec;
  for(int j=1; j<3; j++){
    enerr += echs->ecal[j]*en_redpow;
    en_redpow *= en_rec;
  }

  return en_rec;
} 
