
// *****************************************************************************
//
// source:    DQ.C
//
// type:      source code
//
// created:   19 Oct 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ routines
//
// changes:
// 20 Oct 1999 (AG) make DQ_flag() usable for C         
// 01 Nov 1999 (AG) use LVL_M_ABC for nrclevel
// 04 Nov 1999 (AG) repair kuip
// 26 Nov 1999 (AG) introduce DQ_RECO
// 12 Jan 2000 (AG) introduce GLOB = CORR for AC
// 01 Feb 2000 (AG) AC works without levels now
// 03 Feb 2000 (AG) always use level LVL_REC for AC_ID
// 21 Feb 2000 (AG) fixed bug
// 25 Feb 2000 (AG) moved IDs to dq.h (ONLINE)
// 26 Feb 2000 (AG) extract AC stuff to alca/AC.C
//  3 Mar 2000 (AG) VDS=SVD and HPT=HIPT
// 21 Mar 2000 (AG) check for DQ_flag() if component is in
// 27 Jul 2000 (AG) added printout for DQ/DB
// 01 Aug 2000 (AG) DQ_ID() and AC_ID() in ONLINE/libgth.a (from Rev0614 on)
// 13 Sep 2000 (AG) local HBOOK histogram file
// 30 Nov 2000 (AG) improved memory handling
// 22 Jan 2001 (AG) added FLT
// 23 Oct 2001 (JH) DQ_Comp_SetRAW() added
// 28 Jan 2002 (JH) DQ_Comp_SetHIT() added
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

// ONLINE includes
#include "component.h"

// Arte includes
#include "farm/farm.hh"
#include "dq/DQ.h"

// DQ externals
extern int DQ_DBS;
extern int DQ_EVAL;

extern int DQ_SVD;
extern int DQ_ITR;
extern int DQ_HIPT;
extern int DQ_OTR;
extern int DQ_RICH;
extern int DQ_TRD;
extern int DQ_ECAL;
extern int DQ_MUON;

extern int DQ_FLT;
extern int DQ_SLT;
extern int DQ_TLT;
extern int DQ_FARM;

extern int DQ_CORR;
extern int DQ_RECO;
extern int DQ_TARG;

extern char *DQ_HBOOK;

// prototypes
extern "C" void kugetc_( char *string, int *len, int len1 );

//
// DQ_flag() function
//
extern "C" int
DQ_flag( int COMP_XYZ, int LVL_M_ABC )
{
  int rc = 0;

  if( ( COMP_XYZ == COMP_SVD  ) && ( DQ_SVD  & LVL_M_ABC ) && faComp( COMP_M_SVD  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_ITR  ) && ( DQ_ITR  & LVL_M_ABC ) && faComp( COMP_M_ITR  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_HIPT ) && ( DQ_HIPT & LVL_M_ABC ) && faComp( COMP_M_HIPT ) ) rc = 1;
  if( ( COMP_XYZ == COMP_OTR  ) && ( DQ_OTR  & LVL_M_ABC ) && faComp( COMP_M_OTR  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_RICH ) && ( DQ_RICH & LVL_M_ABC ) && faComp( COMP_M_RICH ) ) rc = 1;
  if( ( COMP_XYZ == COMP_TRD  ) && ( DQ_TRD  & LVL_M_ABC ) && faComp( COMP_M_TRD  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_ECAL ) && ( DQ_ECAL & LVL_M_ABC ) && faComp( COMP_M_ECAL ) ) rc = 1;
  if( ( COMP_XYZ == COMP_MUON ) && ( DQ_MUON & LVL_M_ABC ) && faComp( COMP_M_MUON ) ) rc = 1;

  if( ( COMP_XYZ == COMP_FLT  ) && ( DQ_FLT  & LVL_M_ABC ) && faComp( COMP_M_FLT  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_SLT  ) && ( DQ_SLT  & LVL_M_ABC ) && faComp( COMP_M_SLT  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_TLT  ) && ( DQ_TLT  & LVL_M_ABC ) && faComp( COMP_M_TLT  ) ) rc = 1;
  if( ( COMP_XYZ == COMP_FARM ) && ( DQ_FARM & LVL_M_ABC ) && faComp( COMP_M_FARM ) ) rc = 1;

  if( ( COMP_XYZ == COMP_CORR ) && ( DQ_CORR & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_RECO ) && ( DQ_RECO & LVL_M_ABC ) ) rc = 1;

  //cout << "DQ_flag(): " << "COMP_XYZ=" << COMP_XYZ << " LVL_M_ABC=" << LVL_M_ABC << " rc=" << rc << endl;

  return rc; 
}

//
// dq_comp *************************************************************
//
extern "C"
void dq_comp_()
{
  ARTE_mark( "dq_comp 0", 0 );

  //--------------------------------------------------------------------
  //   read in subdetector
  //--------------------------------------------------------------------
  
  int len;        
  int len1 = 4;
  char input[7];
  kugetc_( input, &len, len1 );
  input[len] = 0;
  
  char *subdetector;
  //subdetector = (char *) malloc( len+1 );
  subdetector = new char[len+1];
  strncpy( subdetector, input, len );
  subdetector[len] = '\0';
  
  for( int i=0; i<len; i++ ) subdetector[i] = toupper( subdetector[i] );
  
  if( strncmp( subdetector,"SVD",   3 ) &&
      strncmp( subdetector,"VDS",   3 ) &&
      strncmp( subdetector,"ITR",   3 ) && 
      strncmp( subdetector,"HPT",   3 ) &&
      strncmp( subdetector,"HIPT",  4 ) &&
      strncmp( subdetector,"OTR",   3 ) && 
      strncmp( subdetector,"RICH",  4 ) &&
      strncmp( subdetector,"TRD",   3 ) && 
      strncmp( subdetector,"ECAL",  4 ) &&
      strncmp( subdetector,"MUON",  3 ) && 
      strncmp( subdetector,"FLT",   3 ) &&
      strncmp( subdetector,"SLT",   3 ) &&
      strncmp( subdetector,"TLT",   3 ) &&
      strncmp( subdetector,"FARM",  4 ) && 
      strncmp( subdetector,"TARG",  4 ) &&
      strncmp( subdetector,"CORR",  4 ) &&
      strncmp( subdetector,"RECO",  4 ) && 
      strncmp( subdetector,"ALL",   3 ) ){
    cout << "DQ_init(): ********* DQ/SUBD: unknown subdetector: "
	 << subdetector << endl;
  }
  //else {
  //cout << "DQ/SUBD: " << subdetector << endl;
  //}
  
  //--------------------------------------------------------------------
  //   read in subdetetcor status
  //--------------------------------------------------------------------

  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = 0;
  //char *status = (char *) malloc( len+1 );
  char *status = new char[len+1];
  strncpy( status, input, len );
  status[len] = '\0';
  
  if( len>0 ) for( int i=0; i<len; i++ ) status[i] = toupper( input[i] );
  
  if(strncmp(status,"ON",2) && strncmp(status,"OFF",3)){
    cout << "DQ_init(): ********* DQ/SUBD: unknown subdetector status: "
	 << status << endl;
  }
  //else {
  //cout << "DQ/SUBD: " << status << endl;
  //}
  
  //--------------------------------------------------------------------
  //   read in dq level (hit/raw/rec)
  //--------------------------------------------------------------------
  
  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = 0;
  int  level = 0;  
  //char *rclevel  = (char *) malloc( len+1 );
  char *rclevel  = new char[len+1];
  strncpy( rclevel, input, len );
  rclevel[len] = '\0';
  
  if( len>0 ) for( int i=0; i<len; i++ ) rclevel[i] = toupper(input[i]);
  
  if(      !strncmp( rclevel, "RAW", 3 ) ){
    level = LVL_M_RAW;
  }
  else if( !strncmp( rclevel, "HIT", 3 ) ){
    level = LVL_M_HIT;
  } 
  else if( !strncmp( rclevel, "REC", 3 ) ){
    level = LVL_M_REC;
  }
  else {
    cout << "DQ_init(): ****** DQ/SUBD: unknown level: " << rclevel << " "
	 << "Taking default." << endl;
  }
  //cout << "DQ/SUBD: " << level << endl;

  //--------------------------------------------------------------------
  //   set dq status
  //--------------------------------------------------------------------
  
  if( !strncmp( status,"OFF", 3 ) ) level=0;
  
  if( !strncmp( subdetector, "SVD" ,   3 ) ) DQ_SVD  |= level;
  if( !strncmp( subdetector, "VDS" ,   3 ) ) DQ_SVD  |= level;
  if( !strncmp( subdetector, "ITR" ,   3 ) ) DQ_ITR  |= level;
  if( !strncmp( subdetector, "HPT",    3 ) ) DQ_HIPT |= level;
  if( !strncmp( subdetector, "HIPT",   4 ) ) DQ_HIPT |= level;
  if( !strncmp( subdetector, "OTR" ,   3 ) ) DQ_OTR  |= level;
  if( !strncmp( subdetector, "RICH",   4 ) ) DQ_RICH |= level;
  if( !strncmp( subdetector, "TRD" ,   3 ) ) DQ_TRD  |= level;
  if( !strncmp( subdetector, "ECAL",   4 ) ) DQ_ECAL |= level;
  if( !strncmp( subdetector, "MUON",   4 ) ) DQ_MUON |= level;
  if( !strncmp( subdetector, "FLT" ,   3 ) ) DQ_FLT  |= level;
  if( !strncmp( subdetector, "SLT" ,   3 ) ) DQ_SLT  |= level;
  if( !strncmp( subdetector, "TLT" ,   3 ) ) DQ_TLT  |= level;
  if( !strncmp( subdetector, "FARM",   4 ) ) DQ_FARM |= level;
  if( !strncmp( subdetector, "CORR",   4 ) ) DQ_CORR |= level;
  if( !strncmp( subdetector, "RECO",   4 ) ) DQ_RECO |= level;
  
  if (!strncmp( subdetector, "ALL" , 3 ) ){
    DQ_SVD  |= level;
    DQ_HIPT |= level;
    DQ_OTR  |= level;
    DQ_ITR  |= level;
    DQ_RICH |= level;
    DQ_TRD  |= level;
    DQ_ECAL |= level;
    DQ_MUON |= level;
    DQ_FLT  |= level;
    DQ_SLT  |= level;
    DQ_TLT  |= level;
    DQ_FARM |= level;
    DQ_CORR |= level;
    DQ_RECO |= level;
  }

  // print
  cout << "DQ/SUBD: " << subdetector << " " << status << " " << level << endl;

  // clean up
  delete subdetector;
  delete status;
  delete rclevel;

  ARTE_mark( "dq_comp 1", 0 );
  return;
}

//
// dq_dbs **************************************************************
//
extern "C"
void dq_dbs_()
{  
  ARTE_mark ( "dq_dbs 0", 0 );

  //
  // read in status for saving histograms to the DB
  //
  int len  = 0;        
  int len1 = 3;
  char input[7];
  memset(  input, '\0', 7 );
  kugetc_( input, &len, len1 );
  //char *status = (char *) malloc( len+1 );
  char *status = new char[len+1];
  memset(  status, '\0', len+1 );
  strncpy( status, input, len );
  
  if( len>0 ) for( int i=0; i<len; i++ ) status[i] = toupper( status[i] );
  
  if( strncmp( status, "ON", 2 ) && strncmp( status, "OFF", 3 ) ){
    cout << "DQ_init(): ********* DQ/DB: unknown status for saving histograms to DB: "
	 << status << endl;
  }

  //
  // status
  //
  if( !strncmp( status, "OFF", 3 ) ) DQ_DBS = 0;
  if( !strncmp( status, "ON",  2 ) ) DQ_DBS = 1;

  // print
  cout << "DQ/DBS: " << DQ_DBS << endl;

  // clean up
  delete status;

  ARTE_mark ( "dq_dbs 1", 0 );
  return;
}

//
// dq_hbook *************************************************************
//
extern "C"
void dq_hbook_()
{
  ARTE_mark( "dq_hbook 0", 0 );

  //
  // read in local HBOOK file name
  //
  int len  = 0;        
  int len1 = 80;
  char input[80];
  memset(  input, '\0', 80 );
  kugetc_( input, &len, len1 );
  //char *file = (char *) malloc( len+1 );
  char *file = new char[len+1];
  memset(  file, '\0', len+1 );
  strncpy( file, input, len );

  if( !strncmp( file, " ", 1 ) ) return;

  DQ_HBOOK = new char[80];
  memset( DQ_HBOOK, '\0', 80 );
  strcpy( DQ_HBOOK, file );

  // print
  cout << "DQ/HBOOK: " << DQ_HBOOK << endl;

  // clean up
  delete file;

  ARTE_mark( "dq_hbook 0", 0 );
  return;
}

//
// dq_eval *************************************************************
//
extern "C"
void dq_eval_()
{  
  ARTE_mark( "dq_eval 0", 0 );

  //--------------------------------------------------------------------
  //   read in demand for evaluating histograms
  //--------------------------------------------------------------------
  int len  = 0;        
  int len1 = 3;
  char input[7];
  kugetc_(input, &len, len1);
  input[len] = 0;
  //char *status = (char *) malloc(len+1);
  char *status = new char[len+1];
  strncpy(status, input, len);
  status[len] = '\0';
  
  if(len>0) for(int i=0; i<len; i++) status[i] = toupper(status[i]);
  
  if(strncmp(status,"ON",2)&&strncmp(status,"OFF",3)){
    cout << "DQ_init(): ********* DQ/EVAL: unknown demand for evaluating the histograms: "
	 << status << endl;
  }

  //--------------------------------------------------------------------
  //   set status
  //--------------------------------------------------------------------  
  if (!strncmp(status,"OFF",3)) DQ_EVAL=0;
  if (!strncmp(status,"ON", 2)) DQ_EVAL=1;

  // clean up
  delete status;

  ARTE_mark( "dq_eval 0", 0 );
  return;
}

//
// print
//
void
DQ_print( const char *txt )
{
  cout << "DQ_print(): Called from " << txt << ":" << endl;
  cout << "            DQ_SVD= " << DQ_SVD  << endl;
  cout << "            DQ_ITR= " << DQ_ITR  << endl;
  cout << "            DQ_HIPT=" << DQ_HIPT << endl;
  cout << "            DQ_OTR= " << DQ_OTR  << endl;
  cout << "            DQ_RICH=" << DQ_RICH << endl;
  cout << "            DQ_TRD= " << DQ_TRD  << endl;
  cout << "            DQ_ECAL=" << DQ_ECAL << endl;
  cout << "            DQ_MUON=" << DQ_MUON << endl;
  cout << "            DQ_FLT= " << DQ_FLT  << endl;
  cout << "            DQ_SLT= " << DQ_SLT  << endl;
  cout << "            DQ_TLT= " << DQ_TLT  << endl;
  cout << "            DQ_FARM=" << DQ_FARM << endl;
  cout << "            DQ_CORR=" << DQ_CORR << endl;
  cout << "            DQ_RECO=" << DQ_RECO << endl;
  cout << "            DQ_DBS= " << DQ_DBS  << endl;
  cout << "            DQ_EVAL=" << DQ_EVAL << endl;
}


// Set DQ level to only raw (e.g. when no online reconstruction)
void
DQ_Comp_SetRAW(void)
{
  if (DQ_SVD)  DQ_SVD  = LVL_M_RAW;
  if (DQ_ITR)  DQ_ITR  = LVL_M_RAW;
  if (DQ_HIPT) DQ_HIPT = LVL_M_RAW;
  if (DQ_OTR)  DQ_OTR  = LVL_M_RAW;
  if (DQ_RICH) DQ_RICH = LVL_M_RAW;
  if (DQ_TRD)  DQ_TRD  = LVL_M_RAW;
  if (DQ_ECAL) DQ_ECAL = LVL_M_RAW;
  if (DQ_MUON) DQ_MUON = LVL_M_RAW;
  if (DQ_FLT)  DQ_FLT  = LVL_M_RAW;
  if (DQ_SLT)  DQ_SLT  = LVL_M_RAW;
  if (DQ_TLT)  DQ_TLT  = LVL_M_RAW;
  if (DQ_FARM) DQ_FARM = LVL_M_RAW;
  if (DQ_CORR) DQ_CORR = LVL_M_RAW;
  if (DQ_RECO) DQ_RECO = LVL_M_RAW;
}

// Set DQ level to only raw+hit (e.g. when no online reconstruction but hit level on)
void
DQ_Comp_SetHIT(void)
{
  if (DQ_SVD)  DQ_SVD  = LVL_M_RAW | LVL_M_HIT;
  if (DQ_ITR)  DQ_ITR  = LVL_M_RAW | LVL_M_HIT;
  if (DQ_HIPT) DQ_HIPT = LVL_M_RAW | LVL_M_HIT;
  if (DQ_OTR)  DQ_OTR  = LVL_M_RAW | LVL_M_HIT;
  if (DQ_RICH) DQ_RICH = LVL_M_RAW | LVL_M_HIT;
  if (DQ_TRD)  DQ_TRD  = LVL_M_RAW | LVL_M_HIT;
  if (DQ_ECAL) DQ_ECAL = LVL_M_RAW | LVL_M_HIT;
  if (DQ_MUON) DQ_MUON = LVL_M_RAW | LVL_M_HIT;
  if (DQ_FLT)  DQ_FLT  = LVL_M_RAW | LVL_M_HIT;
  if (DQ_SLT)  DQ_SLT  = LVL_M_RAW | LVL_M_HIT;
  if (DQ_TLT)  DQ_TLT  = LVL_M_RAW | LVL_M_HIT;
  if (DQ_FARM) DQ_FARM = LVL_M_RAW | LVL_M_HIT;
  if (DQ_CORR) DQ_CORR = LVL_M_RAW | LVL_M_HIT;
  if (DQ_RECO) DQ_RECO = LVL_M_RAW | LVL_M_HIT;
}
