// *****************************************************************************
//
// source:    DQ_ITR_hit_evnt.C
// author:    Sonja Wunderlich
// called by: DQ_evnt()
//
// changes:
// 6/26 Jun 2000 (AG) creation
// 11/24 (SW) rhp_hbook: tracks per event (history)            ID:22
// 11/24 (SW) rhp_hbook: Hits on tracks per event              ID:2
// 11/24 (SW) rhp_hbook: number of ITR-segments                ID:1000
// 11/24 (SW) rhp_hbook: number of ITR-Vds-segments            ID:1001
// 11/24 (SW) rhp_hbook: number of matched segements           ID:1002
// 12/19 (SW) all history plots are rubisch, removed
// 12/21 (SW) loops for debugging with hbook
//  1/16 (SW) rhp_hbook: Hits on tracks per event and HSL      ID:200
//  1/17 (SW) rhp_hbook: residuals per HSL                     ID:900
//  2/5  (SW) rhp_hbook: 2d resiuals (large window)            ID:9
//  2/5  (SW) rhp_hbook: residuals per HSL (smaller window)    ID:900 
//  2/5  (SW) uses the code DQ_ITR_my_hbook (D_ITR.h) to book rhp and hbook  
//  4/14 (SW) use finer binning for the fine bined residual histogram
//            move all residuals histograms in one 2d histogramm ID:99
// 9/4   (SW) take care to use no DQ when there are Monte Carlo's  
// 1/4/02 (JK) Adaption to new geometry (missing magnet stations)
// ******************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"
#include "hbook.h"
// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_ITR.h"

int DQ_ITR_rec_init(){
#if defined (DQ_ITR_COMPILE)
  cout<<"IIIIIIIIIIIIIIIIII DQ_ITR_rec_init.C IIIIIIIIIIIIIIIIIIIIIIIII "<<endl;
#endif
  ARTE_mark( "DQ_ITR_rec_init", 0 );
  // check (again) if this is Monte Carlo
  if (ArteTable<MTRA>::size() > 0) {
    ARTE_mark( "DQ_ITR_rec_init 0", 0 );  
    return (GRR_OK);
  }

  


  DQ_ITR_my_hbook1(h_HitsOnRSEGPerEvent, LVL_REC, "ITR: Hits on RSEGs", 40, 0, 40);
  DQ_ITR_my_hbook1(h_RSEGPerEvent,       LVL_REC,"ITR: RSEG per Event", 200, -0.05, 9.05 ); 
  DQ_ITR_my_hbook1(h_ITR_numRSEG,        LVL_REC, "ITR: Number of ITR RTRA-RSEGs", 100, 0, 50);
  DQ_ITR_my_hbook1(h_ITR_matchedRSEG,    LVL_REC , "ITR: Number of ITR-VDS matches", 100, 0, 50);
  DQ_ITR_my_hbook1(h_ITR_matchRatio,     LVL_REC,"ITR: Fraction of matched RSEGs" , 42, -0.05, 1.);
  DQ_ITR_my_hbook2(h_HitsOnRSEGQU_all,   LVL_REC,"ITR: Hits on RSEGs (segment)",
		   (2*k_numHalfStation),-.5, (2*k_numHalfStation)- 0.5, 11, -0.5, 10.5); 
  DQ_ITR_my_hbook2(h_Residual,           LVL_REC, "ITR: 2D Residual (station)",
		   k_numHalfStation,-.5, k_numHalfStation - 0.5, 100, -5, 5);  
  DQ_ITR_my_hbook2(h_Residualfine,LVL_REC, "ITR: 2D Residual (fine, station)", 
		   k_numHalfStation, -0.5 ,k_numHalfStation -0.5, 200, -1 , 1);
  //when you change the number of bins, 
  //you have to chamge it in mon_ITR.C(DQ__ITR_getmaxbin) als well
  

  
  ARTE_mark( "DQ_ITR_hit_init 1", 0 );          
  return 0; 
}
  
