
// ******************************************************************
//
// D. Samtleben 10 Aug 1999
// A. Gellrich
//
// Filling of histograms
//
// hints:
//        components: COMP_SVD
//                    COMP_ITR
//                    COMP_HIPT
//                    COMP_OTR
//                    COMP_RICH
//                    COMP_TRD
//                    COMP_ECAL
//                    COMP_MUON
//                    COMP_FLT
//                    COMP_SLT
//                    COMP_TLT
//                    COMP_FARM
//
//         levels:    LVL_RAW
//                    LVL_HIT
//                    LVL_REC
//
//         histogram id: 1 - 65535
//
// changes:
// 27 Oct 1999 (AG) use defines in DQ.h
//  1 Nov 1999 (AG) get rid of dq_hid.h
//  2 Oct 2001 (UH) complete rewriting
// 14 Feb 2002 (UH) bypass MC events
//
// ******************************************************************

// standard includes
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"

// Arte includes
#include "arte/arte.hh"
#include "dq/DQ.h"
#include "dq/DQ_MUON.h"

// float faDTARia();
// int faDDAQFCS_tcode();

//
// function code
//
int DQ_MUON_hit_evnt()
{

  unsigned int n_tubehits[5]  = {0};
  unsigned int n_padhits[5]   = {0};
  unsigned int n_pixelhits[5] = {0};

  for(ArteTable<HITB>::iterator hitbIt = ArteTable<HITB>::begin();
      hitbIt != ArteTable<HITB>::end(); ++hitbIt) {

    ArtePointer<HITB> hitb = *hitbIt;
    int   cmp  = hitb->gede->cmp;
    int   nsl  = hitb->gede->gesl->nsl;
    int   lay  = hitb->gede->lay;
    float symx = hitb->gede->symx;

    if(cmp == 8) { // tubes and pads
      switch(nsl) {
      case 1: // MU1
	switch(lay) {
	case 1:
	  if(symx > 200) {
	    DQ_MUON_hf1(MT11_OCC,LVL_HIT,hitb->x,1.); // upper
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,1.,hitb->x,1.);
	  } else { 
	    DQ_MUON_hf1(MT12_OCC,LVL_HIT,hitb->x,1.); // lower
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,2.,hitb->x,1.);
	  }

	  break;
	case 2:
	  if(symx > 200) {
	    DQ_MUON_hf1(MT13_OCC,LVL_HIT,hitb->x,1.); // upper
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,3.,hitb->x,1.);
	  } else { 
	    DQ_MUON_hf1(MT14_OCC,LVL_HIT,hitb->x,1.); // lower
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,4.,hitb->x,1.);
	  }
	  break;
	case 3:
	  if(symx > 200) {
	    DQ_MUON_hf1(MT15_OCC,LVL_HIT,hitb->x,1.); // upper
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,5.,hitb->x,1.);
	  } else { 
	    DQ_MUON_hf1(MT16_OCC,LVL_HIT,hitb->x,1.); // lower
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,6.,hitb->x,1.);
	  }
	  break;
	}
	n_tubehits[nsl]++;
	break;
      case 2: // MU2
	switch(lay) {
	case 1:
	  if(symx > 200) {
	    DQ_MUON_hf1(MT21_OCC,LVL_HIT,hitb->x,1.); // upper
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,7.,hitb->x,1.);
	  } else { 
	    DQ_MUON_hf1(MT22_OCC,LVL_HIT,hitb->x,1.); // lower
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,8.,hitb->x,1.);
	  }
	  break;
	case 2:
	  if(symx > 200) {
	    DQ_MUON_hf1(MT23_OCC,LVL_HIT,hitb->x,1.); // upper
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,9.,hitb->x,1.);
	  } else { 
	    DQ_MUON_hf1(MT24_OCC,LVL_HIT,hitb->x,1.); // lower
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,10.,hitb->x,1.);
	  }
	  break;
	case 3:
	  if(symx > 200) {
	    DQ_MUON_hf1(MT25_OCC,LVL_HIT,hitb->x,1.); // upper
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,11.,hitb->x,1.);
	  } else { 
	    DQ_MUON_hf1(MT26_OCC,LVL_HIT,hitb->x,1.); // lower
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,12.,hitb->x,1.);
	  }
	  break;
	}
	n_tubehits[nsl]++;
	break;
      case 3: // MU3
	if(fabs(hitb->y) < 1e-6) { // tubes don't have y coord (along wire)
	  if(symx > 200) {
	    DQ_MUON_hf1(MT31_OCC,LVL_HIT,hitb->x,1.); // upper
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,13.,hitb->x,1.);
	  } else {
	    DQ_MUON_hf1(MT32_OCC,LVL_HIT,hitb->x,1.); // lower
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,14.,hitb->x,1.);
	  }
	  n_tubehits[nsl]++;
	} else { 
	  DQ_MUON_hfill(MP3_OCC,LVL_HIT,hitb->x,hitb->y,1.);
	  n_padhits[nsl]++;
	}
	break;
      case 4: // MU4
	if(fabs(hitb->y) < 1e-6) { // tubes don't have y coord (along wire)
	  if(symx > 200) {
	    DQ_MUON_hf1(MT41_OCC,LVL_HIT,hitb->x,1.); // upper
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,15.,hitb->x,1.);
	  } else {
	    DQ_MUON_hf1(MT42_OCC,LVL_HIT,hitb->x,1.); // lower
	    DQ_MUON_hfill(MT_OCC,LVL_HIT,16.,hitb->x,1.);
	  }
	  n_tubehits[nsl]++;
	} else { 
	  DQ_MUON_hfill(MP4_OCC,LVL_HIT,hitb->x,hitb->y,1.);
	  n_padhits[nsl]++;
	}
	break;
      }
    } else if(cmp == 9) { // pixels
      switch(nsl) {
      case 1:
	DQ_MUON_hfill(MX1_OCC,LVL_HIT,hitb->x,hitb->y,1.);
	break;
      case 2:
	DQ_MUON_hfill(MX2_OCC,LVL_HIT,hitb->x,hitb->y,1.);
	break;
      case 3:
	DQ_MUON_hfill(MX3_OCC,LVL_HIT,hitb->x,hitb->y,1.);
	break;
      case 4:
	DQ_MUON_hfill(MX4_OCC,LVL_HIT,hitb->x,hitb->y,1.);
	break;
      }
      n_pixelhits[nsl]++;
    }
  }

  DQ_MUON_hf1(MT1_HITS,LVL_HIT,n_tubehits[1], 1.);
  DQ_MUON_hf1(MT2_HITS,LVL_HIT,n_tubehits[2], 1.);
  DQ_MUON_hf1(MT3_HITS,LVL_HIT,n_tubehits[3], 1.);
  DQ_MUON_hf1(MT4_HITS,LVL_HIT,n_tubehits[4], 1.);
  DQ_MUON_hf1(MP3_HITS,LVL_HIT,n_padhits[3],  1.);
  DQ_MUON_hf1(MP4_HITS,LVL_HIT,n_padhits[4],  1.);
  DQ_MUON_hf1(MX1_HITS,LVL_HIT,n_pixelhits[1],1.);
  DQ_MUON_hf1(MX2_HITS,LVL_HIT,n_pixelhits[2],1.);
  DQ_MUON_hf1(MX3_HITS,LVL_HIT,n_pixelhits[3],1.);
  DQ_MUON_hf1(MX4_HITS,LVL_HIT,n_pixelhits[4],1.);

  return 0; // return error code (0 = okay)
}
