
/******************************************************************* 
 *
 * D. Samtleben 10 Aug 1999
 * A. Gellrich
 *
 * Filling of histograms
 *
 * hints:
 *        components: COMP_SVD
 *                    COMP_ITR
 *                    COMP_HIPT
 *                    COMP_OTR
 *                    COMP_RICH
 *                    COMP_TRD
 *                    COMP_ECAL
 *                    COMP_MUON
 *                    COMP_FLT
 *                    COMP_SLT
 *                    COMP_TLT
 *                    COMP_FARM
 *
 *         levels:    LVL_RAW
 *                    LVL_HIT
 *                    LVL_REC
 *                         
 * changes:                
 * 22 Oct 1999 (AG) code polishing
 * 27 Oct 1999 (AG) use defines from DQ.h
 *  1 Nov 1999 (AG) get rid of dq_hid.h
 *  2 Oct 2001 (UH) complete revision
 * 14 Feb 2002 (UH) add new FED-based occupancies
 *                                       
 *******************************************************************/

/* standard includes */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

/* ONLINE includes */
#include "rhp.h"
#include "event.h"
#include "etb.h"
#include "grr.h"
#include "table.h"

/* Arte includes */
#include "dq/DQ.h"
#include "dq/DQ_MUON.h"

#define MUON_MAX_CRATES (36)

char* MUON_DATABASE_FEDCONFIG = "MuonFEDConfig";
char* MUON_DATABASE_SERVER    = "/MUON_NEW";

/* Mode to read configuration information */
enum MuonConfigurationModes {
  MUON_AUTO,      /* 0 */
  MUON_DATABASE,  /* 1 */
  MUON_DBFILE,    /* 2 */
  MUON_FILE,      /* 3 */
  MUON_HARDCODED  /* 4 */
};

/* structure for FED readout information */
typedef struct {
  int fed;         /* number of the FED ("FED name")*/
  int position;    /* position in readout */
  int fedid;       /* ID of the FED */
  int compid;      /* component ID */
  char sharc[10];  /* sharc port */
  char cluster[4]; /* cluster card */
} MuonDAQInfo;

/* FED IDs in a reference event */
typedef struct {
  int tube_fed_ids[MUON_MAX_CRATES];  /* array of tube 
					 chamber FED ids */
  int pad_fed_ids[MUON_MAX_CRATES];   /* array of pad 
					 chamber FED ids */
  int pixel_fed_ids[MUON_MAX_CRATES]; /* array of pixel 
					 chamber FED ids */
  int n_tube_feds;                    /* number of tube feds */
  int n_pad_feds;                     /* number of pad feds */
  int n_pixel_feds;                   /* number of pixel feds */
  int bx_id;                          /* reference BX ID */
} MuonReferenceEvent;


FILE* DQ_MUON_open_file_read( const char* name, 
			      int mode );
int   DQ_MUON_arrange_readout( MuonDAQInfo daq_position[ ],
			       int* ptr_daq_position_length,
			       MuonReferenceEvent* ptr_ref_event );
int   DQ_MUON_read_fed_config( MuonDAQInfo crate[ ], 
			       int* ptr_crate_length,
			       int mode );
int   DQ_MUON_bad_fed( ETB_fed fed, 
		       int ref_fed_id, 
		       int ref_bx_id );
void  DQ_MUON_sumhits( int i_fed, 
		       ETB_fed* fed,
		       int bx ); 
int   DQ_MUON_get_phys_bx ( EVENT_HDR* event );

/* from farm/farm.hh */
extern int ARTE_get_feds_DMUT( int max, ETB_fed *feds, int *n );
extern int ARTE_get_feds_DMUX( int max, ETB_fed *feds, int *n );
extern int ARTE_get_feds_DMUP( int max, ETB_fed *feds, int *n );



/*               */
/* function code */
/*               */
int DQ_MUON_raw_evnt( EVENT_HDR *event ) {

  const char* fcn_name = "DQ_MUON_raw_evnt";
  int i_fed,offset,rc;
  static int firstevent = 1;
  static MuonDAQInfo daq_position[MUON_MAX_CRATES];
  static int daq_position_length = 0;

  int tube_rc, pad_rc, pixel_rc;
  ETB_fed tube_feds[MUON_MAX_CRATES];
  ETB_fed pad_feds[MUON_MAX_CRATES];
  ETB_fed pixel_feds[MUON_MAX_CRATES];
  int n_tube_feds, n_pad_feds, n_pixel_feds;
  int physbx;

  static MuonReferenceEvent ref_event;

  /* SLT-like FED readout */
  tube_rc  = ARTE_get_feds_DMUT(MUON_MAX_CRATES, tube_feds,  &n_tube_feds );
  pad_rc   = ARTE_get_feds_DMUP(MUON_MAX_CRATES, pad_feds,   &n_pad_feds  );
  pixel_rc = ARTE_get_feds_DMUX(MUON_MAX_CRATES, pixel_feds, &n_pixel_feds);
  if ( tube_rc != ETB_ok || pad_rc != ETB_ok || pixel_rc != ETB_ok ) {
    return -1;
  }

  /* first event: prepare everything */
  if(firstevent) {
    rc = DQ_MUON_arrange_readout(daq_position,
				 &daq_position_length,
				 &ref_event);
    if(rc == 0) {
      firstevent = 0;

      if(debug_level >= 2) {
	printf(" * %s: readout channels successfully arranged.\n",fcn_name);
      }
    }
  }

  /* event counter */
  DQ_MUON_hf1(BADFED,LVL_RAW,4.,1.);

  /* check the tube FEDs */
  if(tube_feds[0].hdr.bxid == pad_feds[0].hdr.bxid &&
     tube_feds[0].hdr.bxid == pixel_feds[0].hdr.bxid) {
    ref_event.bx_id = tube_feds[0].hdr.bxid;
  }

  physbx = DQ_MUON_get_phys_bx( event );

  for(i_fed = 0; i_fed < n_tube_feds; ++i_fed) {
    
    DQ_MUON_sumhits( i_fed, &tube_feds[ i_fed ], physbx );

    tube_rc = DQ_MUON_bad_fed(tube_feds[i_fed],
			      daq_position[i_fed].fedid,
			      ref_event.bx_id);
    if(tube_rc & 1) {
      DQ_MUON_hf1(BADFED,LVL_RAW,0.,1.);
      DQ_MUON_hf1(TUBE_BADFED,LVL_RAW,(float)i_fed,1.);
    }
    if(tube_rc & 2) {
      DQ_MUON_hf1(BADFED,LVL_RAW,1.,1.);
      DQ_MUON_hf1(TUBE_BADREC,LVL_RAW,(float)i_fed,1.);
    }
    if(tube_rc & 4) {
      DQ_MUON_hf1(BADFED,LVL_RAW,2.,1.);
      DQ_MUON_hf1(TUBE_BADBX,LVL_RAW,(float)i_fed,1.);
    }   
    if(tube_rc & 8) {
      DQ_MUON_hf1(BADFED,LVL_RAW,3.,1.);
      DQ_MUON_hf1(TUBE_BADLEN,LVL_RAW,(float)i_fed,1.);
    }
  }

  /* check the pad FEDs */
  for(i_fed = 0; i_fed < n_pad_feds; ++i_fed) {
    offset = n_tube_feds;

    DQ_MUON_sumhits( i_fed + offset, &pad_feds[ i_fed ], physbx );
  
    pad_rc = DQ_MUON_bad_fed(pad_feds[i_fed],
			     daq_position[i_fed+offset].fedid,
			     ref_event.bx_id);
    if(pad_rc & 1) {
      DQ_MUON_hf1(BADFED,LVL_RAW,0.,1.);
      DQ_MUON_hf1(PAD_BADFED,LVL_RAW,(float)i_fed,1.);
    }
    if(pad_rc & 2) {
      DQ_MUON_hf1(BADFED,LVL_RAW,1.,1.);
      DQ_MUON_hf1(PAD_BADREC,LVL_RAW,(float)i_fed,1.);
    }
    if(pad_rc & 4) {
      DQ_MUON_hf1(BADFED,LVL_RAW,2.,1.);
      DQ_MUON_hf1(PAD_BADBX,LVL_RAW,(float)i_fed,1.);
    }   
    if(pad_rc & 8) {
      DQ_MUON_hf1(BADFED,LVL_RAW,3.,1.);
      DQ_MUON_hf1(PAD_BADLEN,LVL_RAW,(float)i_fed,1.);
    }
  }

  /* check the pixel FEDs */
  for(i_fed = 0; i_fed < n_pixel_feds; ++i_fed) {
    offset = n_tube_feds + n_pad_feds;

    DQ_MUON_sumhits( i_fed + offset, &pixel_feds[ i_fed ], physbx );

    pixel_rc = DQ_MUON_bad_fed(pixel_feds[i_fed],
			       daq_position[i_fed+offset].fedid,
			       ref_event.bx_id);
    if(pixel_rc & 1) {
      DQ_MUON_hf1(BADFED,LVL_RAW,0.,1.);
      DQ_MUON_hf1(PIXEL_BADFED,LVL_RAW,(float)i_fed,1.);
    }
    if(pixel_rc & 2) {
      DQ_MUON_hf1(BADFED,LVL_RAW,1.,1.);
      DQ_MUON_hf1(PIXEL_BADREC,LVL_RAW,(float)i_fed,1.);
    }
    if(pixel_rc & 4) {
      DQ_MUON_hf1(BADFED,LVL_RAW,2.,1.);
      DQ_MUON_hf1(PIXEL_BADBX,LVL_RAW,(float)i_fed,1.);
    }   
    if(pixel_rc & 8) {
      DQ_MUON_hf1(BADFED,LVL_RAW,3.,1.);
      DQ_MUON_hf1(PIXEL_BADLEN,LVL_RAW,(float)i_fed,1.);
    }
  }
  return 0;
}

/**********************************************************************/

int DQ_MUON_bad_fed(ETB_fed fed, 
		    int ref_fed_id, 
		    int ref_bx_id) {
  
  int rc = 0;
  
  if (fed.hdr.fedid != ref_fed_id) rc |= 1;
  if (fed.hdr.recid>>12 != 0x1)    rc |= 2;
  if (fed.hdr.bxid != ref_bx_id)   rc |= 4; 
  if (fed.hdr.length != 136)       rc |= 8;
  return rc;
}

/**********************************************************************/

int DQ_MUON_arrange_readout(MuonDAQInfo daq_position[],
			    int* ptr_daq_position_length,
			    MuonReferenceEvent* ptr_ref_event) {

  const char* fcn_name = "DQ_MUON_arrange_readout";
  int i_fed, rc;
  MuonDAQInfo crate[MUON_MAX_CRATES];
  int crate_length = 0;

  /* read muon FED configuration */
  rc = DQ_MUON_read_fed_config( crate, &crate_length, MUON_HARDCODED );

  if(rc != 0) {
    printf(" * %s: error while reading FED configuration\n",fcn_name);
    return -1;
  }

  if(crate_length < 1 || crate_length > MUON_MAX_CRATES) {
    printf(" * %s: wrong array length %d\n",fcn_name,crate_length);
    return -1;
  }
  
  /* copy DAQ position to the right place and reset reference event */
  for(i_fed = 0; i_fed < crate_length; ++i_fed) {
    daq_position[crate[i_fed].position] = crate[i_fed];
    ptr_ref_event->tube_fed_ids[i_fed]  = 0;
    ptr_ref_event->pad_fed_ids[i_fed]   = 0;
    ptr_ref_event->pixel_fed_ids[i_fed] = 0;
  }

  ptr_ref_event->n_tube_feds  = 0;
  ptr_ref_event->n_pad_feds   = 0;
  ptr_ref_event->n_pixel_feds = 0;
  ptr_ref_event->bx_id        = 0;

  for(i_fed = 0; i_fed < crate_length; ++i_fed) {
    switch(daq_position[i_fed].compid) {
    case COMP_ID_MUON: 
      ptr_ref_event->tube_fed_ids[ptr_ref_event->n_tube_feds++] =
	daq_position[i_fed].fedid;
      break;
    case COMP_ID_MPAD: 
      ptr_ref_event->pad_fed_ids[ptr_ref_event->n_pad_feds++] =
	daq_position[i_fed].fedid;
      break;
    case COMP_ID_MPIX: 
      ptr_ref_event->pixel_fed_ids[ptr_ref_event->n_pixel_feds++] =
	daq_position[i_fed].fedid;
      break;
    default: 
      break;
    }
  }

  *ptr_daq_position_length = crate_length;
  return 0;
}

/**********************************************************************/

int DQ_MUON_read_fed_config(MuonDAQInfo crate[], 
			    int* ptr_crate_length,
			    int mode) {
  char* fcn_name = "DQ_MUON_read_fed_config";
  int i = 0;
  int thefed, thenumber, thefedid, thecompid;
  char line[MUON_LONG_STRING];

  /* database-related stuff */
  int T = 0;
  Tables X;
  int  db_fed     [MUON_MAX_CRATES];
  int  db_position[MUON_MAX_CRATES];
  int  db_fedid   [MUON_MAX_CRATES];
  int  db_compid  [MUON_MAX_CRATES];
  char db_sharc  [MUON_MAX_CRATES][10];
  char db_cluster[MUON_MAX_CRATES][4];
  int nentries;
  int major, minor;
  int try_file = 0;

  FILE* fp = NULL;

  /* for hardcoded configuration */
  const int hard_fed[MUON_MAX_CRATES] = {
    411, 412, 421, 413, 432, 422, 211, 221, 212, 111, 222, 112, 
    451, 452, 512, 521, 131, 132, 431, 442, 441, 522, 513, 121, 
    122, 523, 232, 231, 311, 312, 331, 332, 341, 342, 352, 351
  };

  const int hard_fedid[MUON_MAX_CRATES] = {
    1057, 1042, 1041, 1043, 1074, 1058,  
    529,   545,  530,  273,  546,  274, 
    1105, 1106, 1298, 1313,  305,  306, 
    1073, 1090, 1089, 1314, 1299,  289, 
    290,  1315,  562,  561,  785,  786, 
    817,  818,   833,  834,  850,  849
  };

  const int hard_compid[MUON_MAX_CRATES] = {
    2048, 2048, 2048, 2048, 2048, 2048, 
    2048, 2048, 2048, 2048, 2048, 2048, 
    2048, 2048, 2048, 2048, 2048, 2048, 
    2560, 2560, 2560, 2560, 2560, 2560, 
    2560, 2560, 2560, 2560, 2304, 2304, 
    2304, 2304, 2304, 2304, 2304, 2304
  };
  
  if ( mode == MUON_HARDCODED ) {
    for ( i = 0; i < MUON_MAX_CRATES; ++i ) {
      crate[ i ].fed = hard_fed[ i ];
      crate[ i ].position = i;
      crate[ i ].fedid = hard_fedid[ i ];
      crate[ i ].compid = hard_compid[ i ];
      strcpy(crate[i].sharc, "");
      strcpy(crate[i].cluster, "");
    }
    *ptr_crate_length = MUON_MAX_CRATES;
    return 0;
  }

  if ( mode == MUON_DATABASE || mode == MUON_AUTO ) {
    X = OpenTables ( MUON_DATABASE_SERVER, TABLEreadonly );
    if(X == NULL) {
      printf( " * %s: Cannot open database server %s\n",
	      fcn_name, MUON_DATABASE_SERVER );
      if ( mode == MUON_DATABASE ) {
	return -1;
      } else {
	try_file = 1;
      }
    }
    if ( try_file == 0 ) {
      T = BookTable ( X, MUON_DATABASE_FEDCONFIG, 0, 0 );
      if ( T <= 0 ) {
	printf( " * %s: Cannot open database table %s\n",
		fcn_name, MUON_DATABASE_FEDCONFIG );
	if ( mode == MUON_DATABASE ) {
	  return -1;
	} else {
	  try_file = 1;
	}
      }
    }

    major = TABLEmaxrev;
    minor = TABLEmaxrev;
    
    TableStamp(X, major, minor);
    
    /* fetch the table with the revision number closest to the last revision */
    FetchTable(X,T,TABLEle);
    major = TableMajor(X);
    minor = TableMinor(X);
    
    /* jump to first entry in the table */
    SkipTableField(X,1);
    

    nentries = TableFieldSize(X);
    
    GetTableField(X, TABLEint, nentries, db_fed);
    GetTableField(X, TABLEint, nentries, db_position);
    GetTableField(X, TABLEint, nentries, db_fedid);
    GetTableField(X, TABLEint, nentries, db_compid);
    GetTableField(X, TABLEascii, 10*nentries, db_sharc);
    GetTableField(X, TABLEascii, 4*nentries, db_cluster);
    
    CloseTables(X);

    /* copy the DB entries in the right place */
    for(i = 0; i < nentries; ++i) {
      crate[i].fed      = db_fed[i];
      crate[i].position = db_position[i];
      crate[i].fedid    = db_fedid[i];
      crate[i].compid   = db_compid[i];
      strcpy(crate[i].sharc, db_sharc[i]);
      strcpy(crate[i].cluster, db_cluster[i]);
/*       printf("%d\t%d\t0x%x\t0x%x\t%s\t%s\n", */
/* 	     crate[i].fed, crate[i].position,  */
/* 	     crate[i].fedid, crate[i].compid, */
/* 	     crate[i].sharc, crate[i].cluster); */
    } 
    
    *ptr_crate_length = nentries;
    return 0;
  }      
  
  fp = DQ_MUON_open_file_read("Mu.fedconfig",mode);
  
  if(fp == NULL) {
    *ptr_crate_length = 0;
    return -1;
  } else {
    while(fgets(line, sizeof(line), fp) != NULL) {
      if(sscanf(line, "FED %d POS %d FEDID %x COMPID %x "
		"SHARC %s CLUSTER %s\n",
		  &thefed, &thenumber, &thefedid, &thecompid,
		crate[i].sharc, crate[i].cluster) == 6) {
	crate[i].fed      = thefed;
	crate[i].position = thenumber;
	crate[i].fedid    = thefedid;
	crate[i].compid   = thecompid;
	i++;
      }
      }
    
    *ptr_crate_length = i;
    fclose(fp);
    return 0; /* return error code (0 = okay) */
  }
}

/**********************************************************************/

FILE* DQ_MUON_open_file_read(const char* name, 
			     int mode) {
  
  char* fcn_name = "DQ_MUON_open_file_read";
  FILE* fp;
  char hbroot[MUON_MEDIUM_STRING];
  char filename[MUON_MEDIUM_STRING];

  strcat(hbroot,getenv("HBROOT"));

  switch (mode) {
  case MUON_FILE:
    fp = fopen(name,"r");
    if(fp == NULL) {
      printf(" * %s: *\n  file '%s'"
	     " not found in current directory\n",fcn_name,name);
      return NULL;
    } else {
      printf(" * %s: *\n  file '%s' "
             "read from current directory\n",fcn_name,name);
    }
    break;
  case MUON_DBFILE:
    strcpy(filename,hbroot);
    strcat(filename,"/MUON/db/");
    strcat(filename,name);
    fp = fopen(filename,"r");
    if(fp == NULL) {
      strcpy(filename,"/afs/desy.de/group/hera-b/MUON/db/");
      strcat(filename,name);
      fp = fopen(filename,"r");
      if(fp == NULL) {
        printf(" * %s: *\n  file '%s'"
	       " not found in $HBROOT or AFS path \n",fcn_name,name);
        return NULL;
      } else {
        printf(" * %s: *\n  file '%s' "
               "read from %s path\n", fcn_name,name,
               strncmp(filename,"/afs",4) == 0 ? "AFS" : "$HBROOT");
      }  
    }
    break;
  case MUON_AUTO:
    fp = fopen(name,"r");
    if(fp == NULL) {
      strcpy(filename,hbroot);
      strcat(filename,"/MUON/db/");
      strcat(filename,name);
      fp = fopen(filename,"r");
      if(fp == NULL) {
        strcpy(filename,"/afs/desy.de/group/hera-b/MUON/db/");
        strcat(filename,name);
        fp = fopen(filename,"r");
        if(fp == NULL) {
          printf(" * %s: *\n  file '%s'"
                  " not found!\n",fcn_name,name);
          return NULL;
        } else {
          printf(" * %s: *\n  file '%s' "
                 "read from muon database file in AFS path\n",fcn_name,name);
        }
      } else {
        printf(" * %s: *\n  file '%s' "
                 "read from muon database file in $HBROOT path\n",fcn_name,name);
      }
    } else {
      printf(" * %s: *\n  file '%s' "
             "read from current directory\n",fcn_name,name);
    }
    break;
  case MUON_DATABASE:
    printf(" * %s: *\n  "
           "database access not yet implemented.\n",fcn_name);
  default:
    return NULL;
  }
  return fp;
}

/**********************************************************************/

void DQ_MUON_sumhits ( int i_fed, ETB_fed* fed, int bx ) 
{
  unsigned int* data_ptr;
  unsigned int* current_ptr;
  unsigned int i_cable, i_bit;
  unsigned int nhits = 0;

  static const unsigned int hard_equipped[ MUON_MAX_CRATES ] = {
    0xffff3fff, 0xffffffff, 0x7fffffff, 0xffffffff, 0xffffffff, 0xffff3fff,
    0x7fffffff, 0x07ff3fff, 0x1fff3f00, 0x07ffffff, 0x3fff0fff, 0x3fff7f1f,
    0x7fff0700, 0x03ffff00, 0x7fff0700, 0x03ffff00, 0x7fff7fff, 0xffffffff, 
    0x3fffffff, 0x3f3f0000, 0x3fffffff, 0x3fffffff, 0x3fffffff, 0x3fffffff,
    0x3fffffff, 0x3f3f0000, 0x3fffffff, 0x3fffffff, 0xffffff00, 0xffffff00,
    0xffffff00, 0xffffff00, 0xffffff00, 0xffffff00, 0xffffff00, 0xffffff00 
  };

  data_ptr = ( unsigned int* ) fed->data;

  for( i_cable = 0; i_cable < 32; ++i_cable ) {
    if( hard_equipped[ i_fed ] & ( 1U << i_cable ) ) {
      current_ptr = data_ptr + i_cable;
      for ( i_bit = 0; i_bit < 32; ++i_bit ) {
	if ( *( current_ptr ) & ( 1U << i_bit ) ) {
	  nhits++;
	  DQ_MUON_hfill( CABLE_OCC, LVL_RAW,
			 ( float ) i_fed * 32. + i_cable + 1, 
			 ( float )i_bit + 1,
			 1. ); 
	}
      }
    }
  }
  DQ_MUON_hfill( FED_OCCBX, LVL_RAW,
		 ( float ) i_fed,
		 ( float ) bx,
		 ( float ) nhits );
}

/**********************************************************************/

int DQ_MUON_get_phys_bx ( EVENT_HDR* event ) 
{
  EVT_DATA_RECORD all_daqs;
  int nb_daqs;
  
  ETB_get_daqs( event, 1, &all_daqs, &nb_daqs );

  return ( int ) all_daqs.data.physBX;
}

