
// ******************************************************************
//
// M. Braeuer / D. Samtleben
//
// Filling of histograms
//
// changes:
// 22 Oct 1999 (AG)  polsih SVD stuuf
// 28 Oct 1999 (AG)  use defines from DQ.h
//  1 Nov 1999 (AG)  get rid of dq_hid.h
// 10 Dez 1999 (MBr) major update
// 28 Nov 2000 (GS)  Arte_mark included
// 07 Dec 2000 (GS)  normalisation to target rate introduced
// 08 Dec 2000 (GS)  trigger flag included
// 18 Mar 2002 (GS)  handling target rates<1, bug fix in hit counting,
//		     trigger flag works now also on minis
// ******************************************************************

// standard includes
#include <stdlib.h>
#include <stdio.h>
#include "math.h"

// ONLINE includes
#include "rhp.h"
#include "event.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_SVD.h"

extern "C" {
#include "sltinfo.h"
}

// from Alignment stuff:
inline int fUnsin(int sl,int sec,int lay) { 
  int i = (lay-1)+(4*(sec-1))+(16*(sl-1));
  return(i); 
}

//
// function code
//
int
DQ_SVD_hit_evnt()
{
  // show up
  ARTE_mark( "DQ_SVD_hit_evnt 0", 0 );

  int NrSVDHits=0,VDSusid;
  float VDSocc;
  int unusedhits=0; 

  //targer rate and bg
  float IR = faDTARia();
  float BG = faDTARbg();

  //no scaling if no target
  if (IR<=1)
    IR = 1;

//====================================================================
//     trigger flag   
//====================================================================

  int tflag = -100;
  int daqcode=faDDAQFCS_tcode();
  if (daqcode==6 || daqcode>=8){
    //cout<<"Random (4Hz) triggered event"<<endl;
    tflag = 0;
  }
  else{
    if (ArteTable<DSLT>::size() != 0){  // this is a DST file
      if (SLTReadDSLT()==SLT_OK) {
        SLT_STATUS stat;
        if (SLTStatus(&stat)==SLT_OK) {
          int sltcode=stat.status;
          if (sltcode==SLTEVENT_ACCEPT) {
            //cout << "Event accepted by SLT " << endl;
            tflag = 1;
          }
        }
      }
    }
    else {  // this is only a mini file
      if( ArteTable<RSLI>::size()!=0 ){
        //cout << "Event accepted by SLT " << endl;
        tflag = 1;
      }
    }
  }

//====================================================================
//     loop over HITB to fill histograms 
//====================================================================
 
  for( ArteTableIterator<HITB> hitb; hitb.valid(); hitb.next() ){
    
    ArtePointer<GEDE> gede(hitb->gede); 
    if(gede->cmp==1) {
      if(hitb->rseg.begin()==hitb->rseg.end() ) {
	unusedhits++;
      }
      NrSVDHits++;
      VDSusid = float(fUnsin(gede->gesl->nsl,gede->sec,gede->lay) ); 
      
      // hit/plane raw:
      rhp_hf1(DQ_ID(COMP_SVD,LVL_HIT,101),VDSusid,(1/IR));
      
      // occupancy: (go back to real strips on plane)
      VDSocc = (hitb->x - gede->xfw[0]) / gede->pit;
      rhp_hfill(DQ_ID(COMP_SVD,LVL_HIT,102),VDSusid,VDSocc,(1/IR));
      
    }
  }
  //====================================================================
  //       histogram of Hit multiplicities
  //====================================================================

  rhp_hf1(DQ_ID(COMP_SVD,LVL_HIT,99+tflag),(NrSVDHits/IR),1);
  rhp_hf1(DQ_ID(COMP_SVD,LVL_HIT,301),(unusedhits/IR),1);
   
  ARTE_mark( "DQ_SVD_hit_evnt 1", 0 );

  return 0; // return error code (0 = okay)
}
