#ifndef _DQ_MUON_H_
#define _DQ_MUON_H_

/*******************************************************************
 *                                                                 *
 * A. Gellrich 11 Aug 1999                                         *
 *                                                                 *
 * function prototypes                                             *
 *                                                                 *
 * changes:                                                        *
 * 02 Oct 2001 (UH) adding new function prototypes                 *
 * 03 Apr 2002 (UH) some changes in histograms                     *
 *                                                                 *
 *******************************************************************/

/* some definitions */
#define NO_ALARMS

/* includes */
#include "event.h"
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif
  
  /* definitions, enums, structures */

  /* string lengths */
#define MUON_LONG_STRING (300)
#define MUON_MEDIUM_STRING (100)
#define MUON_SHORT_STRING (20)

  /* logic:
     4th digit (leftmost): component (1 tube, 2 pad, 3 pixel)
     3th digit           : different types (occupancy, hits, ...)
     digits 2+1          : chamber number
  */

  enum MuonHistogramIDsRaw {
    BADFED       = 0x0001, 
    FED_OCCBX    = 0x0011,
    CABLE_OCC,
    TUBE_BADFED  = 0x1100,
    TUBE_BADREC,
    TUBE_BADBX,
    TUBE_BADLEN,
    PAD_BADFED   = 0x2100,
    PAD_BADREC,
    PAD_BADBX,
    PAD_BADLEN,
    PIXEL_BADFED = 0x3100,
    PIXEL_BADREC,
    PIXEL_BADBX,
    PIXEL_BADLEN,
  };

  enum MuonHistogramIDsHit {
    MT1_HITS = 0x1101,
    MT2_HITS,
    MT3_HITS,
    MT4_HITS,
    MP3_HITS = 0x2103,
    MP4_HITS,
    MX1_HITS = 0x3101,
    MX2_HITS,
    MX3_HITS,
    MX4_HITS,
    MT_OCC   = 0x1001,
    MT11_OCC = 0x1011,
    MT12_OCC,
    MT13_OCC,
    MT14_OCC,
    MT15_OCC,
    MT16_OCC,
    MT21_OCC = 0x1021,
    MT22_OCC,
    MT23_OCC,
    MT24_OCC,
    MT25_OCC,
    MT26_OCC,
    MT31_OCC = 0x1031,
    MT32_OCC,
    MT41_OCC = 0x1041,
    MT42_OCC,
    MP3_OCC  = 0x2003,
    MP4_OCC,
    MX1_OCC  = 0x3001,
    MX2_OCC,
    MX3_OCC,
    MX4_OCC,
  };

  enum MuonHistogramIDsRec {
    NTRACKS  = 0x0001,
    IARATE   = 0x0002,
    TRACKPOS = 0x0010,
    NHITS    = 0x0100,
  };

  int DQ_MUON_raw_init();
  int DQ_MUON_raw_stop();
  int DQ_MUON_raw_evnt( EVENT_HDR *event );
  
  int DQ_MUON_hbook1(int ID, int level, char* title, 
		     int nbins, float xmin, float xmax);
  int DQ_MUON_hf1(int ID, int level, float x, float weight);
  int DQ_MUON_hbook2(int ID, int level, char* title, 
		     int nbinsx, float xmin, float xmax,
		     int nbinsy, float ymin, float ymax);  
  int DQ_MUON_hfill(int ID, int level, float x, float y, float weight);
#ifdef __cplusplus
}
#endif

 /* C++ prototypes */
int DQ_MUON_hit_init();
int DQ_MUON_hit_stop();
int DQ_MUON_hit_evnt();

int DQ_MUON_rec_init();
int DQ_MUON_rec_stop();
int DQ_MUON_rec_evnt();

/* constants (defined in DQ_MUON_utilities) */
extern const int debug_level;

#endif
