/************************************************************************
* HGKINE
* written by Siegmund Nowak
*
* This routines translates ARTE tables (MTRA,MVER) from event
* generator into GEANT structures JVERT, JTRACK  
*
*       called from HGEVNT
***********************************************************************/
#include <vector.h>
#include <math.h>
#include <stdio.h>
#include <string.h>
#include "gean/gclink.h"
#include "gean/gcnum.h"
#include "gean/ZGMOTH.h"
#include "gean/zgkine.hh"
#include "gean/zgmimp.hh"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteFlag.h"
#include "gean/zgbeam.hh"

vector<int>    *hg_vert;
vector<hgmtra> *hg_mtra;
vector<hgmimp> *normal_mimp;
vector<hgmimp> *modul_mimp;
vector<hgmimp> *TRD_mimp;
vector<hgmcal> *TR_mimp;
vector<hgmtcp> *ECA_ep;
vector<hgmcal> *ECA_mimp;
vector<hgmimp> *RICH_mimp;

extern "C" {
  void hgkine_();
  void gsvert_(float[],int*, int*, float*, int*, int* );
  void gskine_(float[],int*, int*, float*, int*, int* );
  void mmbook_(int &, const int&);
  void mmdrop_(int &);
}  
  void cr_ipoint(int &,float[],float &);

void hgkine_() {
   
// allocate memory for temporary storage - delete is done in guout
   hg_vert     = new vector<int>;
   if(hg_vert == NULL) {
      arteMsg << "hgkine: hg_vert = NULL";         arteMsg.error();}  
   hg_mtra     = new vector<hgmtra>;
   if(hg_mtra == NULL) {
      arteMsg << "hgkine: hg_mtra = NULL";         arteMsg.error();}  
   normal_mimp = new vector<hgmimp>;
   if(normal_mimp == NULL) {
      arteMsg << "hgkine: normal_mimp = NULL";     arteMsg.error();}  
   modul_mimp  = new vector<hgmimp>;
   if(modul_mimp == NULL) {
      arteMsg << "hgkine: modul_mimp = NULL";      arteMsg.error();}  
   TRD_mimp    = new vector<hgmimp>;
   if(TRD_mimp == NULL) {
      arteMsg << "hgkine: TRD_mimp = NULL";        arteMsg.error();}
   TR_mimp     = new vector<hgmcal>;
   if(TR_mimp  == NULL) {
      arteMsg << "hgkine: TR_mimp = NULL";         arteMsg.error();}  
   ECA_ep      = new vector<hgmtcp>;
   if(ECA_ep == NULL) {
      arteMsg << "hgkine: ECA_ep = NULL";          arteMsg.error();}  
   ECA_mimp    = new vector<hgmcal>;
   if(ECA_mimp == NULL) {
      arteMsg << "hgkine: ECA_mimp = NULL";        arteMsg.error();}  
   RICH_mimp   = new vector<hgmimp>; 
   if(RICH_mimp == NULL) {
      arteMsg << "hgkine: RICH_mimp = NULL";       arteMsg.error();}  

   hg_vert->reserve(1000);
   hg_mtra->reserve(2000);
   normal_mimp->reserve(3000);
   modul_mimp->reserve(500);
   TRD_mimp->reserve(1000); 
   ECA_ep->reserve(500);
   ECA_mimp->reserve(1000);  

// create banks for tracks and mimps   
   if(ArtePointer<MVER>::getLink().getRef()) 
          mmbook_(ArtePointer<MVER>::getLink().getRef(),0);
   if(ArtePointer<MTRA>::getLink().getRef()) 
          mmbook_(ArtePointer<MTRA>::getLink().getRef(),0);
   if(ArtePointer<MIMP>::getLink().getRef())
          mmdrop_(ArtePointer<MIMP>::getLink().getRef());
   if(ArtePointer<HITC>::getLink().getRef()) 
          mmdrop_(ArtePointer<HITC>::getLink().getRef());
   mmbook_(ArtePointer<MIMP>::getLink().getRef(),0);
   mmbook_(ArtePointer<MCAL>::getLink().getRef(),0);

// some useful quantities
   zgkine ZGKINE; 
   zgbeam ZBS;
   int nmcme = ArteTable<MCME>::size();
   int nmver = ArteTable<MVER>::size();
   int nmtra = ArteTable<MTRA>::size();
   int i2 = 0;  int i0 = 0;   int nvx;
   float vertex[3]; float plab[3]; float time_gen; float ubuf; 

// temporary storage for relation ARTE-GEAN - delete at the end of hgkine
   int *kv = new int[nmver];         int *kt = new int[nmtra];      
   for(int i =0; i<nmver; i++ ) kv[i] =0;
   for(int i =0; i<nmtra; i++ ) kt[i] =0;

// rotate tracks and vertices due to beam slope  
   for( ArteTableIterator<MTRA> mtra; mtra.valid(); mtra.next()) {
      ZBS.rotate(plab,&mtra->px); 
      memcpy(&mtra->px,plab,12);
      float p = sqrt(mtra->px*mtra->px+mtra->py*mtra->py+mtra->pz*mtra->pz);
      if(mtra->flag == 3) {
        ArtePointer<MVER> vxin = mtra->bver;
        for(ArteRelationIterator<MVER> vxex = mtra->ever.begin();
          vxex != mtra->ever.end() ; ++vxex ) { //Loop over all points on track
          ArtePointer<MVER> mve(vxex);
          mve->x = vxin->x + mtra->px*mtra->ml/p; 
          mve->y = vxin->y + mtra->py*mtra->ml/p;
          mve->z = vxin->z + mtra->pz*mtra->ml/p; 
        }  
      }
   }

// for every primary vertex: generate coordinates (cr_ipoint) and transform
// coordinates of all vertices belonging to this event  
   for(int j = 1; j <= nmcme; j++) {
      int i1 = i2 + 1;
      ArtePointer<MVER> mver(i1); 
      if(!mver->flag ) {
         cr_ipoint(j,vertex,time_gen);
         if(j < nmcme) { 
            ArtePointer<MCME> mcme(j+1); 
            ArtePointer<MVER> j2 = mcme->mver;
            i2 = j2.index() -1; 
            
         }
         else   i2 = nmver ;
         for(int i=i1; i<=i2; i++){
            ArtePointer<MVER> mve(i);
            mve->x += vertex[0];
            mve->y += vertex[1];
            mve->z += vertex[2]; 
            if(i==i1) mve->flag = 1; 
         }       
      } 
   }

// loop over all existing tracks 
   for( ArteTableIterator<MTRA> mtra; mtra.valid(); mtra.next()) { 
      int i_track = mtra.index();       
      int nt = 0;
      if(mtra->flag == 2) continue;  // finish the loop if it is a resonance
      int i_vorig = mtra->orig;       // no resonance - find vertex origin
 L10:  
      if(i_vorig  >  0) {             // mother track is no resonance 
         ArtePointer<MVER> vorig(i_vorig); 
         if(kv[i_vorig-1] == 0) {     // vertex not yet translated to gean
            vertex[0] = vorig->x;
            vertex[1] = vorig->y;
            vertex[2] = vorig->z + zgmoth_.ZZERO;
            ArtePointer<MTRA> trmoth = vorig->btra; //find vertex origin track
            int i_trmoth = trmoth.index();
            if(i_trmoth > 0) {     // the vertex vorig is not a primary vertex 
               ArtePointer<TOPP>  part_moth =  trmoth->topp;
               int ipart = part_moth->gean;                         
               if(ipart == 10 || ipart == 16) continue; //fin if mother is a K0
               ArtePointer<MVER> vxbef = trmoth->bver;
               int jvxbef =*(gcbank_.lq + gclink_.jvertx -1 - kv[vxbef.index()-1]);
               float time, px, py, pz, e;
               memcpy(&time,gcbank_.iq + jvxbef + 3,4);
               gsvert_(vertex,&kt[i_trmoth-1], &i0, &ubuf, &i0 , &nvx);
               int jtr_moth = *(gcbank_.lq + gclink_.jkine -1 -kt[i_trmoth-1]);
               memcpy(&px,gcbank_.iq + jtr_moth    ,4);
               memcpy(&py,gcbank_.iq + jtr_moth + 1,4);
               memcpy(&pz,gcbank_.iq + jtr_moth + 2,4);
               memcpy(&e ,gcbank_.iq + jtr_moth + 3,4);
               float beta = float(sqrt(px*px+py*py+pz*pz)/e);
               time +=  trmoth->leng/beta/AR_CLIGHT;
               int jv = *(gcbank_.lq + gclink_.jvertx -1 - nvx);
               memcpy(gcbank_.iq + jv + 3,&time,4);
            }
            else {                          // it is a primary vertex 
               gsvert_(vertex,&i0,&i0, &ubuf, &i0 , &nvx );
               int jv = *(gcbank_.lq + gclink_.jvertx -1 - nvx);
               float jx = time_gen; 
               memcpy(gcbank_.iq + jv + 3,&jx,4);
            }

            kv[i_vorig-1] = nvx;             // fill vertex rel arte - gean
            ZGKINE.mvpush(nvx,i_vorig);      // fill vertex rel gean - arte
         }
         else    nvx = kv[i_vorig-1];       // vertex was already translated
      }
      else {
         ArtePointer<MTRA> trmoth(-i_vorig); // mother track is a resonance, 
         i_vorig = trmoth->orig;                 // go back to its vertex
         goto L10;   
      }

//    make an entry for geant
      ArtePointer<TOPP>  part_track = mtra->topp;
      int ipart = part_track->gean;                 // particle id to gean id
      if(ipart == 0) ipart = 48;
      if(ipart == 10 || ipart == 16)   mtra->flag = 0;
      gskine_(&mtra->px, &ipart, &nvx, &ubuf, &i0, &nt); // put track in gean
      if(!nt) {
         arteMsg << "hgkine: track booking error-track number = " << i_track
                 << " for ipart = " << ipart;
         arteMsg.error();
         continue;  
      }

//    store existing data about the track in the temporary vector
      hgmtra mt;
      mt.mtra = i_track;
      mt.nhit = 0;
      mt.len  = mtra->ml;
      mt.flag = mtra->flag;
      mt.ep = 0;
      kt[i_track-1] = nt;                  // fill track rel arte - gean
      ZGKINE.mtpush(nt,mt);            // fill track rel gean - arte   
    }                                  // end of track loop

   delete [] kv;           delete [] kt;          
   }
     

















