
/***********************************************************************
*
* MAG_ARTE
* written by Siegmund Nowak     04/02
*
* This routines adds some air walls to the Magnet (if only one magnet chamber)
*
*       called from cr_arte_user
*
* The transferred parameters are
* void sil_arte_(int nede,int nwal,char lvols[],int lcopy[],int nlev)
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteFlag.h"
#include "gean/ZGMOTH.h"

extern "C" {
  int  matnum_(char*,int);
}

void mag_arte() {

  if(arteflag_.geomvers > 99. || arteflag_.geomvers <= 2.01031) return;

  float wzmin[5],wzmax[5],wxmin[5],wxmax[5],wymin[5],wymax[5];
  float xcenter=0., ycenter=0.;
  int fullflag = 1;

  wzmax[0] = 270.;
  wzmin[1] = 270.1;
  int imat = matnum_("AIR",3);
  ArtePointer<GMAT> gm(imat);

  for( ArteTableIterator<NESL> gs; gs.valid(); gs.next()){
     if(!strncmp(gs->name,"MAGN",4)) { 
        xcenter = 0.5*(gs->x1 + gs->x2);
        ycenter = 0.5*(gs->y1 + gs->y2);
     } 
     else if(!strncmp(gs->name,"MC01",4)) wzmin[0] = gs->z2 + .1;

     else if(!strncmp(gs->name,"PC01",4)) {
       if(gs->nsl != 2) {
         fullflag = 0;
         break; 
       }
     }
     else if(!strncmp(gs->name,"MS10",4)) {
       if(gs->nsl != 2) {
         fullflag = 0;
         break; 
       }
       wzmax[4] = gs->z1 - .1;          
     }
     else if(!strncmp(gs->name,"PTL1",4)) {
       wzmax[1] = gs->z1 - .1; 
       wzmin[2] = gs->z2 + .1; 
     }
     else if(!strncmp(gs->name,"PTL2",4)) {
       wzmax[2] = gs->z1 - .1;
       wzmin[3] = gs->z2 + .1; 
     }
     else if(!strncmp(gs->name,"PTL3",4)) {
       wzmax[3] = gs->z1 - .1;
       wzmin[4] = gs->z2 + .1;
     }
  }
  if(!fullflag) return;

  float tgx = tan(0.250);
  float tgy = tan(0.160); 
  for(int i=0; i<5;i++) {
     wxmin[i] = xcenter - tgx*wzmax[i]; 
     wxmax[i] = xcenter + tgx*wzmax[i];
     wymin[i] = ycenter - tgy*wzmax[i];
     wymax[i] = ycenter + tgy*wzmax[i];
     struct NWAL nw;
     memset(&nw,0,sizeof(nw));
     nw.shp = 1;
     nw.xmin = xcenter - tgx*wzmax[i];
     nw.xmax = xcenter + tgx*wzmax[i];
     nw.ymin = ycenter - tgy*wzmax[i];
     nw.ymax = ycenter + tgy*wzmax[i];
     nw.zmin = wzmin[i];
     nw.zmax = wzmax[i];
     nw.rot[0] = 1.; nw.rot[4] = 1.; nw.rot[8] = 1.;
     nw.nsl = 1;
     float xms = (nw.zmax - nw.zmin) /gm->radl;  
     nw.ms = xms*pow(0.0136*(1.+ 0.038*log(double(xms))),2.);
     strncpy(nw.gvol,"    ",4);  
     ArteTable<NWAL>::push_back(nw);
     ArtePointer<NESL> NS(zgmoth_.MAGNSL);
     int nnwal = ArteTable<NWAL>::size(); 
     ArtePointer<NWAL> NW(nnwal);
     createRelation(NS,NW,&NESL::nwal);
     createRelation(gm, NW, &GMAT::nwal);
     if(NS->fwl == 0) NS->fwl = nnwal;   
     NS->nwl = NS->nwl + 1;
  } 
}

/* mag_arte_ */
