// ********************************************************************
//
// source:    otplates.C
//
// type:      initialization of OTR plates geometry
//
// Reads data from plates'csl'.dat file, where csl is superlayer name.
// Function gets two arrays with values of minimal z coordinate
// in each stereo layer for plus and minus parts;
// 2 variables zmin_all and zmax_all - first/last coordinates of all
// superlayer by z;
// zwal_mc04 - from this point (end of MC04 wall) iron rails for MC05
// are positioned.
// MC04.dat file has additional variable besides: y_ - shift by z from
// zstereo to the middle of aluminium plate or high voltage board. It's
// necessary to calculate absence y coordinates.
// z1,z2 coordinates (stored to coord[4] and coord[5] respectively)
// are not absolute values, they depends on zstereo in each stereo layer.
//
// created:   Sep-2001
//
// author:    Oxana Barsoukova
//
// changes:
//
// ********************************************************************

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "arte/ArteMessage.hh"
#include "gean/zgvers.hh"
#include "gean/ZGESL.h"
 
FILE *utfilp(const char*,const char*);

extern "C" void otplates_(char* csl, float zstereo_minus[3],
			  float zstereo_plus[3], float* zmin_all,
			  float* zmax_all, float* zwal_mc04,
			  bool* mod_exist, int len_csl)
{
  void crbox_(char*,char*,float*,float*,float*,float*,float*,float*);
  
  float coord[6];
  char material[5];
  char smod[5];
  char str[100], fname[20], dir[60];
  float y_;
  int half, layer, wall;

  float zmn_all = *zmin_all, zmx_all = *zmax_all, zwl_mc04 = *zwal_mc04;
  bool mod_ex = *mod_exist;
  
  strncpy(smod,csl,len_csl);
  smod[len_csl] = '\0';

  zgvers ZGVERS;
  strcpy(dir,ZGVERS.Get_vers_datafile(LGESL_OTR));
  sprintf(fname, "plates%s.dat", smod);
  
  
  FILE* f = utfilp(fname,dir);
  if (!f)
    {           
      arteMsg <<  "Ignore this mess if your geovers is older than 1.0701 ";
      arteMsg.warn();
      return;
    }

  for (int i = 0; i < 6; i++) coord[i] = 0.;

  while (!feof(f))
    {
      fgets(str, 100, f);
      if (str[0] == '*') continue;
      if (!strlen(str)) continue;
      if (str[0] == 'm')
	sscanf(str, "material %4s", material);
      else
	{
	  if (!strncmp(smod, "MC04", 4))
	    {
	      sscanf(str, "%f%f%f%f%f%f%d%d%f", &coord[0], &coord[1], &coord[2],
		     &coord[3], &coord[4], &coord[5], &half, &layer, &y_);
	      coord[2] += (zstereo_plus[layer-1]+y_)*tan(-0.16) + 0.25;
	      coord[3] += coord[2] + 0.25;
	    }
	  else if (!strncmp(smod, "MC05", 4))
	    {
	      sscanf(str, "%f%f%f%f%f%f%d", &coord[0], &coord[1],
		     &coord[2], &coord[3], &coord[4], &coord[5], &wall);
	      if (wall == 0)
		{
		  coord[4] += zmn_all;
		  coord[5] += zmn_all;
		}
	      else if (wall == 1)
		{
		  coord[4] += zmx_all;
		  coord[5] += zmx_all;
		}
	      else
		{
		  coord[4] += zwl_mc04;
		  coord[5] += zwl_mc04;
		}
	    }
	  else
	    sscanf(str, "%f%f%f%f%f%f%d%d", &coord[0], &coord[1],
		   &coord[2], &coord[3], &coord[4], &coord[5], &half, &layer);
	  
	  if (strncmp(smod, "MC05", 4))
	    {
	      if (half == -1)
		{
		  coord[4] += zstereo_minus[layer-1];
		  coord[5] += zstereo_minus[layer-1];
		}	  
	      else
		{
		  coord[4] += zstereo_plus[layer-1];
		  coord[5] += zstereo_plus[layer-1];
		}
	    }

	  if (mod_ex || (!mod_ex && !strncmp(smod, "MC05", 4) 
			 && !strncmp(material, "WTIM", 4)))
	    crbox_(material,smod,
		   &coord[0],&coord[1],&coord[2],&coord[3],&coord[4],&coord[5]);
	}
    }
  
  fclose(f);
}
