*CMZ :          05/09/96  20.21.36  by  Fedor Ratnikov
*CMZ :  1.00/01 06/03/95  00.11.26  by  Fedor Ratnikov
*CMZ :  1.00/00 10/01/95  12.40.04  by  Dominik Ressing
*-- Author :    Dominik Ressing   09/01/95
************************************************************************
*
* L1ECAPO
* written by Dominik Ressing, 09/01/95 20.56.35
*
* Returns in an array ot type L1POINT the real position of the most
* reasonable track in the ECAL. The most reasonable track is the one
* with the largest energy deposited in the central cell of the ECAL cluster
*
* Arguments  kind         description
*          (in,update,out)
*----------------------------------------------------------------------*
* MePo        in          Message pointer
*
* itrk         out          closest track
* x,y,z        out          point at center of shower position of ecal
*
************************************************************************
c
      SUBROUTINE L1ECAPO(MePo,itrk,x,y,z0)
c
      IMPLICIT NONE
#include "arte/Arte.fi"
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1mesdf.fi"
#include "l1si/l1clink.fi"
#include "l1si/rcfdef.fi"
#include "l1si/l1banks.fi"
#include "l1si/ydef.fi"
c
      REAL x,y,z0
      INTEGER MePo
c
      REAL sx,sy,x0,y0,Pa,Emax,Etra
      INTEGER NtrkMAX
      PARAMETER (NtrkMAX=100)
      INTEGER isl,iFirstPlate,nPlate,iPlate,
     &     iTraHit,j,Ntrk,Itrs(NtrkMAX),imimp,itypeparent,
     &     itra,iver,itype,itraparent,itrk,iX,iY,idet,ndim,iPoPo,
     &     idGEANT
      CHARACTER*4 cName1,cName2
      LOGICAL inited
      DATA inited /.FALSE./
c
#include "arte/Arte.ff"
#include "l1si/rcfunc.ff"
#include "l1si/l1mesdf.ff"
#include "l1si/ydef.ff"
c
      IF(.NOT.inited)    THEN
         DO isl=1,NGESL()    ! Loop over all detector ``superlayers''
            itype = IGESL_CMP(isl)    ! type of detector
            IF(itype.EQ.LGESL_ECA)   THEN ! is Calorimeter SL
               iFirstPlate = IGESL_FPL(isl) ! first ECAL module
               nPlate = IGESL_NPL(isl)  ! number of ECAL modules
               GOTO 100
            ENDIF
         ENDDO
 100     CONTINUE
         inited = .TRUE.
      ENDIF

c     --- try to match tracks:
      iPoPo = MePo+L1MEpoint(0)+L1POreg
      sx = T1MEstor(iPoPo+L1GEOINFsx)
      sy = T1MEstor(iPoPo+L1GEOINFsy)
      x0 = T1MEstor(iPoPo+L1GEOINFx0)+sx*0.5
      y0 = T1MEstor(iPoPo+L1GEOINFy0)+sy*0.5
      z0 = T1MEstor(iPoPo+L1GEOINFz)
c
c        --- Looking for ECAL pretrigger track
c
      DO iPlate=iFirstPlate,iFirstPlate+nPlate-1 !Loop over all EC sections
         IF(x0.GT.FGCAL_XMIN(iPlate).AND.
     &        x0.LT.FGCAL_XMAX(iPlate).AND.
     &        y0.GT.FGCAL_YMIN(iPlate).AND.
     &        y0.LT.FGCAL_YMAX(iPlate))  THEN ! Hitted module
            ndim = IGCAL_NCL(iPlate)
            iX = (x0-FGCAL_XMIN(iPlate))/
     &           (FGCAL_XMAX(iPlate)-
     &           FGCAL_XMIN(iPlate))*ndim+1
            iY = (y0-FGCAL_YMIN(iPlate))/
     &           (FGCAL_YMAX(iPlate)-
     &           FGCAL_YMIN(iPlate))*ndim+1
            idet = iPlate*100+(ndim*iX+iY)
            GOTO 200
         ENDIF
      ENDDO
 200  CONTINUE
      Ntrk = NtrkMAX
      Emax = 0.
      iTraHit = 0
      CALL L1GETR0(L1MEstor(MePo+L1MEpoint(0)+L1POtrk0),Ntrk,Itrs)
      DO j=1,Ntrk
         Etra = 0.
         DO imimp=1,NMCAL()
            IF(OMCAL_MTRA(imimp).EQ.Itrs(j).AND.
     &           IMCAL_GCAL(imimp).EQ.idet)  THEN
               Etra = Etra+FMCAL_DE(imimp) ! deposit energy
            ENDIF
         ENDDO
         IF(Etra.GT.Emax)    THEN
            iTraHit = Itrs(j)
            Emax = Etra
         ENDIF
      ENDDO
      IF(iTraHit.GT.0)    THEN
         CALL l1getrp(iTraHit,z0,x,y,Pa)
         IF (L1PARprnlev.gt.1 )   THEN
            IF(Pa.GT.0.)
     &           WRITE( L1LUNprn,900) iTraHit,x,y,z0,sx,sy,Pa
c
            itra = iTraHit
 300        CONTINUE
            iver = IMTRA_ORIG(itra)
            itype = OMTRA_TOPP(itra)
            IF(iver.GT.0)  THEN
             itraparent = OMVER_BTRA(iver)
            ELSEIF(iver.LT.0)  THEN
             itraparent = -iver ! not vertex but resonance
             iver = 0
            ELSE                ! iver=0
            ENDIF

            IF(itraparent.GT.0)    THEN
               itypeparent = OMTRA_TOPP(itraparent)
            ELSE
               itypeparent = 0
            ENDIF
            CALL L1CPART(itype,idGEANT,cName1)
            CALL L1CPART(itypeparent,idGEANT,cName2)
            IF(iver.GT.0)  THEN
             WRITE(L1LUNprn,901) cName1,itra,cName2,
     &            FMVER_X(iver),FMVER_Y(iver),FMVER_Z(iver),
     &            FMTRA_PX(itra),FMTRA_PY(itra),FMTRA_PZ(itra)
            ELSE
             WRITE(L1LUNprn,902) cName1,itra,cName2,
     &            FMTRA_PX(itra),FMTRA_PY(itra),FMTRA_PZ(itra)
            ENDIF
            IF(itraparent.GT.0)  THEN
               itra = itraparent
               GOTO 300
            ENDIF
         ENDIF
      ENDIF
      itrk = iTraHit
c
 900  FORMAT(1x,'L1ECAPO==> it,x,y,z,sx,sy,Pa:',i4,6f8.2)
 901  FORMAT(1x,'L1ECAPO==>',A4,'(',I4,')',' originates from ',A4,
     &     ' @xyz:',3f6.1,' with Pxyz:',3f6.1)
 902  FORMAT(1x,'L1ECAPO==>',A4,'(',I4,')',' originates from ',A4,
     &     ' @xyz:',18X,' with Pxyz:',3f6.1)
c
      RETURN
c
      END





