//**********************************************************************/
// Created: 26.07.2000,   Olya Igonkina   for   MARPLE package
//
// implementation of RcRpntPoint class
//
//**********************************************************************/
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"

#include "marple/marple.hh"
#include "marple/RcRpntPoint.hh"

RcRpntPoint::RcRpntPoint( const ArtePointer <RPNT>  rrpntLink)
  :  RcPoint(), fRpntLink(rrpntLink)  
{
  if( !(rrpntLink != 0 ) || 
      rrpntLink - *ArteTable<RPNT>::begin() >  (int)ArteTable<RPNT>::size() )
    { 
      arteMsg << " MARPLE: RPNT entry number " << rrpntLink 
	      << "does not exist";
      arteMsg.allp();
      arteMsg << "- empty object is created" ;
      arteMsg.allp();
      return;
    }

  setX(   rrpntLink->xf  );
  setY(   rrpntLink->yf  );
  setZ(   rrpntLink->zf  );
  settx(  rrpntLink->txf );
  setty(  rrpntLink->tyf );
  setpe(  rrpntLink->pf );

  setcov( rrpntLink->cvf );

  checkAndCorrectCov();
}

// returns relation with MTRA 
const std::vector<ArtePointer<MTRA> > RcRpntPoint::mtraList() const
{
  std::vector<ArtePointer<MTRA> > mList;
  if( ArteTable<MTRA>::size() == 0 ) return mList;
  if( fRpntLink->rseg.index() )
    {
      for( ArteRelationIterator<MTRA> rsMtra = fRpntLink->rseg->mtra.begin();
	   rsMtra != fRpntLink->rseg->mtra.end();
	   ++rsMtra )
	mList.push_back(*rsMtra);
    }else  if( fRpntLink->rtra.begin()  != fRpntLink->rtra.end() ){
      ArtePointer<RTRA> rtra = fRpntLink->rtra.begin();
      for( ArteRelationIterator<MTRA> rsMtra = rtra->mtra.begin();
	   rsMtra != rtra->mtra.end(); ++rsMtra )
	mList.push_back(*rsMtra);
    }
  
  return mList;
}

// returns relation with HITB 
const std::vector<ArtePointer<HITB> > RcRpntPoint::hitbList() const
{
  std::vector<ArtePointer<HITB> > hList;
  if( ArteTable<HITB>::size() == 0 ) return hList;
  if( fRpntLink->rseg.index() )
    {
      for( ArteRelationIterator<HITB> rsHitb = fRpntLink->rseg->hitb.begin();
	   rsHitb != fRpntLink->rseg->hitb.end();
	   ++rsHitb )
	hList.push_back(*rsHitb);
    }else  if( fRpntLink->rtra.begin()  != fRpntLink->rtra.end() ){
      ArtePointer<RTRA> rtra = fRpntLink->rtra.begin();
      for( ArteRelationIterator<HITB> rsHitb = rtra->hitb.begin();
	   rsHitb != rtra->hitb.end(); ++rsHitb )
	hList.push_back(*rsHitb);
    }

  return hList;
}
//------------------------------------------------------
// update RPNT bank by RcRpntPoint information

void RcRpntPoint::updateARTE( int & iErr ) const
{

  iErr = 0;
  ArtePointer<RPNT> rrpnt = link();

  rrpnt->xf  = x();
  rrpnt->yf  = y() ;
  rrpnt->zf  = z() ;
  rrpnt->txf = tx() ;
  rrpnt->tyf = ty();
  rrpnt->pf  = pe();
  for( int i=0; i<5; i++){
    for(int j=i; j<5; j++){
      rrpnt->cvf[c_matrix_pack[i][j]] = cov()[i][j]; 
      // start numbering from [0][0] as in struct RPNT
    }
  }
}

ostream & operator << (ostream & os, const RcRpntPoint &rp)
{
  //os << static_cast<RcPoint&>(rp) ;
  const RcPoint * ptPoint = &rp;
  os << *ptPoint;
  os << "            RPNT link: " << rp.link() << endl ;
  return os ;
}
