
// Boris Fominykh  6.05.2002
// member functions of class MuDbEffPixel

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MuDbEffPixel.h"

extern "C" 
{
  int MuNameDecode(char*,int*,int*,int*,int*,int*,int*);
  int MuSetMajor(int,int);
  int MuSetMinor(int,int);
}

//** Constructor */
MuDbEffPixel::MuDbEffPixel()
  :  MuDbTable::MuDbTable()
{
}

//** Destructor */
MuDbEffPixel::~MuDbEffPixel()
{
  Clear();
}

//## Methods

//** Initialization */
void MuDbEffPixel::Ini(int rormc)
{
  MuDbTable::setDBname("/MUON_NEW");

  RorMC = rormc;
  if(RorMC == 0)
  {
    MuDbTable::setName("MuonEffPix");
    MuDbTable::setTitle("MuonEffPix");
  }
  else
  {
    MuDbTable::setName("MuonGeoEffPix");
    MuDbTable::setTitle("MuonGeoEffPix");
  }


  MuDbTable::setDesc("UB:msl UB:lay UB:col1 UB:col2 UB:row1 UB:row2 F:eff");

  Valid = -1;
  for(int sl=0; sl<SlM; sl++)
  {
    for(int qu=0; qu<QuM; qu++)
    {
      Ncont[sl][qu] = 0;
    }  
  }  
  return;
}

void MuDbEffPixel::Clear()
{
  Valid = -1;
  for(int sl=0; sl<SlM; sl++)
  {
    for(int qu=0; qu<QuM; qu++)
    {
      if(Ncont[sl][qu] > 0) Cont[sl][qu].clear();
      Ncont[sl][qu] = 0;
    }  
  }  
  return;
}


//** Read the table as it's defined in kumac*/
int MuDbEffPixel::Read()
{
  int ret = 0;
  int check = 0;

// read the table from the file
// the case of measured efficiency
  if(RorMC == 0)
  {
    if(getenv("MUON_EFF_PIX") != NULL)
    {
      MuDbTable::setFile(getenv("MUON_EFF_PIX"));
      ret=MuDbEffPixel::ReadFile();
      check = 1;
    }
  }

// the case of MC efficiency
  if(RorMC == 1)
  {
    if(getenv("MUON_GEO_EFF_PIX") != NULL)
    {
      MuDbTable::setFile(getenv("MUON_GEO_EFF_PIX"));
      ret=MuDbEffPixel::ReadFile();
      check = 1;
    }
  }

  if(check == 0) ret=MuDbEffPixel::ReadDB();

  return ret;
}

 
//** Read the table from DB*/
int MuDbEffPixel::ReadDB()
{
  int ret=0;
  int nel=0;

// set major, minor
  int ipr = 0;
  int major = MuSetMajor(6+RorMC,ipr);
  int minor = MuSetMajor(6+RorMC,ipr);

  MuDbTable::setKTstatus(-2);
  MuDbTable::setMajor(major);
  MuDbTable::setMinor(minor);

// open DB
  ret=MuDbTable::Open(0);
  // cout << "MuDbEffPixel::ReadDB: db is open, ret= " << ret << endl;
  if(ret > 0) return ret;

// fetch the table
  ret=MuDbTable::Fetch();
  // cout << "MuDbEffPixel::ReadDB: db is fetched, ret= " << ret << " Size " << Size << endl;
  if(ret != 0)
  {
    MuDbTable::Close();
    return ret;
  }

// read the table fields 
  char* msl   = new char[Size];
  char* lay   = new char[Size];
  char* nx1   = new char[Size];
  char* nx2   = new char[Size];
  char* ny1   = new char[Size];
  char* ny2   = new char[Size];
  float* eff  = new float[Size];

  MuEffPixel con;

  ret=31;
  Nf=SkipTableField(X,1);
  nel = GetTableField(X,TABLEubyte,Size,msl);
  // cout << "MuDbEffPixel::ReadDB: field 1 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,2);
  nel = GetTableField(X,TABLEubyte,Size,lay);
  // cout << "MuDbEffPixel::ReadDB: field 2 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,3);
  nel = GetTableField(X,TABLEubyte,Size,ny1);
  // cout << "MuDbEffPixel::ReadDB: field 3 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,4);
  nel = GetTableField(X,TABLEubyte,Size,ny2);
  // cout << "MuDbEffPixel::ReadDB: field 4 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,5);
  nel = GetTableField(X,TABLEubyte,Size,nx1);
  // cout << "MuDbEffPixel::ReadDB: field 5 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,6);
  nel = GetTableField(X,TABLEubyte,Size,nx2);
  // cout << "MuDbEffPixel::ReadDB: field 6 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,7);
  nel = GetTableField(X,TABLEfloat,Size,eff);
  // cout << "MuDbEffPixel::ReadDB: field 7 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  // cout << "MuDbEffPixel::ReadDB: table is read, Size= " << Size << endl; 

// fill set of containers
  for(int i=0; i<Size; i++)
  {
    if(*(msl+i) > 0 && *(msl+i) < 5 && 
       *(lay+i) > 0 && *(lay+i) < 5)
    {
      con.IDnum = i;  
      con.sl    = *(msl+i);  
      con.qu    = *(lay+i);  
      con.nx1   = *(nx1+i);
      con.nx2   = *(nx2+i);
      con.ny1   = *(ny1+i);
      con.ny2   = *(ny2+i);
      con.key   = *(nx1+i) * 20 + *(ny1+i);
      con.eff   = *(eff+i);  
      con.Valid = 1;

      // con.Print();

      int isl  = con.sl  - 1;
      int iqu  = con.qu  - 1;
      Cont[isl][iqu].insert(con);
      Ncont[isl][iqu] += 1;
    }
  }

  Valid = 1;
  ret = 0;
  arteMsg << DBtable << " from " << DBname << " is initialized";
  arteMsg.warn();

labelexit:
  delete []msl;
  delete []lay;
  delete []nx1;
  delete []nx2;
  delete []ny1;
  delete []ny2;
  delete []eff;

// close DB;
  MuDbTable::Close();

  return ret;
}


//** Read the table from ASCII file*/
int MuDbEffPixel::ReadFile()
{
  FILE *hl = fopen(FileName, "r");
  if(!hl) 
  {
    arteMsg << "MuDbEffPixel::ReadFile - no input file " << FileName << endl;
    arteMsg.error();
    return 1;
  }

  char numb[11] = "0123456789";
  MuEffPixel A;
  int count=0;

// loops over lines: max 5000 lines
  int nline=0;
  char line[121];
  for(int il=0; il<5000; il++) 
  {
// read the line
    if(fgets(line,sizeof(line),hl) == NULL) 
    {
      fclose(hl);
      goto goodexit;

      // if(feof(hl)) goto goodexit;
      // else 
      // { 
      //   arteMsg << "MuDbEffPixel:::ReadFile -  error of file read";
      //   arteMsg.warn();
      //   return 2;
      // }
    }
    ++nline;

    // cout << nline << "  " << line << endl;

    char word[121];
    int  nx1  = 0;
    int  nx2  = 0;
    int  ny1  = 0;
    int  ny2  = 0;
    float eff = 0.;

// decord the line
    int nitem=sscanf(line,"%s col %d %d row %d %d eff %f",
                           word,&nx1,&nx2,&ny1,&ny2,&eff);

// pass  commented cards
    if(word[0]=='#' || word[0]=='*' || word[0]=='/') continue;

// end of file
    if(!strcmp(word,"END")) 
    {
      fclose(hl);
      goto goodexit;
    }

// pass  blank lines
    if(nitem != 6) continue;

// control 'Mx' and '.' in the first word
    if(word[0] == 'M' && word[1] == 'x' && word[3] == '.') 
    {
      // cout << nline << " " << count << "  "; 
      // cout << word << "  nx1 " << nx1 << " nx2 " << nx2;
      // cout << " ny1 " << ny1 << " ny2 " << ny2;
      // cout << " eff " << eff << endl;

      A.Ini();

      int msl  = 0;
      int qu   = 0;

// decode msl
      for(int i=0; i<10; i++) 
      {
        if(word[2] == numb[i])
        {
          msl = i;
          break;
        }
      }

      if(msl > 0 && msl < 5)
      {
// decode qu
        for(int i=0; i<10; i++) 
        {
          if(word[4] == numb[i])
          {
            qu = i;
            break;
          }
        }

	// cout << " msl " << msl << " qu " << qu << endl;

// fill container
        A.IDnum = count;
        count += 1; 
        A.sl   = msl;
        A.qu   = qu;
        A.nx1  = nx1;
        A.nx2  = nx2;
        A.ny1  = ny1;
        A.ny2  = ny2;
        A.key  = nx1*20 + ny1;

        A.eff  = eff;
        A.Valid = 1;

        // A.Print();

        Cont[msl-1][qu-1].insert(A);
        Ncont[msl-1][qu-1] += 1;
      } 
    } 
  } 
goodexit:
  if(count > 0) 
  {
    Valid = 1;
    arteMsg << DBtable << " initialized from file";
    arteMsg.warn();
    cout << FileName << endl;
    return 0;
  }
  else return 3;
}


//** Print the object content */
void MuDbEffPixel::Print()
{
// check print control in kumac
  if(getenv("MUON_EFF_PIX_PRINT") == NULL) return;
  if(strcmp(getenv("MUON_EFF_PIX_PRINT"),"YES")) return;

  char fname[20];
  if(RorMC == 0) strncpy(fname,"MuEffPix.out",19);
  if(RorMC == 1) strncpy(fname,"MuGeoEffPix.out",19);

  // cout << "MuDbEffPixel::Print is called, RorMC= " << RorMC;
  // cout << " file " << fname << endl;

  ofstream fout(fname);
  if (!fout) 
  {
    arteMsg << "MuDbEffPixel::Print - can not open file " << fname << " RorMC= " << RorMC;
    arteMsg.warn();
    return;
  }

  fout << "  " << endl;
  fout << "DataBase: " << DBname << " Table: " << DBtable;
  fout << " Title: <" << Title << ">" << endl;
  fout << "Desc: <" << Desc << ">" << endl;
  fout << "Size= " << Size;
  fout << " Major " << Major << " Minor " << Minor << endl;

// loops over sl, qu, 
  for(int sl=0; sl<SlM; sl++)
  { 
    for(int qu=0; qu<QuM; qu++)
    { 
      fout << " " << endl;
      fout << " Sl " << sl+1 << " Qu " << qu+1;
      fout << " Ncontainers= " << Ncont[sl][qu] << endl;
      if(Ncont[sl][qu] > 0)
      {
        int count =0;
        set<MuEffPixel>::iterator c1 = Cont[sl][qu].begin();
        set<MuEffPixel>::iterator c2 = Cont[sl][qu].end();
        for(set<MuEffPixel>::iterator i=c1; i != c2; i++)
        {
          count += 1;
          fout << "  " << count << "  IDnum " << (*i).IDnum << " sl " << (*i).sl;
          fout << " qu " << (*i).qu;
          fout << " nx " << (*i).nx1 << " " << (*i).nx2;
          fout << " ny " << (*i).ny1 << " " << (*i).ny2;
          fout << " eff " << (*i).eff;
          fout << " Valid " << (*i).Valid << endl;
          fout << " " << endl;
        }
      }
    }
  }
  return;
}


// Get the efficiency */
float MuDbEffPixel::GetEff(int sl,int qu,int nx,int ny)
{
  if(sl<0 || sl >SlM-1 || qu<0 || qu>QuM-1) 
  {
    arteMsg << "MuDbEffPixel::GetPar - wrong request sl " << sl << " qu " << qu;
    arteMsg.warn();
    return -1.;
  }
  if(Ncont[sl][qu] > 0)
  {

    set<MuEffPixel>::iterator c1 = Cont[sl][qu].begin();
    set<MuEffPixel>::iterator c2 = Cont[sl][qu].end();
    for(set<MuEffPixel>::iterator i=c1; i != c2; i++)
    {
      if(nx+1 >= (*i).nx1 && nx+1 <= (*i).nx2 && ny+1 >= (*i).ny1 && ny+1 <= (*i).ny2)
        return (*i).eff;
    }
  }
  arteMsg << "MuDbEffPixel::GetEff - NO eff for sl "<< sl << " qu " << qu << " nx " << nx << " ny " << ny;
  arteMsg.warn();
  return -1.;
}



