
// Boris Fominykh  6.05.2002
// member functions of class MuDbEffTube

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MuDbEffTube.h"

extern "C" 
{
  int MuNameDecode(char*,int*,int*,int*,int*,int*,int*);
  int MuSetMajor(int,int);
  int MuSetMinor(int,int);
}

//** Constructor */
MuDbEffTube::MuDbEffTube()
  :  MuDbTable::MuDbTable()
{
}

//** Destructor */
MuDbEffTube::~MuDbEffTube()
{
  Clear();
}

//## Methods

//** Initialization */
void MuDbEffTube::Ini(int rormc)
{
  MuDbTable::setDBname("/MUON_NEW");

  RorMC = rormc;
  if(RorMC == 0)
  {
    MuDbTable::setName("MuonEffTube");
    MuDbTable::setTitle("MuonEffTube");
  }
  else
  {
    MuDbTable::setName("MuonGeoEffTube");
    MuDbTable::setTitle("MuonGeoEffTube");
  }

  MuDbTable::setDesc("UB:msl UB:lay UB:ud UB:nx1 UB:nx2 UB:side1 UB:side2 UB:wire1 UB:wire2 F:eff");

  Valid = -1;
  for(int sl=0; sl<SlM; sl++)
  {
    for(int lay=0; lay<LayM; lay++)
    {
      for(int ud=0; ud<UdM; ud++)
      {
        Ncont[sl][lay][ud] = 0;
      }  
    }  
  }  
  return;
}

void MuDbEffTube::Clear()
{
  Valid = -1;
  for(int sl=0; sl<SlM; sl++)
  {
    for(int lay=0; lay<LayM; lay++)
    {
      for(int ud=0; ud<UdM; ud++)
      {
        if(Ncont[sl][lay][ud] > 0) Cont[sl][lay][ud].clear();
        Ncont[sl][lay][ud] = 0;
      }  
    }  
  }  
  return;
}


//** Read the table as it's defined in kumac*/
int MuDbEffTube::Read()
{
  int ret = 0;
  int check = 0;

// read the table from the file
// the case of measured efficiency
  if(RorMC == 0) 
  {
    if(getenv("MUON_EFF_TUBE") != NULL) 
    {
      MuDbTable::setFile(getenv("MUON_EFF_TUBE"));
      ret=MuDbEffTube::ReadFile();
      if(ret == 0) check=1;
    }
  }

// the case of MC efficiency
  if(RorMC == 1) 
  {
    if(getenv("MUON_GEO_EFF_TUBE") != NULL) 
    {
      MuDbTable::setFile(getenv("MUON_GEO_EFF_TUBE"));
      ret=MuDbEffTube::ReadFile();
      if(ret == 0) check=1;
    }
  }

  if(check == 0) ret=MuDbEffTube::ReadDB();

  return ret;
}

 
//** Read the table from DB*/
int MuDbEffTube::ReadDB()
{
  int ret=0;
  int nel=0;

// set major, minor
  int ipr = 0;
  int major = MuSetMajor(2+RorMC,ipr);
  int minor = MuSetMinor(2+RorMC,ipr);

  MuDbTable::setKTstatus(-2);
  MuDbTable::setMajor(major);
  MuDbTable::setMinor(minor);

// open DB
  ret=MuDbTable::Open(0);
  // cout << "MuDbEffTube::ReadDB: db is open, ret= " << ret << endl;
  if(ret > 0) return ret;

// fetch the table
  ret=MuDbTable::Fetch();
  // cout << "MuDbEffTube::ReadDB: db is fetched, ret= " << ret << " Size " << Size << endl;
  if(ret != 0)
  {
    MuDbTable::Close();
    return ret;
  }

// read the table fields 
  char* msl   = new char[Size];
  char* lay   = new char[Size];
  char* ud    = new char[Size];
  char* nx1   = new char[Size];
  char* nx2   = new char[Size];
  char* sd1   = new char[Size];
  char* sd2   = new char[Size];
  char* wir1  = new char[Size];
  char* wir2  = new char[Size];
  float* eff  = new float[Size];

  MuEffTube con;

  ret=31;
  Nf=SkipTableField(X,1);
  nel = GetTableField(X,TABLEubyte,Size,msl);
  // cout << "MuDbEffTube::ReadDB: field 1 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,2);
  nel = GetTableField(X,TABLEubyte,Size,lay);
  // cout << "MuDbEffTube::ReadDB: field 2 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,3);
  nel = GetTableField(X,TABLEubyte,Size,ud);
  // cout << "MuDbEffTube::ReadDB: field 3 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,4);
  nel = GetTableField(X,TABLEubyte,Size,nx1);
  // cout << "MuDbEffTube::ReadDB: field 4 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,5);
  nel = GetTableField(X,TABLEubyte,Size,nx2);
  // cout << "MuDbEffTube::ReadDB: field 5 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,6);
  nel = GetTableField(X,TABLEubyte,Size,sd1);
  // cout << "MuDbEffTube::ReadDB: field 6 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,7);
  nel = GetTableField(X,TABLEubyte,Size,sd2);
  // cout << "MuDbEffTube::ReadDB: field 7 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,8);
  nel = GetTableField(X,TABLEubyte,Size,wir1);
  // cout << "MuDbEffTube::ReadDB: field 8 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,9);
  nel = GetTableField(X,TABLEubyte,Size,wir2);
  // cout << "MuDbEffTube::ReadDB: field 9 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,10);
  nel = GetTableField(X,TABLEfloat,Size,eff);
  // cout << "MuDbEffTube::ReadDB: field 10 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  // cout << "MuDbEffTube::ReadDB: table is read, Size= " << Size << endl; 

// fill set of containers
  for(int i=0; i<Size; i++)
  {
    if(*(msl+i) > 0 && *(msl+i) < 5 && 
       *(lay+i) > 0 && *(lay+i) < 5 && *(ud+i) < 2)
    {
      con.IDnum = i;  
      con.sl    = *(msl+i);  
      con.lay   = *(lay+i);
      con.ud    = *(ud+i);  
      con.nx1   = *(nx1+i);
      con.nx2   = *(nx2+i);
      con.sd1   = *(sd1+i);
      con.sd2   = *(sd2+i);
      con.w1    = *(wir1+i);
      con.w2    = *(wir2+i);
      con.key   = *(nx1+i) * 16 + *(wir1+i);
      con.eff   = *(eff+i);
      con.Valid = 1;

      // con.Print();

      int isl  = con.sl  - 1;
      int ilay = con.lay - 1;
      int iud  = con.ud;
      Cont[isl][ilay][iud].insert(con);
      Ncont[isl][ilay][iud] += 1;
    }
  }

  Valid = 1;
  ret = 0;

  arteMsg << DBtable << " from " << DBname << " is initialized";
  arteMsg.warn();

labelexit:
  delete []msl;
  delete []lay;
  delete []ud;
  delete []nx1;
  delete []nx2;
  delete []sd1;
  delete []sd2;
  delete []wir1;
  delete []wir2;
  delete []eff;

// close DB;
  MuDbTable::Close();

  return ret;
}


//** Read the table from ASCII file*/
int MuDbEffTube::ReadFile()
{
  FILE *hl = fopen(FileName, "r");
  if(!hl) 
  {
    arteMsg << "MuDbEffTube::ReadFile - no input file " << FileName << endl;
    arteMsg.error();
    return 1;
  }

  char numb[11] = "0123456789";
  MuEffTube A;
  int count=0;

// loops over lines: max 5000 lines
  int nline=0;
  char line[121];
  for(int il=0; il<5000; il++) 
  {
// read the line
    if(fgets(line,sizeof(line),hl) == NULL) 
    {
      fclose(hl);
      goto goodexit;

      // if(feof(hl)) goto goodexit;
      // else 
      // { 
      //   arteMsg << "MuDbEffTube:::ReadFile -  error of file read";
      //   arteMsg.warn();
      //   return 2;
      // }
    }
    ++nline;

    // cout << nline << "  " << line << endl;

    char word[121];
    int  nx1  = 0;
    int  nx2  = 0;
    int  sd1  = 0;
    int  sd2  = 0;
    int  w1   = 0;
    int  w2   = 0;
    float eff = 0.;

// decord the line
    int nitem=sscanf(line,"%s nx %d %d side %d %d wire %d %d eff %f",
                           word,&nx1,&nx2,&sd1,&sd2,&w1,&w2,&eff);

// pass commented cards
    if(word[0]=='#' || word[0]=='*' || word[0]=='/') continue;

// end of file
    if(!strcmp(word,"END")) 
    {
      fclose(hl);
      goto goodexit;
    }

// pass blank lines
    if(nitem != 8) continue;

// control 'Mt' and '.' in the first word
    if(word[0] == 'M' && word[1] == 't' && word[3] == '.') 
    {
      // cout << nline << " " << count << "  "; 
      // cout << word << "  nx1 " << nx1 << " nx2 " << nx2;
      // cout << " sd1 " << sd1 << " sd2 " << sd2;
      // cout << " w1 " << w1 << " w2 " << w2 << " eff " << eff << endl;

      A.Ini();

      int msl  = 0;
      int lay  = 0;
      int ud   = 0;

// decode msl
      for(int i=0; i<10; i++) 
      {
        if(word[2] == numb[i])
        {
          msl = i;
          break;
        }
      }

      if(msl > 0 && msl < 5)
      {
// decode lay and ud
        int xxx =0;
        for(int i=0; i<10; i++) 
        {
          if(word[4] == numb[i])
          {
            xxx = i;
            break;
          }
        }

// interpretation of word[4]
// MU1, MU2 - tube chambers
        if(msl < 3)
        {
          lay = (xxx+1)/2;
          ud  = xxx - (lay-1)*2 - 1;
        }      
// MU3, MU4 - pad chambers
        else
        {
          lay = 1;
          ud  = xxx - 1;
        }      

	// cout << " msl " << msl << " lay " << lay << " ud " << ud;
        // cout << " nx1 " << nx1 << " nx2 " << nx2;
        // cout << " sd1 " << sd1 << " sd2 " << sd2;
        // cout << " w1 " << w1 << " w2 " << w2 << endl;

// fill container
        A.IDnum = count;
        count += 1; 
        A.sl   = msl;
        A.lay  = lay;
        A.ud   = ud;
        A.nx1  = nx1;
        A.nx2  = nx2;
        A.sd1  = sd1;
        A.sd2  = sd2;
        A.w1   = w1;
        A.w2   = w2;
        A.key  = nx1*16 + w1;

        A.eff  = eff;
        A.Valid = 1;

        // A.Print();

        Cont[msl-1][lay-1][ud].insert(A);
        Ncont[msl-1][lay-1][ud] += 1;
      } 
    } 
  } 
goodexit:
  if(count > 0) 
  {
    Valid = 1;
    arteMsg << DBtable << " initialized from file";
    arteMsg.warn();
    cout  << FileName << endl;
    return 0;
  }
  else return 3;
}


//** Print the object content */
void MuDbEffTube::Print()
{
// check print control in kumac
  if(getenv("MUON_EFF_TUBE_PRINT") == NULL) return;
  if(strcmp(getenv("MUON_EFF_TUBE_PRINT"),"YES")) return;

  char fname[20];
  if(RorMC == 0) strncpy(fname,"MuEffTube.out",19);
  if(RorMC == 1) strncpy(fname,"MuGeoEffTube.out",19);

  // cout << "MuDbEffTube::Print() is called, RorMC= " << RorMC;
  // cout << " file " << fname << endl;

  ofstream fout(fname);
  if (!fout) 
  {
    arteMsg << "MuDbEffTube::Print - can not open file " << fname << " RorMC= " << RorMC;
    arteMsg.warn();
    return;
  }

  fout << "  " << endl;
  fout << "DataBase: " << DBname << " Table: " << DBtable;
  fout << " Title: <" << Title << ">" << endl;
  fout << "Desc: <" << Desc << ">" << endl;
  fout << "Size= " << Size;
  fout << " Major " << Major << " Minor " << Minor << endl;

// loops over sl, lay, ud, 
  for(int sl=0; sl<SlM; sl++)
  { 
    for(int lay=0; lay<LayM; lay++)
    { 
      if( sl < 2 || (sl > 1 && lay == 0))
      {
        for(int ud=0; ud<UdM; ud++)
        { 
          fout << " " << endl;
          fout << " Sl " << sl+1 << " Lay " << lay+1 << " Ud " << ud;
          fout << " Ncontainers= " << Ncont[sl][lay][ud] << endl;
          if(Ncont[sl][lay][ud] > 0)
          {
            int count =0;
            set<MuEffTube>::iterator c1 = Cont[sl][lay][ud].begin();
            set<MuEffTube>::iterator c2 = Cont[sl][lay][ud].end();
            for(set<MuEffTube>::iterator i=c1; i != c2; i++)
  	    {
              count += 1;
              fout << "  " << count << "  IDnum " << (*i).IDnum << " sl " << (*i).sl;
              fout << " lay " << (*i).lay << " ud " << (*i).ud;
              fout << " nx " << (*i).nx1 << " " << (*i).nx2;
              fout << " side " << (*i).sd1 << " " << (*i).sd2;
              fout << " wires " << (*i).w1 << " " << (*i).w2;
              fout << " eff " << (*i).eff;
              fout << " Valid " << (*i).Valid << endl;
              fout << " " << endl;
            }
          }
        }
      }
    }
  }
  return;
}


// Get the efficiency */
const float MuDbEffTube::GetEff(int iv, int wir)
{
  ArtePointer<GEDE> gede(iv);
// chamber name
  char name[10];
  strncpy(name,gede->hwc,9);
// decode the chamber name
  int type  = 0;
  int sl   = 0;
  int lay  = 0;
  int ud   = 0;
  int nx    = 0;
  int ncab  = 0;
  int ret=MuNameDecode(&name[0],&type,&sl,&lay,&ud,&nx,&ncab);
  if(ret != 0 || sl<1 || sl >SlM || lay<1 || lay>LayM || ud<0 || ud>UdM) 
  {
    arteMsg << "MuDbEffTube::GetPar - wrong request iv= " << iv;
    arteMsg.warn();
    return -1.;
  }

// define the side
  int sd=1;
// chamber name in previous gede
  ArtePointer<GEDE> prgd(iv-1);
  char prnm[10];
  strncpy(prnm,prgd->hwc,9);
  if(!strcmp(name,prnm)) sd=2;

  if(Ncont[sl-1][lay-1][ud] > 0)
  {

    set<MuEffTube>::iterator c1 = Cont[sl-1][lay-1][ud].begin();
    set<MuEffTube>::iterator c2 = Cont[sl-1][lay-1][ud].end();
    for(set<MuEffTube>::iterator i=c1; i != c2; i++)
    {
      if(nx >= (*i).nx1 && nx <= (*i).nx2 && sd >= (*i).sd1 && sd <= (*i).sd2 &&
         wir >= (*i).w1 && wir <= (*i).w2)  return (*i).eff;
    }
  }
  arteMsg << "MuDbEffTube::GetEff - NO eff for gede " << iv << " wir " << wir;
  arteMsg << " sl " << sl << " lay " << lay << " ud " << ud << " nx " << nx << " side " << sd;
  arteMsg.warn();
  return -1.;
}



