#ifndef MuDbEffTube_h
#define MuDbEffTube_h 1

//## Class: MuDbEffTube
//      /**Author:        Boris Fominykh
//
//      Date:             May 6th, 2002
//
//      Class:            MuDbEffTube
//
//      Base Class:       MuDbTable
//
//      Description:
//      This class is to read the tube chamber efficiencies from DB MUON_NEW*/

#include "murec/MuDbTable.h"
#include <set>
#include "murec/MuEffTube.h"

typedef unsigned short Word;
typedef unsigned int   LWord;

/////////////////////////////////////
static bool
operator<(const MuEffTube c1, const MuEffTube c2)
{
      return c1.key <= c2.key;
}
/////////////////////////////////////

class MuDbEffTube : public MuDbTable
{
  public:
    //## Constructor
           MuDbEffTube();
    //## Destructor
           ~MuDbEffTube();

    //public methods
      //        /** Initialization */
      void Ini(int rormc);
      //        /**Clear the object*/
      void Clear();
      //        /**Read the table as it's defined in kumac */
      int Read();
      //        /**Read the table from DB */
      int ReadDB();
      //        /**Read the table from ascii-file*/
      int ReadFile();
      //        /**Print the efficiencies collected in set */
      void Print();
      //        /**Get the efficiency, */
      const float GetEff(int gede, int nwir);

    //## Get and Set Operations for Class Attributes
      void MuDbEffTube::setRorMC(int value);

    int    Valid;
    int    RorMC;  // 0- real measured efficiency, 1- MC efficiency

  protected:

  private:

    static const Word SlM  = 4;
    static const Word LayM = 3;
    static const Word UdM  = 2;

  // number of collected containers
    Word Ncont[SlM][LayM][UdM];
  // set of containers
    set<MuEffTube> Cont[SlM][LayM][UdM];

  //        /**Add the next container to object*/
      int Add(MuEffTube* con);
};

//## Get and Set Operations for Class Attributes (inline)

inline void MuDbEffTube::setRorMC(int value)
{
  RorMC=value;
}

#endif

