/* Usage:

   call_via_istr_5par              (F f, (P1*)0, (P2*)0, (P3*)0, (P4*)0,
                                    (P5*)0); or
   call_via_istr_check_EOF_5par    (F f, (P1*)0, (P2*)0, (P3*)0, (P4*)0,
                                    (P5*)0); or
   call_via_istr_conditionally_5par(F f, (P1*)0, (P2*)0, (P3*)0, (P4*)0,
                                    (P5*)0, Condition c);

   where F f - some function object which takes 5 arguments and
   expression f(p1, p2, p3, p4, p5)
   (p1, p2, p3, p4, p5 are of types P1, P2, P3, P4, P5 respectively)
   is valid.
   If call_*_conditionally_5par(..) is used: Condition c is such an object
   that the expression c(is) is valid ("is" is of type istream*).

   call_via_istr*(...) returns an object of type CallViaIstr0_5Par<...>
   which has the operator()(istream* is, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5) =>
   it checks whether Condition "c" applied to "is" returns true, if yes:
   if ( c(is) ) => function f(p1, p2, p3, p4, p5) is called.
   In case of simple call_via_istr_5par(..) version condition is substituted
   by the object which always returns true, so that condition has no effect;
	 call_via_istr_check_EOF_5par(..) has a condition, which checks
   (*is) is empty.
*/

// Author V.Balagura (balagura@mail.desy.de)   (16.04.98).

#ifndef _call_via_istr_5par_hh
#define _call_via_istr_5par_hh 1

#include "ptset/call_via_istr.hh"

/* -------------------- 0 args -------------------- */
template <class F, class P1, class P2, class P3, class P4, class P5,
class Condition>
class CallViaIstr0_5Par {
public:
  F function;
  Condition condition;

  CallViaIstr0_5Par(F f, Condition c = Condition())
	 : function(f), condition(c) {}
  void operator()(istream* is, P1 p1, P2 p2, P3 p3, P4 p4, P5 p5) {
    if ( condition(is) ) function(p1, p2, p3, p4, p5);
  }
};
template <class F, class P1, class P2, class P3, class P4, class P5,
class Condition>
inline CallViaIstr0_5Par<F, P1, P2, P3, P4, P5, Condition>
call_via_istr_conditionally_5par(F f, P1*, P2*, P3*, P4*, P5*, Condition c) {
  return CallViaIstr0_5Par<F, P1, P2, P3, P4, P5, Condition>(f, c);
}

/* -------------------- call_via_istr_5par -------------------- */
/* No condition on istream is required */

template <class F, class P1, class P2, class P3, class P4, class P5>
inline CallViaIstr0_5Par<F, P1, P2, P3, P4, P5, _IstrOk>
call_via_istr_5par(F f, P1*, P2*, P3*, P4*, P5*) {
  return CallViaIstr0_5Par<F, P1, P2, P3, P4, P5, _IstrOk>(f, _IstrOk());
}

/* ----------------- call_via_istr_check_EOF_5par ----------------- */
/* Condition: istream is empty */

template <class F, class P1, class P2, class P3, class P4, class P5>
inline CallViaIstr0_5Par<F, P1, P2, P3, P4, P5, _CheckIstr_EOF>
call_via_istr_check_EOF_5par(F f, P1*, P2*, P3*, P4*, P5*) {
  return CallViaIstr0_5Par<F, P1, P2, P3, P4, P5, _CheckIstr_EOF>
	(f, _CheckIstr_EOF());
}

#endif
