// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modifications:
// - Introduction of ::rotMatrix call     14-Aug-00 Wouter Hulsbergen
//   inline float dedx()                             10/02/02 A.Spiridonov
//

// class rangerGede
// interface to GEDE table

#ifndef rangerGede_HH
#define rangerGede_HH

#ifndef __ARTE_HH__
#include "arte/arte.hh"
#endif
#ifndef rangerGeWa_HH
#include "rang/GeWa.hh"
#endif

class rangerGede : public rangerGeWa {

public:
  rangerGede();                      // default constructor
  rangerGede(ArtePointer<GEDE> ge);  // constructor from GEDE plane
  virtual ~rangerGede();
  inline const int plNumber(){return gede.index();}; // mangled id
  inline const ArtePointer<GEDE> getGede() const {return gede;} //return gede
  inline const ArtePointer<GWAL> getGwal() const {return 0;} //return gwal
  inline const int shp() const {return gede->shp;};  // shape
  inline float ms() const {return gede->ms;}   // mult scattering
  inline float dedx() const {return gede->dedx;}   // energy loss
  inline int cmp() const {return gede->cmp;}   // cmp
  inline const float cos(){return gede->cos;}; // cos of stereo angle
  inline const float sin(){return gede->sin;}; // sin of stereo angle
  inline const float rot(int i){return gede->rot[i-1];}; // rotation matrix  
  virtual const float* rotmatrix() const { return gede->rot ; }
  inline const float xmin(){return gede->xmin;}; // borders of total area
  inline const float xmax(){return gede->xmax;};
  inline const float ymin(){return gede->ymin;};
  inline const float ymax(){return gede->ymax;};
  inline const float sxmin(){return gede->sxmn;};// borders of sensitive area
  inline const float sxmax(){return gede->sxmx;};
  inline const float symin(){return gede->symn;};
  inline const float symax(){return gede->symx;};
  inline const float zmin(){return gede->zmin;};
  inline const float zmax(){return gede->zmax;};
  inline const float xmad(){return gede->xmad;};
  inline const float ymad(){return gede->ymad;};
  inline const float zmad(){return gede->zmad;};
  inline const float type(){return 1;};    // type (1=GEDE, 0=GWAL)
  inline const char gvol(const int i){return gede->gvol[i];}

private:
  ArtePointer<GEDE> gede;
};

#endif

