// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
// Modified:   
// - moved many routines to State.C
// - introduced accessors to covmatrix elements
//                                     14-Aug-00 Wouter Hulsbergen

// class rangerStateLI
// state class for linear tracking with interaction time

#ifndef rangerStateLI_HH
#define rangerStateLI_HH

#ifndef rangerState_HH
#include "rang/State.hh"
#endif
#ifndef rangerHitBase_HH
#include "rang/HitBase.hh"
#endif

class rangerStateLI : public rangerState {

public:
  rangerStateLI() : rangerState(5) {}
  //rangerStateLI(const rangerStateLI& s);
  virtual ~rangerStateLI() {}
  //   virtual rangerStateLI& operator=(const rangerState& s) { 
  //         cout << "rangerStateLI::operator=(const rangerState& s)" << endl ;
  // 	rangerState::operator=(s) ; return *this ; }
  //   rangerStateLI& operator=(const rangerStateLI& s) { rangerState::operator=(s) ; return *this ; }

  virtual rangerFitMode fitMode() const { return ModeLI; }
  virtual rangerState* clone() const { return new rangerStateLI(*this); } // produce copy

  virtual int predict(const double& zDestination,
	      const double& qPExp,rangerState*& targetState,
	      HepMatrix*& FTransport) const ;  // perform Kalman filter prediction
  virtual void project(HepVector& resid,HepMatrix& HH,rangerHitBase* hit) {
    hit->projectLI(resid,HH,this); } ;

public:
  
  virtual const double qP() const { return 0 ;}
  virtual const double I()  const { return p(5) ;}

  virtual void setQp(double qp) {}
  virtual void setI(double i)   { p(5) = i ; }

protected:

  virtual const double covXQp()  const { return 0; }
  virtual const double covYQp()  const { return 0; }
  virtual const double covTxQp() const { return 0; }
  virtual const double covTyQp() const { return 0; }
  virtual const double covQpQp() const { return 0; }
  virtual const double covQpI()  const { return 0; }

  virtual const double covXI()  const { return cov().fast(5,1) ; }
  virtual const double covYI()  const { return cov().fast(5,2) ; }
  virtual const double covTxI() const { return cov().fast(5,3) ; }
  virtual const double covTyI() const { return cov().fast(5,4) ; }
  virtual const double covII()  const { return cov().fast(5,5) ; }

  virtual void setCovXQp(const double v) {}
  virtual void setCovYQp(const double v) {}
  virtual void setCovTxQp(const double v){}
  virtual void setCovTyQp(const double v){}
  virtual void setCovQpQp(const double v){}
  virtual void setCovQpI(const double v) {}
  
  virtual void setCovXI(const double v)  { cov().fast(5,1) = v ; }
  virtual void setCovYI(const double v)  { cov().fast(5,2) = v ; }
  virtual void setCovTxI(const double v) { cov().fast(5,3) = v ; }
  virtual void setCovTyI(const double v) { cov().fast(5,4) = v ; }
  virtual void setCovII(const double v ) { cov().fast(5,5) = v ; }



};

#endif
