
// *****************************************************************************
//
// source:    rcClassJPSIMM.C
//
// type:      source code
//
// created:   20 Jun 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcClassJPSIMM
//
// called by: rcClassify
//
// purpose:   check for J/Psi -> mu+mu- event
//
// hints:     do NOT set flagdst here!
//            muonjpsi:     >= 0 0x1
//            SelectJPSIMM: >  0 0x2
//
// changes:
// 20 Jun 2000 (AG) creation
//  6 Jul 2000 (AG) added FORTRAN function muonjpsi_()
//  7 Jul 2000 (AG) renamed muonjpsi_() to rcmuonjpsi_()
//  6 Oct 2000 (AG) added rcSelectJPSIMM
// 14 Nov 2000 (AG) included Brian's MatchSLT()
// 15 Nov 2000 (AG) added debug printout
// 01 Dec 2000 (AG) removed initialization bug of mass
// 11 Dec 2000 (BP) Changed to use RTRA found by rcSelectJPSIMM
// 15 Dec 2000 (AG) polished mass printout
//
// *****************************************************************************

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "grover/grover.hh"

// SLT includes
#include "sltarte/matchslt.hh"

// externals
extern int   FADEB;
extern float massJPSIMM;

// prototypes
//extern "C" float rcmuonjpsi_();

//
// function code
//
int
rcClassJPSIMM()
{
  ARTE_mark( "rcClassJPSIMM 0", 0 );

  // inits
  int rc     = 0;
  massJPSIMM = 0; // reset mass
  //
  // invoke J/Psi -> mu+mu- finder (Vitaly Eiges)
  //
  //ARTE_mark( "rcmuonjpsi_", 0 );
  //massJPSIMM = rcmuonjpsi_();
  //if( massJPSIMM >= 0 ) rc |= 0x1;

  //
  // Brian's finder
  //
  //  ArtePointer<RTRA> track1, track2;
  //TrigType type;
  //float chi = MatchSLT( track1, track2, type );
  //if( type == PAIR_MU ) { // If it was a muon pair find the invariant mass with Grover
  //  rc = 0x1;
  //  GroverParticle particle1( track1 );  
  //  GroverParticle particle2( track2 );
  //  particle1.SetMass( 0.1056 );
  //  particle2.SetMass( 0.1056 );
  //  GroverVertex gvt( &particle1, &particle2 );
  //  gvt.Fit( -1.0 );
  //  massJPSIMM = gvt.Mass();
  //}

  // Loop over RTRA and find J/psi tracks which decay into mu+mu-

  for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end(); ++rtra ) {
    if ((*rtra)->topp!=0&&(*rtra)->ever!=0) {
      if ((*rtra)->topp->lund==443) { //J/psi
	ArteRelationIterator<RREL> rrel=(*rtra)->ever->rrel.begin();
	if ((*rrel)!=0) {
	  int partcode=(*rrel)->topp->lund;
	  if (abs(partcode)==13) { // first J/psi decay particle is muon
	    rc=1;
	    massJPSIMM=(*rtra)->ever->mass;
	  }
	}
      }
    }
  }
  // print
  if( FADEB ) cout << "JPSIMM: mass=" << massJPSIMM << endl;

  ARTE_mark( "rcClassJPSIMM 1", 0 );
  return rc;
}
