
// *****************************************************************************
//
// source:    rcClassRECYPS.C
//
// type:      source code
//
// created:   10 Nov 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcClassRECYPS
//
// called by: rcClassify
//
// purpose:   check invariant mass of di-lepton candidate
//
// hints:     do NOT set flagdst here!
//
// changes:
// 10 Nov 2000 (AG) creation
// 14 Nov 2000 (AG) polished printout
// 15 Nov 2000 (AG) added debug printout
// 01 Dec 2000 (AG) polished printout
// 08 Dec 2000 (AG) moved defaults to rcSelectJPSI
// 15 Dec 2000 (AG) polished printout
//
// *****************************************************************************

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

// externals
extern int   FADEB;
extern float cutRECYPS;

// globals
float massJPSIEE = 0;
float massJPSIMM = 0;

//
// function code
//
int
rcClassRECYPS()
{
  // show up
  ARTE_mark( "rcClassRECYPS 0", 0 );

  int rc = 0;

  // print
  if( FADEB ){
    cout << "RECYPS: cut=    " << cutRECYPS  << endl;
    cout << "RECYPS: ee-mass=" << massJPSIEE << endl;
    cout << "RECYPS: mm-mass=" << massJPSIMM << endl;
  }

  // e+ e- candidate
  if( massJPSIEE > cutRECYPS ){
    rc |= 0x1;
    cout << " RECYPS: Found e+e- candidate m=" << massJPSIEE << endl;
  }
  
  // mu+ mu- candidate
  if( massJPSIMM > cutRECYPS ){
    rc |= 0x2;
    cout << " RECYPS: Found mu+mu- candidate m=" << massJPSIMM << endl;
  }
  
  // show up
  ARTE_mark( "rcClassRECYPS 1", 0 );
  
  return rc;
}
