
// *****************************************************************************
//
// source:    
//
// type:      source code
//
// created:   26 Jul 2000
//	
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// changes:
// 26 Jul 2000 (AG) creation (code from Denis)
// 01 Aug 2000 (DD) correct ecal,mtra from rseg
// 30 Oct 2000 (DD) Killing of existing reconstruction segments
// 30 Nov 2000 (DD) slt seeded tracks 
//  6 Dec 2000 (DD) removed error messages
// 10 Mar 2001 (DD) Rsegc::slt added
// 22 Mar 2001 (DD) getVdsFromRtra added
//
// *****************************************************************************

#include "reco/rcXsegment.h"
#include "map.h"

ArteTable<RSEG>::iterator segment::ivds =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::iitr =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::iotr =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::itc  =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::irich=ArteTable<RSEG>::end();
ArteTable<RCCL>::iterator segment::iecal=ArteTable<RCCL>::end();
ArteTable<RSEG>::iterator segment::imuon=ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::ivds_rich =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::ivds_ecal =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::irich_ecal=ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::ivds_otr  =ArteTable<RSEG>::end();
ArteTable<RSEG>::iterator segment::ivds_itr  =ArteTable<RSEG>::end();
ArteTable<MTRA>::iterator segment::imtra  =ArteTable<MTRA>::end();
bool                      segment::islt=false;;
ArteTable<RCCL>::iterator segment::ibremse=ArteTable<RCCL>::end();
int                       segment::track_components=0;
int                       segment::covij[5][5]={{0,1,3,6,10},{1,2,4,7,11},{3,4,5,8,12},{6,7,8,9,13},{10,11,12,13,14}};


void segment::findSegments(ArtePointer<RTRA> t) {
  resetSegments();

  // -- find RSEG segment
  for (ArteRelationIterator<RSEG> irseg=t->rseg.begin();
       irseg!=t->rseg.end(); ++irseg) {
    resolveCmp(ArteTableIterator<RSEG>(*irseg));
  }

  // -- find ECAL cluster
  if (t->rccl.begin() != t->rccl.end()) { 
    iecal=ArteTableIterator<RCCL>(*t->rccl.begin()); 
    track_components=track_components|Rsegc::ecal; 
  }

  // -- check if used in triggering
  if (t->cmp&Rsegc::slt) islt=true;

  // -- find MTRA
  if (t->mtra.begin() != t->mtra.end()) { 
    imtra=ArteTableIterator<MTRA>(*t->mtra.begin()); 
  }
}


void segment::findSegments(ArtePointer<RSEG> mt) {
  resetSegments();
  // -- find RSEG
  resolveCmp(ArteTableIterator<RSEG>(mt));
  if (mt->fit!=7) return;
  for (ArteRelationIterator<RSEG>  jrseg=mt->rend.begin(); 
       jrseg!=mt->rend.end(); ++jrseg) {
    resolveCmp(ArteTableIterator<RSEG>(*jrseg));
  }
  // -- find ECAL
  if (mt->rccl.begin()!=mt->rccl.end()) {
    iecal=ArteTableIterator<RCCL>(*mt->rccl.begin());
    track_components=track_components|Rsegc::ecal;
  }
  // -- find MTRA
  if (mt->mtra.begin()!=mt->mtra.end()) {
    imtra=ArteTableIterator<MTRA>(*mt->mtra.begin()); 
  }
}


void segment::resolveCmp(ArteTable<RSEG>::iterator irseg) {
  if ((*irseg)->cmp&Rsegc::bitsign) return;

  // -- pure segments
  if ((*irseg)->fit!=7) {
    switch ((*irseg)->cmp) {
    case (Rsegc::vxd)                          :
    case (Rsegc::vxd|Rsegc::bit16)             : 
      ivds = irseg; break;

    case (Rsegc::msg|Rsegc::sil|Rsegc::trit)              : 
    case (Rsegc::msg|Rsegc::trit)                         :
    case (Rsegc::msg|Rsegc::patt|Rsegc::trit)             : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::trit|Rsegc::magt) : 
      itc = irseg; 
    case (Rsegc::msg|Rsegc::patt)                         :
    case (Rsegc::msg|Rsegc::patt|Rsegc::magt)             : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::sil)              :
    case (Rsegc::msg|Rsegc::patt|Rsegc::sil|Rsegc::magt)  :
      iitr = irseg; break;

    case (Rsegc::wch|Rsegc::trit)                         :
    case (Rsegc::wch|Rsegc::trit|Rsegc::slt)              :
    case (Rsegc::wch|Rsegc::patt|Rsegc::trit)             :
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::trit) : 
      itc = irseg;
    case (Rsegc::wch|Rsegc::magt)                         :
    case (Rsegc::wch|Rsegc::patt)                         : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::slt)              : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt)             : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::sil)  : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::slt)  : 
      iotr = irseg; break;

    case (Rsegc::msg|Rsegc::wch|Rsegc::trit)                       :  
      itc = irseg;
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt)                       :   
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::magt)           :
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::sil):
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::slt):
      iotr = irseg; iitr = irseg; break;

    case (Rsegc::rich)                         : irich = irseg; break;

    case (Rsegc::muon)                         : imuon = irseg; break;
    default: 
    //  cout<<"rcXsegment::findSegments: ERROR segment not found "<<hex<<(*irseg)->cmp<<dec<<endl;
      break;
    }
  }
  else {
    // -- matched segment
    switch ((*irseg)->cmp) {

    case (Rsegc::rich|Rsegc::ecal)             : irich_ecal = irseg; break;

    case (Rsegc::rich|Rsegc::vxd)              : 
    case (Rsegc::rich|Rsegc::vxd|Rsegc::bit16) : ivds_rich  = irseg; break;

    case (Rsegc::ecal|Rsegc::vxd)              :
    case (Rsegc::ecal|Rsegc::vxd|Rsegc::bit16) : ivds_ecal  = irseg; break;

    case (Rsegc::wch|Rsegc::patt|Rsegc::vxd)                           : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::vxd|Rsegc::bit16)              : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::vxd)               : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::vxd|Rsegc::bit16)  : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::sil|Rsegc::vxd)    : 
    case (Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::sil|Rsegc::vxd|Rsegc::bit16):
      ivds_otr = irseg;  break;

    case (Rsegc::msg|Rsegc::patt|Rsegc::vxd)                           : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::vxd|Rsegc::bit16)              : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::magt|Rsegc::vxd)               : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::magt|Rsegc::vxd|Rsegc::bit16)  : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::magt|Rsegc::sil|Rsegc::vxd)               : 
    case (Rsegc::msg|Rsegc::patt|Rsegc::magt|Rsegc::sil|Rsegc::vxd|Rsegc::bit16)  : 
      ivds_itr = irseg;  break;

    case (Rsegc::msg|Rsegc::wch|Rsegc::sil|Rsegc::patt|Rsegc::magt)    :
    case (Rsegc::msg|Rsegc::wch|Rsegc::sil|Rsegc::patt|Rsegc::magt|Rsegc::vxd|Rsegc::bit16):
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::vxd)    :
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::magt|Rsegc::vxd|Rsegc::bit16):
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::vxd)                :                           
    case (Rsegc::msg|Rsegc::wch|Rsegc::patt|Rsegc::vxd|Rsegc::bit16)   :                           
      ivds_itr = irseg;  ivds_otr = irseg;  break;

    default: 
    //  cout<<"rcXsegment::findSegments: ERROR segment not found "<<hex<<(*irseg)->cmp<<dec<<endl;
      break;
    }
  }
  track_components=(track_components|(*irseg)->cmp);
}


void segment::printSegments(ArtePointer<RTRA> t) {
  findSegments(t);
  cout<<t;
  if (vdsf()||t->hsi)      cout<<" vds ";
  if (itrf()||t->hin)      cout<<" itr ";
  if (otrf()||t->hou)      cout<<" otr ";
  if (tcf())               cout<<" tc ";
  if (richf()||t->cer>0)   cout<<" rich ";
  if (ecalf()||t->hca)     cout<<" ecal ";
  if (muonf()||t->hmu)     cout<<" muon ";
  if (vds_otrf())          cout<<" vds_otr ";
  if (vds_richf())         cout<<" vds_rich ";
  if (vds_ecalf())         cout<<" vds_ecal ";
  if (sltf())              cout<<" slt ";
  cout<<endl;
}

void segment::printSegments(ArtePointer<RSEG> s) {
  findSegments(s);
  cout<<s;
  printSegments();
}

void segment::printSegments() {
  cout<<" "<<hex<<track_components<<dec<<":";
  if (vdsf())  cout<<" vds ";
  if (itrf())  cout<<" itr ";
  if (otrf())  cout<<" otr ";
  if (tcf())   cout<<" tc ";
  if (richf()) cout<<" rich ";
  if (ecalf()) cout<<" ecal ";
  if (muonf()) cout<<" muon ";
  if (vds_otrf())  cout<<" vds_otr ";
  if (vds_richf()) cout<<" vds_rich ";
  if (vds_ecalf()) cout<<" vds_ecal ";
  if (sltf())      cout<<" slt ";
  cout<<endl;
}

void segment::resetSegments() {
  iecal=ibremse=ArteTable<RCCL>::end();
  irich=ivds=iotr=imuon=itc=iitr=ArteTable<RSEG>::end();
  ivds_rich=ivds_ecal=irich_ecal=ivds_otr=ivds_itr=ArteTable<RSEG>::end();
  imtra=ArteTable<MTRA>::end();
  track_components=0;
  islt=false;
}


int segment::TrackDiff(ArtePointer<RTRA> t1, ArtePointer<RTRA> t2) {
  if (t1==t2) return 1;

  int ret=0;
  ArtePointer<RVER> v1;
  int               v1f=0;
  if (t1->fit%100==2 || t1->fit%100==3)  { v1=t1->ever; v1f=1; }
  ArtePointer<RVER> v2;
  int               v2f=0;
  if (t2->fit%100==2 || t2->fit%100==3)  { v2=t2->ever; v2f=1; }

  if (v1f && !v2f) {
    for (ArteRelationIterator<RREL> irrel=v1->rrel.begin();
	 irrel!=v1->rrel.end(); ++irrel) {
      if ((ret=checkTrackDiff((*irrel)->rtra,t2))) return ret;
    }
  }

  else if (!v1f && v2f) {
    for (ArteRelationIterator<RREL> irrel=v2->rrel.begin();
	 irrel!=v2->rrel.end(); ++irrel) {
      if ((ret=checkTrackDiff(t1,(*irrel)->rtra))) return ret;
    }
  }

  else if (v1f && v2f) {
    for (ArteRelationIterator<RREL> irrel=v1->rrel.begin();
	 irrel!=v1->rrel.end(); ++irrel) {
      for (ArteRelationIterator<RREL> jrrel=v2->rrel.begin();
	   jrrel!=v2->rrel.end(); ++jrrel) {
	if ((ret=checkTrackDiff((*irrel)->rtra,(*jrrel)->rtra))) return ret;
      }
    }
  }

  else ret=checkTrackDiff(t1,t2);

  return ret;
}
 


int segment::checkTrackDiff(ArtePointer<RTRA> t1, ArtePointer<RTRA> t2) {
  int ret=0;
 
  // -- mini/dst - only rtra is available
  if (t1->csvd==t2->csvd&&t1->csvd>0)    ret=(ret|Rsegc::vxd);
  if (t1->cmain==t2->cmain&&t1->cmain>0) ret=(ret|Rsegc::wch);
  if (t1->crich==t2->crich&&t1->crich>0) ret=(ret|Rsegc::rich);
  if (t1->cecal==t2->cecal&&t1->cecal>0) ret=(ret|Rsegc::ecal);
  if (t1->cmuon==t2->cmuon&&t1->cmuon>0) ret=(ret|Rsegc::muon);
 
  if (ret) return ret;
 
  // -- if rseg table is available 
  findSegments(t1);
  int vds1 = (vdsf())  ? (*ivds).index() : 0;
  int otr1 = (otrf())  ? (*iotr).index() : 0;
  int rich1= (richf()) ? (*irich).index(): 0;
  int ecal1= (ecalf()) ? (*iecal).index(): 0;
  int muon1= (muonf()) ? (*imuon).index(): 0;
  
  findSegments(t2);
  int vds2 = (vdsf())  ? (*ivds).index() : 0;
  int otr2 = (otrf())  ? (*iotr).index() : 0;
  int rich2= (richf()) ? (*irich).index(): 0;
  int ecal2= (ecalf()) ? (*iecal).index(): 0;
  int muon2= (muonf()) ? (*imuon).index(): 0;

  if (vds1  && vds2  && vds1==vds2)     ret=(ret|Rsegc::vxd);
  if (otr1  && otr2  && otr1==otr2)     ret=(ret|Rsegc::wch);
  if (rich1 && rich2 && rich1==rich2)   ret=(ret|Rsegc::rich);
  if (ecal1 && ecal2 && ecal1==ecal2)   ret=(ret|Rsegc::ecal);
  if (muon1 && muon2 && muon1==muon2)   ret=(ret|Rsegc::muon);
 
  return ret;
}


int segment::TrackDiff(ArtePointer<RSEG> s1, ArtePointer<RTRA> t2) {
  int ret=0;

  // -- if rseg table is available 
  findSegments(s1);
  int vds1 = (vdsf())  ? (*ivds).index() : 0;
  int otr1 = (otrf())  ? (*iotr).index() : 0;
  int rich1= (richf()) ? (*irich).index(): 0;
  int ecal1= (ecalf()) ? (*iecal).index(): 0;
  int muon1= (muonf()) ? (*imuon).index(): 0;
  
  findSegments(t2);
  int vds2 = (vdsf())  ? (*ivds).index() : 0;
  int otr2 = (otrf())  ? (*iotr).index() : 0;
  int rich2= (richf()) ? (*irich).index(): 0;
  int ecal2= (ecalf()) ? (*iecal).index(): 0;
  int muon2= (muonf()) ? (*imuon).index(): 0;

  if (vds1  && vds2  && vds1==vds2)     ret=(ret|Rsegc::vxd);
  if (otr1  && otr2  && otr1==otr2)     ret=(ret|Rsegc::wch);
  if (rich1 && rich2 && rich1==rich2)   ret=(ret|Rsegc::rich);
  if (ecal1 && ecal2 && ecal1==ecal2)   ret=(ret|Rsegc::ecal);
  if (muon1 && muon2 && muon1==muon2)   ret=(ret|Rsegc::muon);

  return ret;
}


void  segment::killRTRA(int fitflag, int itopp) {
  ArtePointer<TOPP> topp;
  if (itopp) topp=ArtePointer<TOPP>(itopp);
  for (ArteTable<RTRA>::iterator irtra=ArteTable<RTRA>::begin();
       irtra!=ArteTable<RTRA>::end(); ++irtra) {
    if ((*irtra)->fit%100!=fitflag)    continue;
    if (itopp && (*irtra)->topp!=topp) continue;
    (*irtra)->fit=9999;
  }
}



void segment::makeParArray(ArtePointer<RTRA> rtra, float *par) {
  par[0]=rtra->xf;  par[1]=rtra->yf;  par[2]=rtra->zf;
  par[3]=rtra->txf; par[4]=rtra->tyf; par[5]=rtra->pf;
}

void segment::makeParArray(ArtePointer<RSLA> rsla, float *par) {
  par[0]=rsla->bx; par[1]=rsla->by; par[2]=0;
  par[3]=rsla->tx; par[4]=rsla->ty; par[5]=1/rsla->p;
}


void segment::makeDirArray(ArtePointer<RTRA> rtra, float *par) {
  par[2]=1/sqrt(rtra->txf*rtra->txf+rtra->tyf*rtra->tyf+1);
  par[0]=rtra->txf*par[2];
  par[1]=rtra->tyf*par[2];
}


void segment::getVdsFromRtra(vector<ArtePointer<RTRA> > &vds_list) {
  map<float, ArtePointer<RTRA>, less<float> > tmp_list;
  for (ArteTable<RTRA>::iterator irtra=ArteTable<RTRA>::begin();
       irtra!=ArteTable<RTRA>::end(); ++irtra) {
    if ((*irtra)->hsi<5) continue;
    int   dof=(*irtra)->hsi + (*irtra)->hin + (*irtra)->hou;
    float quality = dof - (*irtra)->chi/dof;
    tmp_list.insert( pair<float,ArtePointer<RTRA> >(quality,(*irtra)) );
  }

  for (map<float, ArtePointer<RTRA> >::iterator itmp=tmp_list.begin();
       itmp!=tmp_list.end(); ++itmp) {
    vector<ArtePointer<RTRA> >::iterator ivds=vds_list.begin();
    for ( ;ivds!=vds_list.end(); ++ivds) {
      if ( (*ivds)->csvd == itmp->second->csvd) break;
    }
    if (ivds==vds_list.end()) vds_list.push_back(itmp->second);
  }
}


int segment::nt_vds=0;
int segment::nt_main=0;
int segment::nt_rich=0;
int segment::nt_ecal=0;
int segment::nt_muon=0;
int segment::nt_vds_v0=0;
void segment::countTrackOccupancy() {
  int verb=0;

  // ... avoid double effort
  static int old_event=0;
  if (old_event==ArtePointer<EVHD>(1)->evt) return;
  old_event=ArtePointer<EVHD>(1)->evt;

  //cout<<"--------> new event <------------"<<endl;

  // ... count tracks
  vector<int> vds_list;
  vector<int> main_list;
  vector<int> rich_list;
  vector<int> ecal_list;
  vector<int> muon_list;
  for (ArteTable<RTRA>::iterator irtra=ArteTable<RTRA>::begin();
       irtra!=ArteTable<RTRA>::end(); ++irtra) {
    if ((*irtra)->fit%100!=0) continue;
    //cout<<"*";

    if ((*irtra)->csvd>0) {
      vector<int>::iterator ivds=vds_list.begin();
      for ( ; ivds!=vds_list.end(); ++ivds) if ((*ivds)==(*irtra)->csvd) break;
      if (ivds==vds_list.end()) { vds_list.push_back((*irtra)->csvd); /*cout<<" vds";*/ }
    }

    if ((*irtra)->cmain>0) {
      vector<int>::iterator imain=main_list.begin();
      for ( ; imain!=main_list.end(); ++imain) if ((*imain)==(*irtra)->cmain) break;
      if (imain==main_list.end()) { main_list.push_back((*irtra)->cmain); /*cout<<" main";*/ }
    }

    if ((*irtra)->crich>0) {
      vector<int>::iterator irich=rich_list.begin();
      for ( ; irich!=rich_list.end(); ++irich) if ((*irich)==(*irtra)->crich) break;
      if (irich==rich_list.end()) { rich_list.push_back((*irtra)->crich); /*cout<<" rich";*/ }
    }

    if ((*irtra)->cecal>0) {
      vector<int>::iterator iecal=ecal_list.begin();
      for ( ; iecal!=ecal_list.end(); ++iecal) if ((*iecal)==(*irtra)->cecal) break;
      if (iecal==ecal_list.end()) { ecal_list.push_back((*irtra)->cecal); /*cout<<" ecal";*/ }
    }

    if ((*irtra)->cmuon>0) {
      vector<int>::iterator imuon=muon_list.begin();
      for ( ; imuon!=muon_list.end(); ++imuon) if ((*imuon)==(*irtra)->cmuon) break;
      if (imuon==muon_list.end()) { muon_list.push_back((*irtra)->cmuon); /*cout<<" moun";*/ }
    }
    //cout<<endl;
  }
  nt_vds =vds_list.size();
  nt_main=main_list.size();
  nt_rich=rich_list.size();
  nt_ecal=ecal_list.size();
  nt_muon=muon_list.size();

  // ... count V0 tracks
  vector<int> vds_v0_list;
  for (ArteTable<RVER>::iterator irver=ArteTable<RVER>::begin();
       irver!=ArteTable<RVER>::end(); ++irver) {
    if ((*irver)->wire<1 || (*irver)->wire>8) continue;
    for (ArteRelationIterator<RREL> irrel=(*irver)->rrel.begin();
	 irrel!=(*irver)->rrel.end(); ++irrel) {
      ArtePointer<RTRA> rtra=(*irrel)->rtra;
      if (rtra->csvd<1) continue;
      vector<int>::iterator iv0=vds_v0_list.begin();
      for ( ; iv0!=vds_v0_list.end(); ++iv0) if ((*iv0)==rtra->csvd) break;
      if (iv0==vds_v0_list.end()) vds_v0_list.push_back(rtra->csvd);
    }
  }
  nt_vds_v0=vds_v0_list.size();
  if (verb) { 
    cout<<"vds="; for (vector<int>::iterator ivds=vds_list.begin(); ivds!=vds_list.end(); ++ivds) cout<<" "<<(*ivds); cout<<endl;
    cout<<"main="; for (vector<int>::iterator imain=main_list.begin(); imain!=main_list.end(); ++imain) cout<<" "<<(*imain); cout<<endl;
    cout<<"rich="; for (vector<int>::iterator irich=rich_list.begin(); irich!=rich_list.end(); ++irich) cout<<" "<<(*irich); cout<<endl;
    cout<<"ecal="; for (vector<int>::iterator iecal=ecal_list.begin(); iecal!=ecal_list.end(); ++iecal) cout<<" "<<(*iecal); cout<<endl;
    cout<<"muon="; for (vector<int>::iterator imuon=muon_list.begin(); imuon!=muon_list.end(); ++imuon) cout<<" "<<(*imuon); cout<<endl;
    cout<<"v0=";   for (vector<int>::iterator iv0=vds_v0_list.begin(); iv0!=vds_v0_list.end(); ++iv0) cout<<" "<<(*iv0); cout<<endl;
  }
  if (verb) 
    cout<<"NT: vds="<<nt_vds
	<<"  main="<<nt_main
	<<"  rich="<<nt_rich
	<<"  ecal="<<nt_ecal
	<<"  muon="<<nt_muon
	<<"  v0="<<nt_vds_v0<<endl;

}
