#include "multimap.h"
#include "map.h"

#include <stdio.h>
#include <cmath>
#include <cstdio>
#include <vector>

#include <arte/arte.hh>
#include "silarte/silarte.hh"

#include "grr.h"
#include "aframelib.h"
#include "svdalign.h"

#include "Struct_Hitb.h"
#include "Struct_Gede.h"

#include "Class_GEDE.h"

// global variables
static multimap < int, Hitb_t, less<int> > hitbM; 

static GEDE_class *GEDEp = 0;

//
// =========================
// silicon alignment program 
// =========================
//
// We rely on RSEG tracks and HITB,RHIT hits being
// correctly filled into ARTE tables before
// (eg. by Holmes or CATS or ...)
//
int SvdAlignEvent()
{
  static int first = 1;

  char *routine = "SvdAlignEvent";

  // shall svdalign be used at all ?
  if ( !svdalign_iol.use_svdalign.value ) {
    if ( first )
      aframe_mess(routine,GRR_SINFO,"SVD alignment is disabled.");
    first = 0;
    return GRR_OK;
  }

  // ---------------
  // initializations
  // ---------------
  if ( !GEDEp ) {

    // get target spot(s)
    float *xtarget, *ytarget, *ztarget;
    int   ntarget = HolmesGetTargets( &xtarget, &ytarget, &ztarget );

    // read geometry information
    GEDEp = new GEDE_class();

    // initialize svdalign
    if ( svdalign_iol.level.value == 1 )
      TspotAlignInit ( GEDEp, xtarget, ytarget, ztarget, ntarget );
    else if ( svdalign_iol.level.value == 2 )
      Level2AlignInit( GEDEp, xtarget, ytarget, ztarget, ntarget );
  }

  // -------------
  // process event
  // -------------

  // clean hit map
  if( hitbM.size() )
    hitbM.erase(hitbM.begin(), hitbM.end());

  // read the hits
  for( ArteTable<HITB>::iterator iHITB = ArteTable<HITB>::begin();
       iHITB != ArteTable<HITB>::end(); iHITB++ ) {
    ArtePointer<HITB> ihit = *iHITB;

    if ( ihit->gede == 0 )              continue;

    Gede_t *GedeP = GEDEp->gedeIm[ ihit->gede.index() ];
    
    if ( GedeP == 0 )                   continue;
    if ( (ihit->q)*(GedeP->sign) < 0 )  continue; 
    //    if ( fabs(ihit->q) > 100 )          continue;

    // fill internal hitb structure
    Hitb_t hitb;
    hitb.u      = ihit->x;
    hitb.cog    = ( ihit->x - ihit->gede->xfw[0] ) / ihit->gede->pit;
    hitb.gede   = GedeP;
    hitb.charge = ihit->q;
    hitb.width  =  1;
    hitb.sn     =  1;
    hitb.flag   =  1;
    hitb.hitb   =  0;

    // now insert into hitbM map
    hitbM.insert( pair<const int, Hitb_t> (hitb.gede->gede, hitb) );
  }  

  // process event
  if ( svdalign_iol.level.value == 1 )
    TspotAlignCollect ( &hitbM );
  else if ( svdalign_iol.level.value == 2 )
    Level2AlignCollect( &hitbM );
  
  return GRR_OK;
}

//
//  After accumulating the events the actual alignment is called here
//
int SvdDoAlign()
{
  // is svdalign in use ?
  if ( !svdalign_iol.use_svdalign.value )
    return GRR_OK;

  // ----------------
  // do the alignment
  // ----------------
  if ( svdalign_iol.level.value == 1 )
    TspotDoAlignment();
  else if ( svdalign_iol.level.value == 2 )
    Level2DoAlignment();
 
  return GRR_OK;
}

//
//  Stop Alignment code
//
int SvdAlignStop()
{
  // is svdalign in use ?
  if ( !svdalign_iol.use_svdalign.value )
    return GRR_OK;

  // delete SEED class
  if ( svdalign_iol.level.value == 1 )
    TspotAlignStop();
  else if ( svdalign_iol.level.value == 2 )
    Level2AlignStop();

  // delete GEDE class
  delete GEDEp;
  GEDEp = 0;

  return GRR_OK;
}
