//----------------------------------------------------------------------------
//
//  Subroutine to fill HITB Table from Marvin_Sevt_t structure
//  ========================================================
//
//  Based on: fillHITBfromSPARSEV.C by Rainer Wanke / 3-Oct-1998
// ---------------------------------------------------------------------------
// 
//  Returns:   GRR_OK    in case of no problems
//             GRR_FISHY otherwise
// 
// ---------------------------------------------------------------------------
//
// 23.01.2001 - (M.Baeuer) TEST-modification on fillHITBfromSPARSEV.C : single event-crunching introduced
// 26.06.2001 - (G. Sciacca) reads Helix mask maps from DBS by calling silarte_masking();
// 01.07.2001 - (G. Sciacca) can read Helix mask maps both from DBS and from FILE
// 02.07.2001 - (G. Sciacca) noaw a stand-alone routine renamed to fillHITBfromSPARSEVTmask.C
// 20.09.2001 - (G. Sciacca) print n. of bad chips to stdout only if event is corrupt
// 20.01.2002 - (DG) little memory leak fixed
//----------------------------------------------------------------------------

// include files
#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <math.h>

// Arte banks,  C++ version
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "silarte/silarte.hh"
#include "silarte/EVinfo.hh"

#include "aframelib.h"
#include "vdsleda.h"
#include "marvin.h"
#include "grr.h"

// -------------------------- From here the DQ defs are following:
// single event-crunching: 
// ATTENTION: These guys need the 
//   median and robust-means even though those ARE NOT USED 
//   internally in this application!
#include "silarte/Class_HCheck.h"
#include "silarte/Class_HCUT.h"

// Definition of charge-band around ZERO for rejecting:
#define INVALIDCHARGE  5.0

// Uncomment below to loose everything:
//#define MAXBADCHIPS   4000  /* Max no. of bad chips per event         */
//#define MAXRIGHTHITS  1500  /* Max no. of right-sign hits/chip/event  */
//#define MAXWRONGHITS   500  /* Max no. of right-sign hits/chip/event  */

// strictest but only working cut up to now !!!
#define MAXBADCHIPS   40  /* Max no. of bad chips per event         */
#define MAXRIGHTHITS  15  /* Max no. of right-sign hits/chip/event  */
#define MAXWRONGHITS   5  /* Max no. of right-sign hits/chip/event  */
#define MINHITSEVENT   1  /* Min no. of hits/chip/event to evaluate */

// a bit looser 
//#define MAXBADCHIPS   40  /* Max no. of bad chips per event         */
//#define MAXRIGHTHITS  15  /* Max no. of right-sign hits/chip/event  */
//#define MAXWRONGHITS  15  /* Max no. of right-sign hits/chip/event  */

// a cut file can exclude specific HELIXES wich are always bad
#define CUTFILE "cuts.txt" //it works when-> setenv VDS_MASKING FILE

// ---------------------- End of DQ-defs. 

int fillHITBfromSPARSEVTmask()
{
  char *routine = "fillHITBfromSPARSEVTmask";

  ArtePointer<GEDE> oldgede = 0;

  //int silarte_masking(char *chips);

  static bool  firsttime  = true;
  static bool afirsttime  = true;
  static bool bfirsttime  = true;
  static bool cfirsttime  = true;
  static bool xfirsttime  = true;

  static bool iFirstCheck = true; // HCheck NOT initilaised yet!
  static HCheck_class *HCheckp = NULL; 
  static HCUT_class   *HCUTp   = NULL; 

  char chipstat[MAXPLANES][MAXCHIP];
  static char *message = 0;
  if (!message) message = new char[512];

  //--------------------------------------------------------------------------


  // First of all get event from marvin
  Marvin_Sevt_t *sevt = marvin_fetch_sevt();
  if ( !sevt ) {

    if(firsttime == true) {
      aframe_mess(routine,GRR_SERROR,
		  "NULL pointer to sparsified information!");
      firsttime = false;
    }

    return GRR_FISHY;
  }

  // check reasonable number of clusters in format 3
  Marvin_Header_t *header = marvin_sevtHeader( (char*) sevt->data );
  if ( header->format == 3 && header->ncl > 0x7FFF / 8 ) {

    if(afirsttime == true) {
      aframe_mess(routine,GRR_SERROR,
		  "Too many clusters (%d) in event --> skipping it!", header->ncl );
      afirsttime = false;
    }

    return GRR_FISHY;
  }

  // Initialise the HCheck && HCUT classes:
  if (iFirstCheck == true) {
    if (HCheckp != NULL) {
      aframe_mess(routine,GRR_SERROR,
		  "Helix-check class has initialisation problems!");
      return(GRR_FISHY);
    }

    // create object:
    HCheckp = new HCheck_class(NULL);
    //printf("address passed to HCUT = %p\n", &chipstat[0][0]);

    char * mask;
    mask  = getenv("VDS_MASKING");
    if (!strcmp(mask, "DBS")) {
      // retrieve helix masking map from Database
      //printf("address of chipstat = %p\n", &chipstat[0][0]);
      silarte_masking(&chipstat[0][0]);

/*     printf("# Contents of VDSchipmask retrieved from DBS:\n");
     printf("# -------------------------------------------------------------------\n");
     printf("#        UsId         SideID - Chip0 Chip1 Chip2 Chip3 Chip4 Chip5 Chip6 Chip7 Chip8 Chip9\n");
     printf("#\n"); 
     for (int j=0; j<MAXPLANES; j++) {
       printf("++usid[%i] = %i sideid = %i - %d %d %d %d %d %d %d %d %d %d \n",
            j,j,j,chipstat[j][0],
	    chipstat[j][1], chipstat[j][2], chipstat[j][3], chipstat[j][4],
	    chipstat[j][5], chipstat[j][6], chipstat[j][7], chipstat[j][8], chipstat[j][9]);
     }
*/
      HCUTp   = new HCUT_class(&chipstat[0][0],1);
    } else if (!strcmp(mask, "FILE")) {
      HCUTp   = new HCUT_class(CUTFILE);
    } else {
       sprintf(message,
          "%s: env variable VDS_MASKING not correctly set - MUST be <OFF> or <DBS> or <FILE>\n", routine);
       ARTE_exit(GRR_SFATAL, message, 1); //return 1
    };

    iFirstCheck = false; // stuff seems OK.

    // Init with parameters
    HCheckp->fseSetPars(MAXRIGHTHITS,MAXWRONGHITS,MINHITSEVENT);
    printf("\n%s init of HELIX-cuts to MaxRight=%d MaxWrong=%d MaxBadChips=%d\n\n",
	   routine,MAXRIGHTHITS,MAXWRONGHITS,MINHITSEVENT);
  } // if iFirstCheck == true

  // -------------------------------
  //  fill HITB and update GEDE bank
  // -------------------------------
  if ( silarte_iol.debug_mode.value )
    cout << "MC flag = " << CheckMC() << " -> filling HITB bank" << endl;

  // 1. loop over clusters to fill HITB bank
  Marvin_Cluster_t *cluster = marvin_sevtCluster( (char*) sevt->data, -1 );
  while ( cluster ) {
    // get GEDE id from database
    LedaPointer<class VDS_Side> side = VDS_Side->get->key( cluster->side );
    if ( !side ) {

      if(xfirsttime == true) {
	aframe_mess(routine,GRR_SERROR,
		    "SVD side %d not in database", cluster->side );
	xfirsttime = false;
      }

      cluster = marvin_sevtCluster(NULL, -1); // get next event
      continue;
    }
    int gederow = silarte_returnGEDE( side->slname, 
				      (int) side->lay, (int) side->sec );
    ArtePointer<GEDE> gede = ArtePointer<GEDE>( gederow );
    if ( gede == 0 ) {

      if(bfirsttime == true) {
	aframe_mess(routine,GRR_SERROR,
		    "GEDE row %d for side %d (SL=%s,lay=%d,sec=%d) is non-existent",
		    gederow, (int) side->id, side->slname, 
		    (int) side->lay, (int) side->sec );
	bfirsttime = false;
      }

      cluster = marvin_sevtCluster(NULL, -1); // get next event
      continue;
    }

    if ( gede->cmp != 1 ) {

      if(cfirsttime == true) {
	aframe_mess(routine,GRR_SERROR,
		    "GEDE row %d (side %d) is not an SVD GEDE table",
		    gederow, (int) side->id );
	cfirsttime = false;
      }

      cluster = marvin_sevtCluster(NULL, -1); // get next cluster
      continue;
    }

    // put in check:
    HCheckp->fseAccuHit(cluster->side,
			cluster->cog,
			((cluster->charge) * ((signed char) side->polarity)) );    
    // get next cluster
    cluster = marvin_sevtCluster(NULL, -1);
  } // of first loop

  // evaluate which chips are ok:
  int iBadHe = HCheckp->fseEvaluate(); // ---------------- EVAL ------
  if (iBadHe > MAXBADCHIPS) {
    printf("Marvins BAD-chips: %d\n",iBadHe);
  }

  // corrupt event? :
  if (iBadHe > MAXBADCHIPS) {
    return(GRR_OK);
  }

  // back to event-start:
  cluster  = marvin_sevtCluster( (char*) sevt->data, -1 );

  // seond loop over clusters to fill HITB bank  
  while ( cluster ) {

//   if ( cluster->charge  < -INVALIDCHARGE || cluster->charge  > INVALIDCHARGE)
//   printf("cl: s=%d cog=%10.4f width=%3d q=%10.4f var=%10.4f sn=%10.6f\n",
//	   cluster->side, cluster->cog,cluster->width,cluster->charge,
//	   cluster->variance, cluster->sn);
 
   // print cluster if in debug
   if ( silarte_iol.debug_mode.value )
      cout << " marvin cluster " << cluster->side
	   << ": side = "  << cluster->side
	   << ", x = "     << cluster->cog
	   << ", width = " << cluster->width
	   << ", q = "     << cluster->charge
	   << ", var = "   << cluster->variance
	   << endl;

    // get GEDE id from database
    LedaPointer<class VDS_Side> side = VDS_Side->get->key( cluster->side );
    if ( !side ) {
      cluster = marvin_sevtCluster(NULL, -1); // get next event
      continue;
    }
    int gederow = silarte_returnGEDE( side->slname, (int) side->lay, (int) side->sec );
    ArtePointer<GEDE> gede = ArtePointer<GEDE>( gederow );
    if ( gede == 0 ) {
      cluster = marvin_sevtCluster(NULL, -1); // get next event
      continue;
    }
    if ( gede->cmp != 1 ) {
      cluster = marvin_sevtCluster(NULL, -1); // get next cluster
      continue;
    }

    // Keep right sign hits only (usually p=-, n=+, but taken from database)
    if ( (float) ((signed char) side->polarity) * cluster->charge < 0. ) {
      cluster = marvin_sevtCluster(NULL, -1); // get next cluster
      continue;
    }

    // put away the cluster if chip not OK:
    if (( HCheckp->fseHitOkay(cluster->side,cluster->cog) < 1)  ||
	( HCUTp->fCutSide(cluster->side,cluster->cog)    <  1 )    ){
      cluster = marvin_sevtCluster(NULL, -1); // get next cluster
//    if  ( HCUTp->fCutSide(cluster->side,cluster->cog)    <  1 ) {
//      printf("********** CLUSTER on a BAD CHIP: REJECTING **********\n");
//    }
      continue;
    }

    // Sort out cluster with bad charge: (MBr, Jan2001)
    if ( cluster->charge > -INVALIDCHARGE && 
	 cluster->charge <  INVALIDCHARGE    ) {
      cluster = marvin_sevtCluster(NULL, -1); // get next cluster
      continue;
    }

    // cluster coordinates
    HITB hit;
    hit.z    = gede->zfw[0];
    hit.x    = cluster->cog * gede->pit + gede->xfw[0];
    hit.y    = cluster->sn;
    hit.q    = cluster->charge;
    hit.r    = silarte_iol.hit_resolution.value;
    hit.gede = gede;

    // append hit to HITB table
    ArtePointer<HITB> hitb = ArteTable<HITB>::push_back( hit );

    // update GEDE bank: hit1 only , if GEDEnr has changed,
    //                   hitl always (every additional hit overwrites it)
    //   !!! We rely on already ordered hits !!!
    if ( hitb->gede != oldgede ) {
      gede->hit1 = hitb;
      oldgede = hitb->gede;
    }
    gede->hitl = hitb;

    // get next cluster
    cluster = marvin_sevtCluster(NULL, -1);

  }

  return GRR_OK;
}
