// =================================================================
//
// silarte_updateGEDEeff_CnA.C
//
// G. Sciacca - MPI-K Heidelberg
//
// Called by: acReadVDS
// 
// Retrieve VDS efficencies from Database /MC_MASEF according to
// the CnA scheme and update efficiency entries in GEDE banks
//
// Return values:
//		0 => OK
//		1 => error
//
// DBS: default is /MC_MASEF - Can be changed by setenv DB_VDS_EFF
// RunN: def is 0 (ARTE def efficiencies) - Can be changed by setenv VDSrunnumber
//
// 06.12.01 - (GS) Derived from silarte_updateGEDEeff.C, adapted to CnA scheme
// =================================================================

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <math.h>
#include <ctype.h>
#include <sys/time.h>
#include "database/table.h"
#include "database/keytable.h"
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "grr.h"   
#include "silarte/silarte.hh"   

#define MAXPLANES 128
   
int vdsmc_usid_to_arteid( int );

int silarte_updateGEDEeff_CnA(char *dbserver, Tables y, char *tablename)
{
   int rc = 0;	//  OK
   int error = 0;
   char *routine = "silarte_updateGEDEeff_CnA";

   Tables x;
   int t;
   int tmpid;
   int sl, sec, lay;
   time_t stime;
   int major = 0;
   int minor = 0;

   extern int CnAkey;
   extern int RunNoToSimu;

   struct VDSeff_struct
   {       
     int usid;
     float eff;   
     float chipeff0;
     float chipeff1;
     float chipeff2;
     float chipeff3;
     float chipeff4;
     float chipeff5;
     float chipeff6;
     float chipeff7;
     float chipeff8;
     float chipeff9;
   };
      
   struct VDSeff_struct VDSefficiency[MAXPLANES];

   char *message = 0;
   if (!message) message = new char[512];

   cout<<""<<endl;
//   cout<<"######## silarte_updateGEDEeff_CnA development version ########"<<endl;

//   printf("extern int CnAkey: %i\n", CnAkey);
//   printf("extern int RunNoToSimu: %i\n", RunNoToSimu);

// ------------------------------------------------------------------
// Looking up and retrieving VDSefficncy
// ------------------------------------------------------------------
   if((t=BookTable(y,"VDSefficiency",0,0))==0) {
     sprintf(message,
	"%s: Error booking table VDSefficiency\n", routine);
     ARTE_exit(GRR_SFATAL, message, 1);	//return 1
   }
   cout<<"acReadVDS: reading VDSefficiency information"<<endl;
   if (KeyTableStamp(y,dbserver,tablename)) {   
     sprintf(message,"%s: table VDSefficiency not found in Keytable\n", routine);
     ARTE_exit(GRR_SFATAL, message, 1);       //return 1
   }

   // Retrieving tables
   if(FetchTable(y,t,TABLEeq)==0) {
//printf("TableMajor(y) = %i\n",TableMajor(y));
//printf("TableMinor(y) = %i\n",TableMinor(y));
     sprintf(message,
	"%s: Error fetching table VDSefficiency\n", routine);
     ARTE_exit(GRR_SFATAL, message, 1);	//return 1
   }
//printf("TableMajor(y) = %i\n",TableMajor(y));
//printf("TableMinor(y) = %i\n",TableMinor(y));

   int structsize = sizeof(struct VDSeff_struct);

   SkipTableField(y,1);
   rc = GetTableStruct(y,TABLEint,structsize,MAXPLANES,&VDSefficiency[0].usid);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i usid elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].eff);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i eff elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].chipeff0);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipeff0 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].chipeff1);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipeff1 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].chipeff2);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipeff2 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].chipeff3);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipeff3 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].chipeff4);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipeff4 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].chipeff5);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipeff5 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].chipeff6);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipeff6 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].chipeff7);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipeff7 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].chipeff8);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipeff8 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   rc = GetTableStruct(y,TABLEfloat,structsize,MAXPLANES,&VDSefficiency[0].chipeff9);
   if (rc != MAXPLANES) {
     sprintf(message,
	"%s: Error retrieving data: only %i chipeff9 elements retrieved, %i expected\n",
		 routine, rc, MAXPLANES);
     ARTE_exit(GRR_SFATAL, message, 1);		//return 1
   }
   
// --------------------------------------------------------
// Unbooking tables
// -------------------------------------------------------- 
   if(!UnbookTable(y,t)) {
     sprintf(message,
	"%s: Error unbooking table VDSefficiency\n", routine);
     ARTE_error(GRR_SERROR, message, 0);
   }
// --------------------------------------------------------
// Update efficiency entry in GEDE bank 
// --------------------------------------------------------
   sprintf(message,
	"%s: Loadind VDS efficiency entries on to GEDE bank", routine);
   cout<<message<<endl;

   // loop over efficiency entries retrieved from DB
   for (int i=0; i<MAXPLANES; i++) {

//if (i==5) cout<<"### plane 5 ###"<<endl;
     tmpid = vdsmc_usid_to_arteid( i );
//if (i==5) cout<<"### tmpid = "<<tmpid<<endl;
     if (tmpid == 0) continue;		// not mounted plane
     if (tmpid < 0) {
       sprintf(message,
	   "%s: %i is not a valid VDS plane\n", routine, tmpid);
       ARTE_error(GRR_SERROR, message, 0);
     }
     // extract sl=A, sec=B, lay=C from tmpid=ABC
     sl = int(floor(tmpid/100));
     tmpid -= sl*100; 
     sec = int(floor(tmpid/10));
     lay = tmpid-(10*sec);
//printf("sl=%i, sec=%i, lay=%i - usid = %i \n", sl, sec, lay,i);

     if (  sl  < 1 || sl  > 8
        && lay < 1 || lay > 4
        && sec < 1 || sec > 4 ) {
        sprintf(message,                   
		"sl=%d, sec=%d, lay=%d is NOT a valid VDS plane\n",
                 sl, sec, lay);
        ARTE_error(GRR_SERROR, message, 0);
     }
     // loop over ARTE GEDE table to get corresponding GEDE row
     int found = 0;

     for ( ArteTable<GEDE>::iterator igede = ArteTable<GEDE>::begin();
          igede != ArteTable<GEDE>::end(); igede++ ) {
       ArtePointer<GEDE> gede = *igede;  

       if (   gede->cmp == 1
          && gede->gesl->nsl == sl
          && gede->lay == lay
          && gede->sec == sec ) {	// VDS plane?

	  // update eff entry value
	  gede->eff = VDSefficiency[i].eff;

	  found = 1;

       } // no VDS plane

     }	// end loop over GEDE rows

    // complain in case no corresponding ARTE GEDE row was found
    if ( !found ) {   
      sprintf(message,     
		"VDS side %i (sl=%i, sec=%i, lay=%i) not found in GEDE table!\n",
                 i, sl, sec, lay);
      ARTE_error(GRR_SERROR, message, 0);
      error++;
    }
   } // end of loop over efficiency entries retrieved from DB

  if ( error ) {
    sprintf(message,
	"%s: *** WARNING *** VDS efficiencies NOT succesfully loaded on to GEDE:",
	 routine);
    cout<<message<<endl;
    sprintf(message,
	" %i GEDE rows not found!\n", error);
    cout<<message<<endl;
    return 1;
  }

  sprintf(message,
	"%s: VDS efficiencies succesfully loaded on to GEDE\n", routine);
  cout<<message<<endl;
  return GRR_OK;
}
