/*** 
 *
 *  Program to produce the L2 buffer map, into either 
 *       - CnA tables
 *       - Binary file  
 *       - ASCii file   
 *            
 *  Author: Peter Kreuzer
 *  Last modified: 
 *  November 20 2000: include ITR (PK)
 *  May 29 2000     : split from buffmap.c (PK)
 *  June 1 2000     : remove MakeLayerMap and MakeSectMap
 *                    The BufferMap is now built from the 
 *                    PLANE table (PK)
 *   
 *
 ***/ 

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>

#define __NO_SHARC__

#include "pltable.h" 
#include "buffmap.h"
#include "l2format.h"
#include "sltable.h"
#include "component.h"
#include "string.h"
#include "rpm.h"
#include "grr.h"
#include "pub.h"
#include "cif.h"
#include "vdsMAP.h"
#include "itrMAP.h"
#include "OtrSlt.h"
#include "database/keytable.h"
#include "errno.h"

typedef L2FR_BM_Entry Item;
static L2FR_BM_Entry l2fr_BMBuffMap[L2FR_BM_ENTRIES]; 
static int MakeBuffMap(int ** l2map,int * length);

static int Write_BuffMapAscii();
static int Write_BuffMapBinary(int *l2map, int size);
static void L2form_BuffMapPrintTable(FILE *fd);
static void L2form_BuffMapPrintEntry(L2FR_BM_Entry *entry, FILE *fd);
extern char Key[20];
extern int  nArg;
extern char **vArg;

  /*****************
   *
   * l2MapFormat:
   * Function called by the l2map_fmt handler  
   * to generated l2map formatted table for the SLP. 
   * Options to produce ASCii and/or BINARY file dumps 
   * are available via compilation flags DUMP_ASCII/BINARY.
   * The DUMP_NODE flag should be used ONLY online !
   *
   ***/
  
  int l2MapFormat(int ** l2map,int * length)
    {
      int *l2formmap;
      int size;
      char mess[200];
      char ModeName[10];
      int i;
        
      if(MakeBuffMap(&l2formmap,&size)!=0){
	sprintf(mess,"L2FormArte/l2MapFormat: MakeBuffMap failed !\n");
        grr_error(GRR_SFATAL,mess,1);
	return 1;
      }
      *l2map = l2formmap;
      *length = size;

      /* decode the SltDumpMode to be used for the L2tabs */
      for( i=1; i<nArg; i++ ){
	if( !strncmp( vArg[i], "-sltdumpmode:", 13 ) ){
	  strcpy(ModeName, vArg[i]+13);
	  sprintf(mess,"L2FormArte/l2MapFormat: sltdumpmode %s\n",ModeName);
	  grr_error(GRR_SINFO,mess,0); 
	} 
      }
      if (!strncmp(ModeName,"ASCII",5)){
	if (Write_BuffMapAscii()!=0) {
	  sprintf(mess,
		  "L2FormArte/l2MapFormat: problems to generate ASCii file (l2in.map)\n"); 
	  grr_error(GRR_SERROR,mess,1);
	} else {
	  sprintf(mess,"L2FormArte/l2MapFormat: ASCii file (l2in.map) generated\n"); 
	  grr_error(GRR_SINFO,mess,0);
	}
      } else if (!strncmp(ModeName,"BINARY",6)){
	if (Write_BuffMapBinary(l2formmap,size)!=0) {
	  sprintf(mess,
		  "L2FormArte/l2MapFormat: problems to generate Binary file (l2bin.map)\n"); 
	  grr_error(GRR_SERROR,mess,1);
	} else {
	  sprintf(mess,"L2FormArte/l2MapFormat: Binary file (l2bin.map) generated\n"); 
	  grr_error(GRR_SINFO,mess,0);
	}
      } else {
	sprintf(mess,"L2FormArte/l2MapFormat: NO l2map File will be dumped !!!\n");
	grr_error(GRR_SINFO,mess,0);
      }

      return 0;
    }

/***
 *
 * MakeBuffMap
 *
 * Makes buffer map either by reading it in as input file (online slt mode) or
 * by generating it. In latter case, mode L2FORMARTE should be specified in Makefile
 * if buffer map based on vds database and real scn.cfg should be produced.
 *
 ***/

static int MakeBuffMap(int ** l2map,int * length) {

  Item *item;
  int il;
  int isl, in, nvdsmap, nitrmap, notrmap;
  VDS_map *vdsmap;
  ITR_map *itrmap;
  otrsltMapEntry *otrmap;
  int nbuffentry=0;
  L2FORMMAPHEAD *header;
  int length_bm;
  char mess[200];
  char *wrPtr;
  int flag = 1;
  PLANE *ThePlane;
  SUPERLAYER *supl;

  for(il=0;il<L2FR_BM_ENTRIES;il++){
    item=&l2fr_BMBuffMap[il];
    item->lay=-1;
    item->cid=0xff;
    item->addr_send=-1;
    item->addr_range=-1;
    item->send_order=-1;
  }
    
  if(( itrmap=itrslt_fillMAP(&nitrmap))==NULL){
    sprintf(mess,"L2FormArte/MakeBuffMap: problem with itrslt_fillMAP \n");
    grr_error(GRR_SFATAL,mess,1);
    return 1;
  }
  if(( vdsmap=vdsslt_fillMAP(&nvdsmap))==NULL){
    sprintf(mess,"L2FormArte/MakeBuffMap: problem with vdsslt_fillMAP \n");
    grr_error(GRR_SFATAL,mess,1);
    return 1;
  }
  if(( otrmap=otrsltFillMap(&notrmap))==NULL){
    sprintf(mess,"L2FormArte/MakeBuffMap: problem with otrslt_fillMAP \n");
    grr_error(GRR_SFATAL,mess,1);
    return 1;
  }
  /* Now loop over the PLANE table and fill in Buffmap entries */
  for(ThePlane = Pl_IterateInit(NULL); ThePlane != NULL; ThePlane = Pl_IterateNext()) {
    supl=SLLookup_bp(ThePlane->cmp,ThePlane->l2npl);
    /* The SILICON section */
    if (strncmp(supl->name,"SI",2)==0) {
      for(in=0;in<nvdsmap;in++){
	if(vdsmap[in].wire1==0 && vdsmap[in].wire2==0) continue;
	item=&l2fr_BMBuffMap[nbuffentry];
	for(isl=0;isl<4;isl++){
	  item->spl[isl]=vdsmap[in].spl[isl];
	}
	item->lay=vdsmap[in].lay;
	item->sec=vdsmap[in].sec;
	item->mod=0;
	item->modtype=0;
	if(!strncmp(supl->name,item->spl,4)&&
	   (ThePlane->lay==item->lay)&&
	   (ThePlane->sec==item->sec)){
	  item->link=vdsmap[in].fedid;
	  item->chan1=0;
	  item->chan2=288;  /* hardwired. not so good. but not contained in database */
	  item->wire1=vdsmap[in].wire1;
	  item->wire2=vdsmap[in].wire2;
	  item->cid=COMP_N_SVD;
	  item->chip_pos=vdsmap[in].sharc;
	  item->vme_id=vdsmap[in].vmeid;
	  item->vme_addr=vdsmap[in].board;
	  nbuffentry++;
	  if(nbuffentry>=L2FR_BM_ENTRIES) {
	    sprintf(mess,
		    "L2FormArte/MakeBuffMap(VDS): not enough space in buffer map array\n");
	    grr_error(GRR_SFATAL,mess,1);
	    return 1;
	  }
	}
      }
    }
    /* The ITR section */
    if (strncmp(supl->name,"MS",2)==0) {
      for(in=0;in<nitrmap;in++){
	if(itrmap[in].wire1==0 && itrmap[in].wire2==0) continue;
	item=&l2fr_BMBuffMap[nbuffentry];
	for(isl=0;isl<4;isl++){
	  item->spl[isl]=itrmap[in].spl[isl];
	}
	item->lay=itrmap[in].lay;
	item->sec=itrmap[in].sec;
	item->mod=0;
	item->modtype=0;
	if(!strncmp(supl->name,item->spl,4)&&
	   (ThePlane->lay==item->lay)&&
	   (ThePlane->sec==item->sec)){
	  item->link=itrmap[in].fedid;
	  item->chan1=itrmap[in].chan1;
	  item->chan2=itrmap[in].chan2;
	  item->wire1=itrmap[in].wire1;
	  item->wire2=itrmap[in].wire2;
	  item->cid=COMP_N_ITR;
	  item->chip_pos=itrmap[in].sharc;
	  item->vme_id=itrmap[in].vmeid;
	  item->vme_addr=itrmap[in].board;
	  nbuffentry++;
	  if(nbuffentry>=L2FR_BM_ENTRIES) {
	    sprintf(mess,
		    "L2FormArte/MakeBuffMap(ITR): not enough space in buffer map array\n");
	    grr_error(GRR_SFATAL,mess,1);
	    return 1;
	  }
	}
      }
    }
    /* The OTR section */
    if (strncmp(supl->name,"MC",2)==0 || strncmp(supl->name,"PC",2)==0 || 
	strncmp(supl->name,"TC",2)==0) {
      for(in=0;in<notrmap;in++){
	item=&l2fr_BMBuffMap[nbuffentry];
	for(isl=0;isl<4;isl++){
	  item->spl[isl]=otrmap[in].spl[isl];
	}
	item->lay=otrmap[in].lay;
	item->sec=otrmap[in].sec;
	item->mod=otrmap[in].mod;
	item->modtype=otrmap[in].moduletype;
	if(!strncmp(supl->name,item->spl,4)&&
	   (ThePlane->lay==item->lay)&&
	   (ThePlane->sec==item->sec)&&
	   (ThePlane->mod==item->mod)){
	  item->link=otrmap[in].fedid;
	  item->chan1=otrmap[in].chan1;
	  item->chan2=otrmap[in].chan2;
	  item->wire1=otrmap[in].wire1;
	  item->wire2=otrmap[in].wire2;
	  item->cid=COMP_N_OTR;
	  item->chip_pos= (otrmap[in].sharc_pos - 1) / 6 + 1;
	  item->vme_id=otrmap[in].vme_num;
	  item->vme_addr=otrmap[in].vme_addr;
	  nbuffentry++;
	  if(nbuffentry>=L2FR_BM_ENTRIES) {
	    sprintf(mess,
		    "L2FormArte/MakeBuffMap(OTR): not enough space in buffer map array\n");
	    grr_error(GRR_SFATAL,mess,1);
	    return 1;
	  }
	}
      }
    }
  }
  length_bm = nbuffentry*sizeof(L2FR_BM_Entry);
  *l2map = (int *) malloc(sizeof(L2FORMMAPHEAD)+length_bm);
  *length = sizeof(L2FORMMAPHEAD)+length_bm;
  sprintf(mess,"L2FormArte/MakeBuffMap: nbuffs: %d (length %d)\n",nbuffentry,*length);
  grr_error(GRR_SINFO,mess,0);
  if (l2map != NULL) {
    header = (L2FORMMAPHEAD *) *l2map;
    header->nbuffs = nbuffentry;
    wrPtr = (char*) (header + 1);
    memcpy((L2FR_BM_Entry*) wrPtr,&l2fr_BMBuffMap[0],sizeof(L2FR_BM_Entry)*nbuffentry);
    flag = 0;
  } else {
    sprintf(mess,"L2FormArte/MakeBuffMap: couldn't allocate space for buffmap table\n"); 
    grr_error(GRR_SFATAL,mess,1);
    flag = 1;
  }
  return flag;
}

/***
 *
 * Write_BuffMapAscii
 *
 * Print out the buffer map in Ascii file
 *
 ***/

static int Write_BuffMapAscii(){
  
  FILE *fdout;
  char fName[100];
  char mess[200];
  int flag = 1;
  int dump = 0;
  int i;
  
  for( i=1; i<nArg; i++ ){
    if( !strncmp( vArg[i], "-sltdumpdir:", 12 ) ){
      dump++;
      strcpy(fName, vArg[i]+12);
      sprintf(mess,"L2FormArte/Write_BuffMapAscii: dumping the L2Map ASCII file (l2in.map) to %s\n",fName);
      grr_error(GRR_SINFO,mess,0); 
    } 
  }
  if (dump == 0) {
    sprintf(mess,"L2FormArte/Write_BuffMapAscii: Couldn't read address of sltdumpdir directory\n");
    grr_error(GRR_SERROR,mess,0);
  } 
  strcat(fName,"l2in.map_");
  strcat(fName,Key);
  umask(0111);

  /*    open file for buffmap information: ---------------- */
  fdout = fopen(fName, "w");
  if( fdout == NULL) {
    sprintf(mess,"L2FormArte/Write_BuffMapAscii: unable to open %s\n", fName);
    grr_error(GRR_SERROR,mess,1);
    sprintf(mess,"L2FormArte/Write_BuffMapAscii: error %s\n",strerror(errno)); 
    grr_error(GRR_SERROR,mess,1);
    flag = 1;
  } else {
    L2form_BuffMapPrintTable(fdout);
    fclose(fdout);
    flag = 0;
  }
  return flag;
}

/***
 *
 * _PrintTable
 *
 * Print out the buffer map file
 *
 ***/

static void L2form_BuffMapPrintTable(FILE *fd)
{
  Item *item;
  int it;
  char mess[200];

  if( fd == NULL) {
    sprintf(mess,"L2form_BuffMapPrintTable: NULL file descriptor\n");
    grr_error(GRR_SERROR,mess,1);
  }
  
  for(it = 0; it < L2FR_BM_ENTRIES ; it++){
    item=&l2fr_BMBuffMap[it];
    if(item->lay>0&&item->cid!= 0xff)
      L2form_BuffMapPrintEntry(item, fd);
  }
}

static void L2form_BuffMapPrintEntry(L2FR_BM_Entry *entry, FILE *fd)
{
  char mess[200];

  if( fd == NULL) {
    sprintf(mess,"L2form_BuffMapPrintEntry: null file descriptor\n");
    grr_error(GRR_SERROR,mess,1);
  }

  fprintf(fd, "%c%c%c%c %4d %5d  %4d %4d %4d  %5d %4d %5d %x %3d %d %d %d\n",
	  entry->spl[0],entry->spl[1],entry->spl[2],entry->spl[3],entry->lay,entry->sec,
	  entry->mod,entry->modtype,entry->chan1,entry->chan2,entry->wire1,entry->wire2, 
	  entry->link,entry->chip_pos,entry->vme_id,entry->vme_addr,entry->cid);
}


/***
 *
 * Write_BuffMapBinary
 *
 * Print out the buffer map in Binary file
 *
 ***/

static int Write_BuffMapBinary(int *l2map, int size){
  
  FILE *fdout;
  char fName[100];
  char mess[200];
  int flag = 1;
  int dump = 0;
  int i;

  for( i=1; i<nArg; i++ ){
    if( !strncmp( vArg[i], "-sltdumpdir:", 12 ) ){
      dump++;
      strcpy(fName, vArg[i]+12);
      sprintf(mess,"L2FormArte/Write_BuffMapBinary: dumping the L2Map BINARY file (l2bin.map) to %s\n",fName);
      grr_error(GRR_SINFO,mess,0); 
    } 
  }
  if (dump == 0) {
    sprintf(mess,"L2FormArte/Write_BuffMapAscii: Couldn't read address of sltdumpdir directory\n");
    grr_error(GRR_SERROR,mess,0);
  } 

  strcat(fName,"l2bin.map_");
  strcat(fName,Key);
  umask(0111);

  /*    open file for buffmap information: ---------------- */
  fdout = fopen(fName, "wb");
  if( fdout == NULL) {
    sprintf(mess,"L2FormArte/Write_BuffMapBinary: unable to open %s\n", fName);
    grr_error(GRR_SERROR,mess,1);
    sprintf(mess,"L2FormArte/Write_BuffMapBinary: error %s\n",strerror(errno)); 
    grr_error(GRR_SERROR,mess,1);
    flag = 1;
  } else {
    fwrite(l2map, size, 1, fdout);
    fclose(fdout);
    flag = 0;
  }  
  return flag;
}
























