*-- Author :    T. Oest   24/10/96

      SUBROUTINE V0MUSC(IRTRA, ZLIMIT, NZMAX, NZ, ZMUSC, NDET, ANG)

************************************************************************
*
* Description: return a list for track IRTRA with all material
*              between the first measured point and ZLIMIT.
*              The list ZMUSC contains the z positions, ANG
*              the multiple scattering angle for 1 GeV.
*              Matter closer than ZMERGE are added in one position.
*
*
* Called from:
*
* External references:
*
* Argument in/out description
* IRTRA    in     track number in RTRA bank
* ZLIMIT   in     limit in z
* NZ       out    number of points where the track hits material
* ZMUSC    out    list with z positions
* ANG      out    (multiple scattering angle)**2 for a 1 GeV track
* NDET     out    number of sensitive layers
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/GEO.fi"
#include "auto/EV.fi"
#include "gean/ZGBPIP.fi"

      INTEGER          IRTRA, NZ, NZMAX
      DOUBLE PRECISION ZLIMIT, ZMUSC(*), ANG(*)
      INTEGER          NDET(*)

      INTEGER          IERROR, IRET
      DOUBLE PRECISION POUT(6), PTRK(6), RKD(6,6), PREC(4), FSTEP

      INTEGER          IGEDE, IWALL, I, ICOVAR
      REAL*4           ZMIN, ZMAX, MS, SIGT
      DOUBLE PRECISION ZMAT, MUSC, XOUT, YOUT, ZOUT, HELP
      REAL*4           COSX, SINX, XMAX, XMIN, YMAX, YMIN, XROT, YROT
      DOUBLE PRECISION DZ, MAGMIN, MAGMAX

      LOGICAL          DETECT, FIRST, MIXED

      SAVE FIRST

#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/EV.ff"

      DATA FIRST/.TRUE./
      DATA DZ/5./, MAGMIN/200.D0/, MAGMAX/700.D0/

*-----------------------------------------------------------------------
* initialization
*-----------------------------------------------------------------------
      NZ   = 0

      POUT(1)   = DBLE( FRTRA_ZF(IRTRA) )
      POUT(2)   = DBLE( FRTRA_XF(IRTRA) )
      POUT(3)   = DBLE( FRTRA_YF(IRTRA) )
      POUT(4)   = DBLE( FRTRA_TXF(IRTRA) )
      POUT(5)   = DBLE( FRTRA_TYF(IRTRA) )
      POUT(6)   = DBLE( FRTRA_PF(IRTRA) )
      DO I=1,6
       PTRK(I) = POUT(I)
      ENDDO
*-----------------------------------------------------------------------
* interaction with beam pipe??
*-----------------------------------------------------------------------


c      ZMAT = POUT(1)
c5     CONTINUE
c      ZMAT = ZMAT - 10.0D0
c      IF (ZMAT.GT.ZLIMIT) THEN

C track to new position       
c       IF ((ZMAT.LT.MAGMIN .AND. PTRK(1).LT.MAGMIN) .OR. 
c     +     (ZMAT.GT.MAGMAX .AND. PTRK(1).GT.MAGMAX) ) THEN
c        CALL V0LEXT(ZMAT, PTRK, POUT, RKD, IERROR)
c       ELSE
c        FSTEP = ZMAT - PTRK(1)
c        PREC(1)= SQRT(FRTRA_CVF( 1,IRTRA)) / ABS(ZMAT-PTRK(1))
c        PREC(2)= SQRT(FRTRA_CVF( 3,IRTRA)) / ABS(ZMAT-PTRK(1))
c        PREC(3)= SQRT(FRTRA_CVF( 6,IRTRA)) / ABS(ZMAT-PTRK(1))
c        PREC(4)= SQRT(FRTRA_CVF(10,IRTRA)) / ABS(ZMAT-PTRK(1))
c        CALL RKMAI2(0,ZMAT, FSTEP, PTRK, PREC, POUT, RKD, IERROR)
c       ENDIF

c       IRET = 0
c       I    = 0
c       DO WHILE (IRET.LE.0 .AND. I.LT.26)
c        I = I + 1
c        CALL V0HITP(I, PTRK, POUT, XOUT, YOUT, ZOUT, MUSC, IRET)
c       ENDDO

c       IF (IRET.EQ.1) THEN
c        NZ = NZ + 1
c        ZMUSC(NZ) = ZOUT
c        ANG(NZ)   = MUSC
c        NDET(NZ) = 0
c       ENDIF


c       DO I=1,6
c        PTRK(I) = POUT(I)
c       ENDDO


c       GOTO 5
c      ENDIF
*-----------------------------------------------------------------------
* loop over all detector planes and walls (increasing z)
* this assumes that the planes and walls are ordered in z
*-----------------------------------------------------------------------
       IGEDE = 1
       IWALL = 1

10    CONTINUE

      IF     (IWALL.GT.NGWAL()) THEN
       DETECT = .TRUE.
      ELSEIF (IGEDE.GT.NGEDE()) THEN
       DETECT = .FALSE.
      ELSEIF (FGWAL_ZMIN(IWALL).LT.FGEDE_ZMIN(IGEDE)) THEN
       DETECT = .FALSE.
      ELSE
       DETECT = .TRUE.
      ENDIF


      IF (DETECT) THEN
       ZMIN        = FGEDE_ZMIN(IGEDE)
       ZMAX        = FGEDE_ZMAX(IGEDE)
       COSX        = FGEDE_ROT(1,IGEDE)
       SINX        = FGEDE_ROT(2,IGEDE)
       XMAX        = FGEDE_XMAX(IGEDE)
       XMIN        = FGEDE_XMIN(IGEDE)
       YMAX        = FGEDE_YMAX(IGEDE)
       YMIN        = FGEDE_YMIN(IGEDE)
       MS          = FGEDE_MS(IGEDE)
       IGEDE       = IGEDE + 1
      ELSE
       ZMIN        = FGWAL_ZMIN(IWALL)
       ZMAX        = FGWAL_ZMAX(IWALL)
       COSX        = FGWAL_ROT(1,IWALL)
       SINX        = FGWAL_ROT(2,IWALL)
       XMAX        = FGWAL_XMAX(IWALL)
       XMIN        = FGWAL_XMIN(IWALL)
       YMAX        = FGWAL_YMAX(IWALL)
       YMIN        = FGWAL_YMIN(IWALL)
       MS          = FGWAL_MS(IWALL)
       IWALL       = IWALL + 1
      ENDIF
       
      IF (ZMIN.LE.ZLIMIT .OR. ZMIN.GT.FRTRA_ZF(IRTRA)) GOTO 20

      ZMAT = DBLE(MIN(0.5*(ZMIN+ZMAX),FRTRA_ZF(IRTRA)))

      MS   = MS * SQRT(1.+FRTRA_TXF(IRTRA)**2+FRTRA_TYF(IRTRA)**2)
c      SIGT = MS * FRTRA_PF(IRTRA)**2
c      print*,' ms ',sigt,FRTRA_CVF( 6,IRTRA) ,FRTRA_CVF(10,IRTRA) 
c      IF (SIGT.LT.16.*FRTRA_CVF( 6,IRTRA) .AND.
c     +    SIGT.LT.16.*FRTRA_CVF(10,IRTRA)) GOTO 20
*-----------------------------------------------------------------------
* track to the z position
*-----------------------------------------------------------------------


       IF (ABS(ZMAT-PTRK(1)) .LT. DZ .OR.
     +    (ZMAT.LT.MAGMIN .AND. PTRK(1).LT.MAGMIN) .OR. 
     +    (ZMAT.GT.MAGMAX .AND. PTRK(1).GT.MAGMAX) ) THEN
        CALL V0LEXT(ZMAT, PTRK, POUT, RKD, IERROR)
       ELSE
        FSTEP = ZMAT - PTRK(1)
        PREC(1)= SQRT(FRTRA_CVF( 1,IRTRA)) / ABS(ZMAT-PTRK(1))
        PREC(2)= SQRT(FRTRA_CVF( 3,IRTRA)) / ABS(ZMAT-PTRK(1))
        PREC(3)= SQRT(FRTRA_CVF( 6,IRTRA)) / ABS(ZMAT-PTRK(1))
        PREC(4)= SQRT(FRTRA_CVF(10,IRTRA)) / ABS(ZMAT-PTRK(1))
        ICOVAR = 0
        CALL RKMAI2(ICOVAR ,ZMAT, FSTEP, PTRK, PREC, POUT, RKD, IERROR)
       ENDIF

      DO I=1,6
       PTRK(I) = POUT(I)
      ENDDO

*-----------------------------------------------------------------------
* rotate x,y and check if material is hitten
*-----------------------------------------------------------------------

      XROT = SNGL(PTRK(2)) * COSX - SNGL(PTRK(3)) * SINX
      YROT = SNGL(PTRK(2)) * SINX + SNGL(PTRK(3)) * COSX

      IF (XROT.GE.XMIN .AND. XROT.LE.XMAX .AND.
     +    YROT.GE.YMIN .AND. YROT.LE.YMAX) THEN
*-----------------------------------------------------------------------
* add new material
*-----------------------------------------------------------------------
       IF (NZ.EQ.0 .OR. ABS(ZMAT-ZMUSC(MAX(NZ,1))).GT.DZ) THEN
        IF (NZ.LT.NZMAX) THEN
  
         NZ        = NZ + 1
         ZMUSC(NZ) = ZMAT
         ANG(NZ)   = DBLE(MS)
         IF (DETECT .AND. FRTRA_ZF(IRTRA).GT.ZMAX) THEN
          NDET(NZ) = 1
         ELSE
          NDET(NZ) = 0
         ENDIF

c        ELSE
c         PRINT*,' V0MUSC: NZ GREATER THAN NZMAX '
        ENDIF
       ELSE

        ZMUSC(NZ) = (ZMUSC(NZ)*ANG(NZ) + ZMAT*MS)/(ANG(NZ)+MS)
        ANG(NZ)   = ANG(NZ) + DBLE(MS)
        IF (DETECT .AND. FRTRA_ZF(IRTRA).GT.ZMAX) 
     +        NDET(NZ) = NDET(NZ) + 1

       ENDIF


      ENDIF




20    IF (IGEDE.LE.NGEDE() .OR. IWALL.LE.NGWAL() .AND.
     +    ZMIN.LE.FRTRA_ZF(IRTRA)) GOTO 10




*-----------------------------------------------------------------------
* now sort list
*-----------------------------------------------------------------------
      MIXED = .TRUE.
      DO WHILE(MIXED)

       MIXED = .FALSE.
       DO I=1,NZ-1
        IF (ZMUSC(I).GT.ZMUSC(I+1)) THEN
         MIXED      = .TRUE.
         HELP       = ZMUSC(I)
         ZMUSC(I)   = ZMUSC(I+1) 
         ZMUSC(I+1) = HELP
         HELP       = ANG(I)
         ANG(I)     = ANG(I+1) 
         ANG(I+1)   = HELP 
         IGEDE      = NDET(I)
         NDET(I)    = NDET(I+1) 
         NDET(I+1)  = IGEDE 
        ENDIF
       ENDDO

      ENDDO
*-----------------------------------------------------------------------
* the end
*-----------------------------------------------------------------------
 9000 END
