/*-------------------------------------------------------------------------
 *
 *  OTrAsdId header file
 * 
 *-----------------------------------------------------------------------*/
#ifndef __OTrAsdId_h
#define __OTrAsdId_h
 
#ifdef __cplusplus
extern "C" 
{
#endif /*  __cplusplus */

/************************************************************************
**
** Description: Simple routines for conversion from ASCII ASDid
**              to numbers and back.
**
** Author: A.Lanyov    September 99
** 
************************************************************************/
 
int asdid2num(const char *asdid,
              unsigned char *SL, unsigned char *lay, unsigned char *sec, 
	      unsigned char *ftb);
/***************************************************************************
  Converts ASDid in the form of "+t2-3-01-80"
  to integer numbers SL, lay, sec, asd which are used in OTrDecode package.
 ***************************************************************************/

int num2asdid(const unsigned char SL, const unsigned char lay, 
	      const unsigned char sec, const unsigned char asd,
              char asdid[12]);
/***************************************************************************
  Converts integer numbers sign, SL, lay, sec, asd
  which are used in OTrDecode package
  to ASDid in the form of "+t2-3-01-80".
 ***************************************************************************/

/*---------------------------------
  Definition of unsigned char codes
  for OTR superlayer halves

     SL  Superlayer name Side
     ==  =============== ==== 
      1  MC01            -x
      2  MC02            -x
      3  MC03            -x
      4  MC04            -x
      5  MC05            -x
      6  MC06            -x
      8  MC08            -x

      9  PC01            -x
     10  PC02            -x
     11  PC03            -x
     12  PC04            -x

     13  TC01            -x
     14  TC02            -x
     -----------------------
     17  MC01            +x
     18  MC02            +x
     19  MC03            +x
     20  MC04            +x
     21  MC05            +x
     22  MC06            +x
     24  MC08            +x

     25  PC01            +x
     26  PC02            +x
     27  PC03            +x
     28  PC04            +x

     29  TC01            +x
     30  TC02            +x
---------------------------------*/

#ifdef __cplusplus
}
#endif /*  __cplusplus */

#endif /*  __OTrAsdId_h */
