#ifndef __WAX_RTRELATION_HH__
#define __WAX_RTRELATION_HH__

#ifndef __SGI_STL_VECTOR
#include <vector>
#endif
#ifndef __IOSTREAM__
#include <iostream>
#endif
// #ifndef __OTR_DB_H__
// #include "otr_db.h"
// #endif

class waxParization ;
class TH1 ;
class otrcalRTProjection ;

class waxRTRelation
{
public:
  // typedefs, embedded classes
  enum EPitch { kPitchFive=0, kPitchTen=1 } ;
  enum EParType { kTofR, kRofT } ;
  typedef float Radius ;
  typedef float Time ;

  class RTResult {
  public:
    RTResult() : fT(0),fR(0),fdRdT(0),fSigmaR(0) {}
    Time   fT ;
    Radius fR ;
    float  fdRdT ;
    Radius fSigmaR ;
  } ;
  
  // constructors, destructor, assignment
  waxRTRelation() ;
  waxRTRelation(int key, istream& ifs) ;
  //waxRTRelation(LedaTableIterator<class OTR_r_t>&) ;
  waxRTRelation(int ledakey, EPitch pitch) ;
  waxRTRelation(const char* filename) ;
  ~waxRTRelation() ;

  // setters and getters
  Radius GetRMin() const { return fgRMin[fEPitch] ; }
  Radius GetRMax() const { return fgRMax[fEPitch] ; }
  Time   GetTMin() const { return fTMin ; }
  Time   GetTMax() const { return fTMax ; }
  
  int GetLedaKey() const { return fLedaKey ; }
  unsigned char GetEPitch() const { return fEPitch ; }
  unsigned char GetEParType() const { return kRofT ; }

  // other inline methods
  inline RTResult GetRTResultFromR(const Radius r) const ;
  inline RTResult GetRTResultFromT(const Time t) const ;
  inline void ProcessTime(Time t, Radius* r, Radius* s) const ; 
  int GetRTIndex() const { return fLedaKey ; }
  
  // other methods
  RTResult CalcRTResultFromR(const Radius r) const ;
  RTResult CalcRTResultFromT(const Time t) const ;

  // IO
  void Read(istream& ifs) ;
  void Write(int format, ostream&) const ;
  void Write(const char* filename) ;

//   void ReadDB() ;
//   void WriteDB() const ;

public:
  inline int GetBinFromR(const Radius r) const ;
  inline int GetBinFromT(const Time r) const ;
  static int GetNTableBins() { return fgNTableBins ; }
  void FillRTTable() ;

public: // stuff called from friend class rtcalRTProjection
  friend class otrcalRTProjection ; // the calibration is friend
  friend class otrcalRTHistoCalib ; // the calibration is friend
  waxParization* GetTParization() { return fTParization ; }
  waxParization* GetSParization() { return fSParization ; }
  void ChangeXBounds(Time tmin, Time tmax) ;
  Radius GetAverageSigmaR() const ;
private:
  // statics
  static const Radius fgRMin[2] ; // nominal minimum driftradius
  static const Radius fgRMax[2] ; // nominal maximum driftradius
  static const Time fgTMin ;      // nominal minimum drifttime
  static const Time fgTMax ;      // nominal maximum drifttime
  // others
  unsigned int fLedaKey ;         // key of this rt-relation in leda table
  unsigned int fEPitch ;          // layer pitch (5 or 10 mm)
  unsigned int fEParType ;        // type of parameterization (obsolete?)
  waxParization* fTParization ;   // parametrization of r(t) in cm  
  waxParization* fSParization ;   //         "          V(t) in cm^2
  Time fTMin ;                    // the minimum time
  Time fTMax ;                    // the maximum time
  
  // temporary data (tables for fast retrieval)
  static int fgNTableBins ; // number of bins in tables
  std::vector<RTResult> ftRTTable ;
  std::vector<int> ftMapTable ;
  Radius fRMin ;  // the minimum radius r(t=fTMax)
  Radius fRMax ;  // the maximum radius r(t=fTMin)
  //Time fDeltaT ;  // fgNTableBins/fTMax-fTMin
  //Radius fDeltaR ; // 
} ;

/*************** inlines ********************/

inline int waxRTRelation::GetBinFromR(const Radius r) const 
{
  int bin = int(fgNTableBins*(r-fRMin)/(fRMax-fRMin) + 0.5) ;
  if(bin>=fgNTableBins) bin=fgNTableBins-1 ;
  else if(bin<0) bin=0;
  return ftMapTable[bin] ;
}

inline int waxRTRelation::GetBinFromT(const Time t) const
{
  int bin = int(fgNTableBins*(t-fTMin)/(fTMax-fTMin) + 0.5) ;
  if(bin>=fgNTableBins) bin=fgNTableBins-1 ;
  else if(bin<0) bin=0;
  return bin ;
}

inline waxRTRelation::RTResult 
waxRTRelation::GetRTResultFromR(const Radius r) const 
{
  RTResult rt = ftRTTable[GetBinFromR(r)] ;
  rt.fT += (r-rt.fR)/rt.fdRdT ;
  rt.fR = r ; 
  return rt;
}

inline waxRTRelation::RTResult 
waxRTRelation::GetRTResultFromT(const Time t) const 
{
  RTResult rt = ftRTTable[GetBinFromT(t)] ;
  rt.fR += (t-rt.fT)*rt.fdRdT ;
  rt.fT = t ;
  return rt;
}

inline void
waxRTRelation::ProcessTime(float t, float* r, float* s) const
{
  RTResult rt = GetRTResultFromT(t) ;
  *r = rt.fR ;
  *s = rt.fSigmaR ;
}

#endif
